package org.apache.torque.test;

import java.sql.Blob;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.Types;
import java.io.IOException;
import java.io.InputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.Reader;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.torque.NoRowsException;
import org.apache.torque.TooManyRowsException;
import org.apache.torque.Torque;
import org.apache.torque.TorqueException;
import org.apache.torque.TorqueRuntimeException;
import org.apache.torque.criteria.Criteria;
import org.apache.torque.criteria.Criterion;
import org.apache.torque.om.mapper.RecordMapper;
import org.apache.torque.om.mapper.CompositeMapper;
import org.apache.torque.om.DateKey;
import org.apache.torque.om.NumberKey;
import org.apache.torque.om.StringKey;
import org.apache.torque.om.ObjectKey;
import org.apache.torque.om.SimpleKey;
import org.apache.torque.om.ComboKey;
import org.apache.torque.map.TableMap;
import org.apache.torque.util.BasePeer;
import org.apache.torque.util.Transaction;
import org.apache.torque.util.ColumnValues;
import org.apache.torque.util.JdbcTypedValue;



/**
 * A table
 *
 * The skeleton for this class was autogenerated by Torque on:
 *
 * [Thu Sep 13 04:35:45 CEST 2012]
 *
 * You should not use this class directly.  It should not even be
 * extended; all references should be to APeer
 */

public abstract class BaseAPeerImpl
    extends org.apache.torque.util.BasePeerImpl<A>
{
    /** The class log. */
    private static Log log = LogFactory.getLog(BaseAPeerImpl.class);

    /** Serial version */
    private static final long serialVersionUID = 1347503745262L;



    /**
     * Constructor.
     * The recordMapper, tableMap and databaseName fields are correctly
     * initialized.
     */
    public BaseAPeerImpl()
    {
        this(new ARecordMapper(),
            APeer.TABLE,
            APeer.DATABASE_NAME);
    }

    /**
     * Constructor providing the objects to be injected as parameters.
     *
     * @param recordMapper a record mapper to map JDBC result sets to objects
     * @param tableMap the default table map
     * @param databaseName the name of the database
     */
    public BaseAPeerImpl(
            RecordMapper<A> recordMapper, 
            TableMap tableMap,
            String databaseName)
    {
        super(recordMapper, tableMap, databaseName);
    }


    /**
     * Selects A objects from the database which have
     * the same content as the passed object.
     *
     * @return The list of selected objects, not null.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<A> doSelect(A obj)
            throws TorqueException
    {
        return doSelect(buildSelectCriteria(obj));
    }

    /**
     * Selects at most one A object from the database
     * which has the same content as the passed object.
     *
     * @return the selected Object, or null if no object was selected.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public A doSelectSingleRecord(
                A obj)
            throws TorqueException
    {
        List<A> aList = doSelect(obj);
        A a = null;
        if (aList.size() > 1)
        {
            throw new TooManyRowsException("Object " + obj 
                + " matched more than one record");
        }
        if (!aList.isEmpty())
        {
            a = aList.get(0);
        }
        return a;
    }

    /**
     * Returns a new instance of the Data object class
     */
    public A getDbObjectInstance()
    {
        return new A();
    }


    /**
     * Method to do inserts.
     *
     * @param columnValues the values to insert.
     *
     * @return the primary key of the inserted row.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public ObjectKey doInsert(ColumnValues columnValues) throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    APeer.DATABASE_NAME);
            ObjectKey result = doInsert(columnValues, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Method to do inserts.  This method is to be used during a transaction,
     * otherwise use the doInsert(Criteria) method.
     *
     * @param columnValues the values to insert.
     * @param con the connection to use, not null.
     *
     * @return the primary key of the inserted row.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public ObjectKey doInsert(ColumnValues columnValues, Connection con)
        throws TorqueException
    {
        correctBooleans(columnValues);
        return super.doInsert(columnValues, con);
    }

    /**
     * Method to do inserts
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public void doInsert(A obj) throws TorqueException
    {
        obj.setPrimaryKey(doInsert(buildColumnValues(obj)));
        obj.setNew(false);
        obj.setModified(false);
    }

    /**
     * Method to do inserts.  This method is to be used during a transaction,
     * otherwise use the doInsert(A) method.  It will take
     * care of the connection details internally.
     *
     * @param obj the data object to insert into the database.
     * @param con the connection to use
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public void doInsert(A obj, Connection con)
        throws TorqueException
    {
        ObjectKey primaryKey = doInsert(buildColumnValues(obj), con);
        if (primaryKey != null)
        {
            obj.setPrimaryKey(primaryKey);
        }
        obj.setNew(false);
        obj.setModified(false);
    }

    /**
     * Method to do updates.
     *
     * @param columnValues the values to update plus the primary key
     *        identifying the row to update.
     *
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(ColumnValues columnValues) throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    APeer.DATABASE_NAME);
            int result = doUpdate(columnValues, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Method to do updates.  This method is to be used during a transaction,
     * otherwise use the doUpdate(Criteria) method.
     *
     * @param columnValues the values to update plus the primary key
     *        identifying the row to update.
     * @param con the connection to use, not null.
     *
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(ColumnValues columnValues, Connection con)
        throws TorqueException
    {
        Criteria selectCriteria 
                = new Criteria(APeer.DATABASE_NAME);
        correctBooleans(columnValues);


    
        selectCriteria.where(
                APeer.A_ID,
                columnValues.remove(APeer.A_ID).getValue());

        return doUpdate(selectCriteria, columnValues, con);
    }

    /**
     * Updates an A in the database.
     * The primary key is used to identify the object to update.
     *
     * @param obj the data object to update in the database.
     *
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(A obj) throws TorqueException
    {
        ColumnValues columnValues = buildColumnValues(obj);
        int result = doUpdate(columnValues);
        obj.setModified(false);
        return result;
    }

    /**
     * Updates a A in the database.
     * The primary key is used to identify the object to update.
     * This method is to be used during a transaction,
     * otherwise the doUpdate(A) method can be used.
     *
     * @param obj the data object to update in the database.
     * @param con the connection to use, not null.
     
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(A obj, Connection con)
        throws TorqueException
    {
        ColumnValues columnValues = buildColumnValues(obj);
        int result = doUpdate(columnValues, con);
        obj.setModified(false);
        return result;
    }

    /**
     * Deletes a data object, i.e. a row in a table, in the database.
     *
     * @param obj the data object to delete in the database, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(A obj) throws TorqueException
    {
        int result = doDelete(buildCriteria(obj.getPrimaryKey()));
        obj.setDeleted(true);
        return result;
    }

    /**
     * Deletes a data object, i.e. a row in a table, in the database.
     * This method is to be used during a transaction, otherwise use the
     * doDelete(A) method.
     *
     * @param obj the data object to delete in the database, not null.
     * @param con the connection to use, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(A obj, Connection con)
        throws TorqueException
    {
        int result = doDelete(buildCriteria(obj.getPrimaryKey()), con);
        obj.setDeleted(true);
        return result;
    }

    /**
     * Deletes data objects, i.e. rows in a table, in the database.
     *
     * @param objects the data object to delete in the database, not null,
     *        may not contain null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(Collection<A> objects)
            throws TorqueException
    {
        int result = doDelete(buildPkCriteria(objects));
        for (A object : objects)
        {
            object.setDeleted(true);
        }
        return result;
    }

    /**
     * Deletes data objects, i.e. rows in a table, in the database.
     * This method uses the passed connection to delete the rows;
     * if a transaction is open in the connection, the deletion happens inside
     * this transaction.
     *
     * @param objects the data objects to delete in the database, not null,
     *        may not contain null.
     * @param con the connection to use for deleting, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(
            Collection<A> objects,
            Connection con)
        throws TorqueException
    {
        int result = doDelete(buildPkCriteria(objects), con);
        for (A object : objects)
        {
            object.setDeleted(true);
        }
        return result;
    }

    /**
     * Deletes a row in the database.
     *
     * @param pk the ObjectKey that identifies the row to delete.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(ObjectKey pk) throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    APeer.DATABASE_NAME);
            int deletedRows = doDelete(pk, connection);
            Transaction.commit(connection);
            connection = null;
            return deletedRows;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Deletes a row in the database.
     * This method is to be used during a transaction,
     * otherwise use the doDelete(ObjectKey) method.
     *
     * @param pk the ObjectKey that identifies the row to delete.
     * @param con the connection to use for deleting, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(ObjectKey pk, Connection con)
        throws TorqueException
    {
        return doDelete(buildCriteria(pk), con);
    }

    /** 
     * Build a Criteria object which selects all objects which have a given
     * primary key.
     *
     * @param pk the primary key value to build the criteria from, not null.
     */
    public Criteria buildCriteria(ObjectKey pk)
    {
        Criteria criteria = new Criteria();
        criteria.and(APeer.A_ID, pk);
        return criteria;
     }

    /** 
     * Build a Criteria object which selects all objects which primary keys
     * are contained in the passed collection.
     *
     * @param pks the primary key values to build the criteria from, not null,
     *        may not contain null.
     */
    public Criteria buildCriteria(Collection<ObjectKey> pks)
    {
        Criteria criteria = new Criteria();
        criteria.andIn(APeer.A_ID, pks);
        return criteria;
     }


    /** 
     * Build a Criteria object which selects all passed objects using their
     * primary key. Objects which do not yet have a primary key are ignored.
     *
     * @param objects the objects to build the criteria from, not null,
     *        may not contain null.
     */
    public Criteria buildPkCriteria(
            Collection<A> objects)
    {
        List<ObjectKey> pks = new ArrayList<ObjectKey>(objects.size());
        for (A object : objects)
        {
            ObjectKey pk = object.getPrimaryKey();
            if (pk != null)
            {
                pks.add(pk);
            }
        }
        return buildCriteria(pks);
    }

    /** 
     * Build a Criteria object from the data object for this peer.
     * The primary key columns are only added if the object is not new.
     *
     * @param obj the object to build the criteria from, not null.
     */
    public Criteria buildCriteria(A obj)
    {
        Criteria criteria = new Criteria(APeer.DATABASE_NAME);
        if (!obj.isNew())
        {
            criteria.and(APeer.A_ID, obj.getAId());
        }
        criteria.and(APeer.NAME, obj.getName());
        return criteria;
    }

    /** 
     * Build a Criteria object from the data object for this peer,
     * skipping all binary columns.
     *
     * @param obj the object to build the criteria from, not null.
     */
    public Criteria buildSelectCriteria(A obj)
    {
        Criteria criteria = new Criteria(APeer.DATABASE_NAME);
        if (!obj.isNew())
        {
            criteria.and(APeer.A_ID, obj.getAId());
        }
        criteria.and(APeer.NAME, obj.getName());
        return criteria;
    }

    /** 
     * Returns the contents of the object as ColumnValues object.
     * Primary key columns which are generated on insertion are not
     * added to the returned object if they still have their initial
     * value. Also, columns which have the useDatabaseDefaultValue
     * flag set to true are also not added to the returned object
     * if they still have their initial value.
     *
     * @throws TorqueException if the table map cannot be retrieved
     *         (should not happen).
     */
    public ColumnValues buildColumnValues(A a)
            throws TorqueException
    {
        ColumnValues columnValues = new ColumnValues();
        if (!a.isNew() 
            || a.getAId() != 0)
        {
            columnValues.put(
                    APeer.A_ID,
                    new JdbcTypedValue(
                        a.getAId(),
                        4));
        }
        columnValues.put(
                APeer.NAME,
                new JdbcTypedValue(
                    a.getName(),
                    12));
        return columnValues;
    }

    /**
     * Retrieve a single object by pk
     *
     * @param pk the primary key
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     * @throws NoRowsException Primary key was not found in database.
     * @throws TooManyRowsException Primary key was not found in database.
     */
    public A retrieveByPK(int pk)
        throws TorqueException, NoRowsException, TooManyRowsException
    {
        return retrieveByPK(SimpleKey.keyFor(pk));
    }

    /**
     * Retrieve a single object by pk
     *
     * @param pk the primary key
     * @param con the connection to use
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     * @throws NoRowsException Primary key was not found in database.
     * @throws TooManyRowsException Primary key was not found in database.
     */
    public A retrieveByPK(int pk, Connection con)
        throws TorqueException, NoRowsException, TooManyRowsException
    {
        return retrieveByPK(SimpleKey.keyFor(pk), con);
    }
    
    
    

    /**
     * Retrieve a single object by pk
     *
     * @param pk the primary key
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     * @throws NoRowsException Primary key was not found in database.
     * @throws TooManyRowsException Primary key was not found in database.
     */
    public A retrieveByPK(ObjectKey pk)
        throws TorqueException, NoRowsException, TooManyRowsException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(APeer.DATABASE_NAME);
            A result = retrieveByPK(pk, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Retrieve a single object by pk
     *
     * @param pk the primary key
     * @param con the connection to use
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     * @throws NoRowsException Primary key was not found in database.
     * @throws TooManyRowsException Primary key was not found in database.
     */
    public A retrieveByPK(ObjectKey pk, Connection con)
        throws TorqueException, NoRowsException, TooManyRowsException
    {
        Criteria criteria = buildCriteria(pk);
        List<A> v = doSelect(criteria, con);
        if (v.size() == 0)
        {
            throw new NoRowsException("Failed to select a row.");
        }
        else if (v.size() > 1)
        {
            throw new TooManyRowsException("Failed to select only one row.");
        }
        else
        {
            return (A)v.get(0);
        }
    }


    /**
     * Retrieve a multiple objects by pk
     *
     * @param pks List of primary keys
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<A> retrieveByPKs(Collection<ObjectKey> pks)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(APeer.DATABASE_NAME);
            List<A> result = retrieveByPKs(pks, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Retrieve multiple objects by pk
     *
     * @param pks List of primary keys
     * @param dbcon the connection to use
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<A> retrieveByPKs(
                Collection<ObjectKey> pks,
                Connection dbcon)
            throws TorqueException
    {
        if (pks == null || pks.size() == 0)
        {
            return new ArrayList<A>();
        }
        Criteria criteria = buildCriteria(pks);
        List<A> result = doSelect(criteria, dbcon);
        return result;
    }





    /** 
     * Fills the referenced RAb objects in the collRAbs fields
     * of the objects in <code>toFill</code> by reading the database.
     *
     * @param toFill Contains the objects to fill, not null,
     *        may not contain null.
     *
     * @return all read List<RAb> objects.
     *
     * @throws TorqueException if an error querying the database occurs.
     * @throws NullPointerException if toFill is null or contains null elements.
     */
    public List<RAb> fillRAbs(
            Collection<A> toFill)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    APeer.DATABASE_NAME);
            List<RAb> result = fillRAbs(toFill, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /** 
     * Fills the referenced RAb objects in the collRAbs fields
     * of the objects in <code>toFill</code> by reading the database.
     *
     * @param toFill Contains the objects to fill, not null,
     *        may not contain null.
     * @param connection the database connection to use, not null.
     *
     * @return all read List<RAb> objects.
     *
     * @throws TorqueException if an error querying the database occurs.
     * @throws NullPointerException if toFill is null or contains null elements.
     */
    public List<RAb> fillRAbs(
            Collection<A> toFill,
            Connection connection)
        throws TorqueException
    {
        Set<ObjectKey> localKeySet = new HashSet<ObjectKey>();
        for (A currentA : toFill)
        {
            ObjectKey localKey = currentA.getPrimaryKey();
            localKeySet.add(localKey);
        }

        int fillerChunkSize = getFillerChunkSize();
        Iterator<ObjectKey> localKeyIt = localKeySet.iterator();
        List<ObjectKey> localKeyListChunk
                = new ArrayList<ObjectKey>(fillerChunkSize);
        Map<ObjectKey, List<RAb>> fkToRAbMap
                = new HashMap<ObjectKey, List<RAb>>();
        while (localKeyIt.hasNext())
        {
            ObjectKey currentKey = localKeyIt.next();
            localKeyListChunk.add(currentKey);
            if (localKeyListChunk.size() < fillerChunkSize
                && localKeyIt.hasNext())
            {
                continue;
            }

            Criteria criteria = new Criteria();
            criteria.andIn(RAbPeer.A_ID, localKeyListChunk);
            List<RAb> referencedRAbList
                 = RAbPeer.doSelect(
                     criteria,
                     connection);
            for (RAb referencedObject : referencedRAbList)
            {
                ObjectKey foreignKey = referencedObject.getForeignKeyForA();
                List<RAb> objectsWithForeignKey 
                    = fkToRAbMap.get(foreignKey);
                if (objectsWithForeignKey == null)
                {
                    objectsWithForeignKey = new ArrayList<RAb>();
                    fkToRAbMap.put(
                        foreignKey,
                        objectsWithForeignKey);
                }
                objectsWithForeignKey.add(referencedObject);
            }
            localKeyListChunk.clear();
        }

        List<RAb> result = new ArrayList<RAb>();
        for (A objectToFill : toFill)
        {
            objectToFill.initRAbs();
            objectToFill.getRAbs().clear();
            ObjectKey localKey = objectToFill.getPrimaryKey();
            List<RAb> referencingList
                = fkToRAbMap.get(localKey);
            if (referencingList == null)
            {
                continue;
            }
            for (RAb referencing : referencingList)
            {
                // copy the referencing object for the case that more than one
                // toFill object references the same object; in this case
                // every toFillObject should have its own instance 
                // of the referencing object to remain consistent
                // with other Torque referencing object treatment
                RAb referencingCopy = referencing.copy(false);
                // copy does not set primary key
                referencingCopy.setPrimaryKey(referencing.getPrimaryKey());
                objectToFill.addRAb(referencingCopy);
                result.add(referencingCopy);
            }
        }
        return result;
    }
    /** 
     * Fills the referenced C objects in the collCs fields
     * of the objects in <code>toFill</code> by reading the database.
     *
     * @param toFill Contains the objects to fill, not null,
     *        may not contain null.
     *
     * @return all read List<C> objects.
     *
     * @throws TorqueException if an error querying the database occurs.
     * @throws NullPointerException if toFill is null or contains null elements.
     */
    public List<C> fillCs(
            Collection<A> toFill)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    APeer.DATABASE_NAME);
            List<C> result = fillCs(toFill, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /** 
     * Fills the referenced C objects in the collCs fields
     * of the objects in <code>toFill</code> by reading the database.
     *
     * @param toFill Contains the objects to fill, not null,
     *        may not contain null.
     * @param connection the database connection to use, not null.
     *
     * @return all read List<C> objects.
     *
     * @throws TorqueException if an error querying the database occurs.
     * @throws NullPointerException if toFill is null or contains null elements.
     */
    public List<C> fillCs(
            Collection<A> toFill,
            Connection connection)
        throws TorqueException
    {
        Set<ObjectKey> localKeySet = new HashSet<ObjectKey>();
        for (A currentA : toFill)
        {
            ObjectKey localKey = currentA.getPrimaryKey();
            localKeySet.add(localKey);
        }

        int fillerChunkSize = getFillerChunkSize();
        Iterator<ObjectKey> localKeyIt = localKeySet.iterator();
        List<ObjectKey> localKeyListChunk
                = new ArrayList<ObjectKey>(fillerChunkSize);
        Map<ObjectKey, List<C>> fkToCMap
                = new HashMap<ObjectKey, List<C>>();
        while (localKeyIt.hasNext())
        {
            ObjectKey currentKey = localKeyIt.next();
            localKeyListChunk.add(currentKey);
            if (localKeyListChunk.size() < fillerChunkSize
                && localKeyIt.hasNext())
            {
                continue;
            }

            Criteria criteria = new Criteria();
            criteria.andIn(CPeer.A_ID, localKeyListChunk);
            List<C> referencedCList
                 = CPeer.doSelect(
                     criteria,
                     connection);
            for (C referencedObject : referencedCList)
            {
                ObjectKey foreignKey = referencedObject.getForeignKeyForA();
                List<C> objectsWithForeignKey 
                    = fkToCMap.get(foreignKey);
                if (objectsWithForeignKey == null)
                {
                    objectsWithForeignKey = new ArrayList<C>();
                    fkToCMap.put(
                        foreignKey,
                        objectsWithForeignKey);
                }
                objectsWithForeignKey.add(referencedObject);
            }
            localKeyListChunk.clear();
        }

        List<C> result = new ArrayList<C>();
        for (A objectToFill : toFill)
        {
            objectToFill.initCs();
            objectToFill.getCs().clear();
            ObjectKey localKey = objectToFill.getPrimaryKey();
            List<C> referencingList
                = fkToCMap.get(localKey);
            if (referencingList == null)
            {
                continue;
            }
            for (C referencing : referencingList)
            {
                // copy the referencing object for the case that more than one
                // toFill object references the same object; in this case
                // every toFillObject should have its own instance 
                // of the referencing object to remain consistent
                // with other Torque referencing object treatment
                C referencingCopy = referencing.copy(false);
                // copy does not set primary key
                referencingCopy.setPrimaryKey(referencing.getPrimaryKey());
                objectToFill.addC(referencingCopy);
                result.add(referencingCopy);
            }
        }
        return result;
    }
    /** 
     * Fills the referenced D objects in the collDs fields
     * of the objects in <code>toFill</code> by reading the database.
     *
     * @param toFill Contains the objects to fill, not null,
     *        may not contain null.
     *
     * @return all read List<D> objects.
     *
     * @throws TorqueException if an error querying the database occurs.
     * @throws NullPointerException if toFill is null or contains null elements.
     */
    public List<D> fillDs(
            Collection<A> toFill)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    APeer.DATABASE_NAME);
            List<D> result = fillDs(toFill, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /** 
     * Fills the referenced D objects in the collDs fields
     * of the objects in <code>toFill</code> by reading the database.
     *
     * @param toFill Contains the objects to fill, not null,
     *        may not contain null.
     * @param connection the database connection to use, not null.
     *
     * @return all read List<D> objects.
     *
     * @throws TorqueException if an error querying the database occurs.
     * @throws NullPointerException if toFill is null or contains null elements.
     */
    public List<D> fillDs(
            Collection<A> toFill,
            Connection connection)
        throws TorqueException
    {
        Set<ObjectKey> localKeySet = new HashSet<ObjectKey>();
        for (A currentA : toFill)
        {
            ObjectKey localKey = currentA.getPrimaryKey();
            localKeySet.add(localKey);
        }

        int fillerChunkSize = getFillerChunkSize();
        Iterator<ObjectKey> localKeyIt = localKeySet.iterator();
        List<ObjectKey> localKeyListChunk
                = new ArrayList<ObjectKey>(fillerChunkSize);
        Map<ObjectKey, List<D>> fkToDMap
                = new HashMap<ObjectKey, List<D>>();
        while (localKeyIt.hasNext())
        {
            ObjectKey currentKey = localKeyIt.next();
            localKeyListChunk.add(currentKey);
            if (localKeyListChunk.size() < fillerChunkSize
                && localKeyIt.hasNext())
            {
                continue;
            }

            Criteria criteria = new Criteria();
            criteria.andIn(DPeer.A_ID, localKeyListChunk);
            List<D> referencedDList
                 = DPeer.doSelect(
                     criteria,
                     connection);
            for (D referencedObject : referencedDList)
            {
                ObjectKey foreignKey = referencedObject.getForeignKeyForA();
                List<D> objectsWithForeignKey 
                    = fkToDMap.get(foreignKey);
                if (objectsWithForeignKey == null)
                {
                    objectsWithForeignKey = new ArrayList<D>();
                    fkToDMap.put(
                        foreignKey,
                        objectsWithForeignKey);
                }
                objectsWithForeignKey.add(referencedObject);
            }
            localKeyListChunk.clear();
        }

        List<D> result = new ArrayList<D>();
        for (A objectToFill : toFill)
        {
            objectToFill.initDs();
            objectToFill.getDs().clear();
            ObjectKey localKey = objectToFill.getPrimaryKey();
            List<D> referencingList
                = fkToDMap.get(localKey);
            if (referencingList == null)
            {
                continue;
            }
            for (D referencing : referencingList)
            {
                // copy the referencing object for the case that more than one
                // toFill object references the same object; in this case
                // every toFillObject should have its own instance 
                // of the referencing object to remain consistent
                // with other Torque referencing object treatment
                D referencingCopy = referencing.copy(false);
                // copy does not set primary key
                referencingCopy.setPrimaryKey(referencing.getPrimaryKey());
                objectToFill.addD(referencingCopy);
                result.add(referencingCopy);
            }
        }
        return result;
    }

    /** 
     * Returns the chunk size for the filler methods.
     *
     * @return the chunk size for the filler methods.
     */
    protected int getFillerChunkSize()
    {
        return 999;
    }


}
