package org.apache.torque.test;

import java.sql.Blob;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.Types;
import java.io.IOException;
import java.io.InputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.Reader;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.torque.NoRowsException;
import org.apache.torque.TooManyRowsException;
import org.apache.torque.Torque;
import org.apache.torque.TorqueException;
import org.apache.torque.TorqueRuntimeException;
import org.apache.torque.criteria.Criteria;
import org.apache.torque.criteria.Criterion;
import org.apache.torque.om.mapper.RecordMapper;
import org.apache.torque.om.mapper.CompositeMapper;
import org.apache.torque.om.DateKey;
import org.apache.torque.om.NumberKey;
import org.apache.torque.om.StringKey;
import org.apache.torque.om.ObjectKey;
import org.apache.torque.om.SimpleKey;
import org.apache.torque.om.ComboKey;
import org.apache.torque.map.TableMap;
import org.apache.torque.util.BasePeer;
import org.apache.torque.util.Transaction;
import org.apache.torque.util.ColumnValues;
import org.apache.torque.util.JdbcTypedValue;



/**
 * c table contains fk to ar_ab, a and b
 *
 * The skeleton for this class was autogenerated by Torque on:
 *
 * [Thu Sep 13 04:35:45 CEST 2012]
 *
 * You should not use this class directly.  It should not even be
 * extended; all references should be to CPeer
 */

public abstract class BaseCPeerImpl
    extends org.apache.torque.util.BasePeerImpl<C>
{
    /** The class log. */
    private static Log log = LogFactory.getLog(BaseCPeerImpl.class);

    /** Serial version */
    private static final long serialVersionUID = 1347503745332L;



    /**
     * Constructor.
     * The recordMapper, tableMap and databaseName fields are correctly
     * initialized.
     */
    public BaseCPeerImpl()
    {
        this(new CRecordMapper(),
            CPeer.TABLE,
            CPeer.DATABASE_NAME);
    }

    /**
     * Constructor providing the objects to be injected as parameters.
     *
     * @param recordMapper a record mapper to map JDBC result sets to objects
     * @param tableMap the default table map
     * @param databaseName the name of the database
     */
    public BaseCPeerImpl(
            RecordMapper<C> recordMapper, 
            TableMap tableMap,
            String databaseName)
    {
        super(recordMapper, tableMap, databaseName);
    }


    /**
     * Selects C objects from the database which have
     * the same content as the passed object.
     *
     * @return The list of selected objects, not null.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<C> doSelect(C obj)
            throws TorqueException
    {
        return doSelect(buildSelectCriteria(obj));
    }

    /**
     * Selects at most one C object from the database
     * which has the same content as the passed object.
     *
     * @return the selected Object, or null if no object was selected.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public C doSelectSingleRecord(
                C obj)
            throws TorqueException
    {
        List<C> cList = doSelect(obj);
        C c = null;
        if (cList.size() > 1)
        {
            throw new TooManyRowsException("Object " + obj 
                + " matched more than one record");
        }
        if (!cList.isEmpty())
        {
            c = cList.get(0);
        }
        return c;
    }

    /**
     * Returns a new instance of the Data object class
     */
    public C getDbObjectInstance()
    {
        return new C();
    }


    /**
     * Method to do inserts.
     *
     * @param columnValues the values to insert.
     *
     * @return the primary key of the inserted row.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public ObjectKey doInsert(ColumnValues columnValues) throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    CPeer.DATABASE_NAME);
            ObjectKey result = doInsert(columnValues, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Method to do inserts.  This method is to be used during a transaction,
     * otherwise use the doInsert(Criteria) method.
     *
     * @param columnValues the values to insert.
     * @param con the connection to use, not null.
     *
     * @return the primary key of the inserted row.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public ObjectKey doInsert(ColumnValues columnValues, Connection con)
        throws TorqueException
    {
        correctBooleans(columnValues);
        return super.doInsert(columnValues, con);
    }

    /**
     * Method to do inserts
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public void doInsert(C obj) throws TorqueException
    {
        obj.setPrimaryKey(doInsert(buildColumnValues(obj)));
        obj.setNew(false);
        obj.setModified(false);
    }

    /**
     * Method to do inserts.  This method is to be used during a transaction,
     * otherwise use the doInsert(C) method.  It will take
     * care of the connection details internally.
     *
     * @param obj the data object to insert into the database.
     * @param con the connection to use
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public void doInsert(C obj, Connection con)
        throws TorqueException
    {
        ObjectKey primaryKey = doInsert(buildColumnValues(obj), con);
        if (primaryKey != null)
        {
            obj.setPrimaryKey(primaryKey);
        }
        obj.setNew(false);
        obj.setModified(false);
    }

    /**
     * Method to do updates.
     *
     * @param columnValues the values to update plus the primary key
     *        identifying the row to update.
     *
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(ColumnValues columnValues) throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    CPeer.DATABASE_NAME);
            int result = doUpdate(columnValues, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Method to do updates.  This method is to be used during a transaction,
     * otherwise use the doUpdate(Criteria) method.
     *
     * @param columnValues the values to update plus the primary key
     *        identifying the row to update.
     * @param con the connection to use, not null.
     *
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(ColumnValues columnValues, Connection con)
        throws TorqueException
    {
        Criteria selectCriteria 
                = new Criteria(CPeer.DATABASE_NAME);
        correctBooleans(columnValues);


    
        selectCriteria.where(
                CPeer.C_ID,
                columnValues.remove(CPeer.C_ID).getValue());



        return doUpdate(selectCriteria, columnValues, con);
    }

    /**
     * Updates an C in the database.
     * The primary key is used to identify the object to update.
     *
     * @param obj the data object to update in the database.
     *
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(C obj) throws TorqueException
    {
        ColumnValues columnValues = buildColumnValues(obj);
        int result = doUpdate(columnValues);
        obj.setModified(false);
        return result;
    }

    /**
     * Updates a C in the database.
     * The primary key is used to identify the object to update.
     * This method is to be used during a transaction,
     * otherwise the doUpdate(C) method can be used.
     *
     * @param obj the data object to update in the database.
     * @param con the connection to use, not null.
     
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(C obj, Connection con)
        throws TorqueException
    {
        ColumnValues columnValues = buildColumnValues(obj);
        int result = doUpdate(columnValues, con);
        obj.setModified(false);
        return result;
    }

    /**
     * Deletes a data object, i.e. a row in a table, in the database.
     *
     * @param obj the data object to delete in the database, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(C obj) throws TorqueException
    {
        int result = doDelete(buildCriteria(obj.getPrimaryKey()));
        obj.setDeleted(true);
        return result;
    }

    /**
     * Deletes a data object, i.e. a row in a table, in the database.
     * This method is to be used during a transaction, otherwise use the
     * doDelete(C) method.
     *
     * @param obj the data object to delete in the database, not null.
     * @param con the connection to use, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(C obj, Connection con)
        throws TorqueException
    {
        int result = doDelete(buildCriteria(obj.getPrimaryKey()), con);
        obj.setDeleted(true);
        return result;
    }

    /**
     * Deletes data objects, i.e. rows in a table, in the database.
     *
     * @param objects the data object to delete in the database, not null,
     *        may not contain null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(Collection<C> objects)
            throws TorqueException
    {
        int result = doDelete(buildPkCriteria(objects));
        for (C object : objects)
        {
            object.setDeleted(true);
        }
        return result;
    }

    /**
     * Deletes data objects, i.e. rows in a table, in the database.
     * This method uses the passed connection to delete the rows;
     * if a transaction is open in the connection, the deletion happens inside
     * this transaction.
     *
     * @param objects the data objects to delete in the database, not null,
     *        may not contain null.
     * @param con the connection to use for deleting, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(
            Collection<C> objects,
            Connection con)
        throws TorqueException
    {
        int result = doDelete(buildPkCriteria(objects), con);
        for (C object : objects)
        {
            object.setDeleted(true);
        }
        return result;
    }

    /**
     * Deletes a row in the database.
     *
     * @param pk the ObjectKey that identifies the row to delete.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(ObjectKey pk) throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    CPeer.DATABASE_NAME);
            int deletedRows = doDelete(pk, connection);
            Transaction.commit(connection);
            connection = null;
            return deletedRows;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Deletes a row in the database.
     * This method is to be used during a transaction,
     * otherwise use the doDelete(ObjectKey) method.
     *
     * @param pk the ObjectKey that identifies the row to delete.
     * @param con the connection to use for deleting, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(ObjectKey pk, Connection con)
        throws TorqueException
    {
        return doDelete(buildCriteria(pk), con);
    }

    /** 
     * Build a Criteria object which selects all objects which have a given
     * primary key.
     *
     * @param pk the primary key value to build the criteria from, not null.
     */
    public Criteria buildCriteria(ObjectKey pk)
    {
        Criteria criteria = new Criteria();
        criteria.and(CPeer.C_ID, pk);
        return criteria;
     }

    /** 
     * Build a Criteria object which selects all objects which primary keys
     * are contained in the passed collection.
     *
     * @param pks the primary key values to build the criteria from, not null,
     *        may not contain null.
     */
    public Criteria buildCriteria(Collection<ObjectKey> pks)
    {
        Criteria criteria = new Criteria();
        criteria.andIn(CPeer.C_ID, pks);
        return criteria;
     }


    /** 
     * Build a Criteria object which selects all passed objects using their
     * primary key. Objects which do not yet have a primary key are ignored.
     *
     * @param objects the objects to build the criteria from, not null,
     *        may not contain null.
     */
    public Criteria buildPkCriteria(
            Collection<C> objects)
    {
        List<ObjectKey> pks = new ArrayList<ObjectKey>(objects.size());
        for (C object : objects)
        {
            ObjectKey pk = object.getPrimaryKey();
            if (pk != null)
            {
                pks.add(pk);
            }
        }
        return buildCriteria(pks);
    }

    /** 
     * Build a Criteria object from the data object for this peer.
     * The primary key columns are only added if the object is not new.
     *
     * @param obj the object to build the criteria from, not null.
     */
    public Criteria buildCriteria(C obj)
    {
        Criteria criteria = new Criteria(CPeer.DATABASE_NAME);
        if (!obj.isNew())
        {
            criteria.and(CPeer.C_ID, obj.getCId());
        }
        criteria.and(CPeer.A_ID, obj.getAId());
        criteria.and(CPeer.B_ID, obj.getBId());
        criteria.and(CPeer.NAME, obj.getName());
        return criteria;
    }

    /** 
     * Build a Criteria object from the data object for this peer,
     * skipping all binary columns.
     *
     * @param obj the object to build the criteria from, not null.
     */
    public Criteria buildSelectCriteria(C obj)
    {
        Criteria criteria = new Criteria(CPeer.DATABASE_NAME);
        if (!obj.isNew())
        {
            criteria.and(CPeer.C_ID, obj.getCId());
        }
        criteria.and(CPeer.A_ID, obj.getAId());
        criteria.and(CPeer.B_ID, obj.getBId());
        criteria.and(CPeer.NAME, obj.getName());
        return criteria;
    }

    /** 
     * Returns the contents of the object as ColumnValues object.
     * Primary key columns which are generated on insertion are not
     * added to the returned object if they still have their initial
     * value. Also, columns which have the useDatabaseDefaultValue
     * flag set to true are also not added to the returned object
     * if they still have their initial value.
     *
     * @throws TorqueException if the table map cannot be retrieved
     *         (should not happen).
     */
    public ColumnValues buildColumnValues(C c)
            throws TorqueException
    {
        ColumnValues columnValues = new ColumnValues();
        if (!c.isNew() 
            || c.getCId() != 0)
        {
            columnValues.put(
                    CPeer.C_ID,
                    new JdbcTypedValue(
                        c.getCId(),
                        4));
        }
        columnValues.put(
                CPeer.A_ID,
                new JdbcTypedValue(
                    c.getAId(),
                    4));
        columnValues.put(
                CPeer.B_ID,
                new JdbcTypedValue(
                    c.getBId(),
                    4));
        columnValues.put(
                CPeer.NAME,
                new JdbcTypedValue(
                    c.getName(),
                    12));
        return columnValues;
    }

    /**
     * Retrieve a single object by pk
     *
     * @param pk the primary key
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     * @throws NoRowsException Primary key was not found in database.
     * @throws TooManyRowsException Primary key was not found in database.
     */
    public C retrieveByPK(int pk)
        throws TorqueException, NoRowsException, TooManyRowsException
    {
        return retrieveByPK(SimpleKey.keyFor(pk));
    }

    /**
     * Retrieve a single object by pk
     *
     * @param pk the primary key
     * @param con the connection to use
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     * @throws NoRowsException Primary key was not found in database.
     * @throws TooManyRowsException Primary key was not found in database.
     */
    public C retrieveByPK(int pk, Connection con)
        throws TorqueException, NoRowsException, TooManyRowsException
    {
        return retrieveByPK(SimpleKey.keyFor(pk), con);
    }
    
    
    

    /**
     * Retrieve a single object by pk
     *
     * @param pk the primary key
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     * @throws NoRowsException Primary key was not found in database.
     * @throws TooManyRowsException Primary key was not found in database.
     */
    public C retrieveByPK(ObjectKey pk)
        throws TorqueException, NoRowsException, TooManyRowsException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(CPeer.DATABASE_NAME);
            C result = retrieveByPK(pk, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Retrieve a single object by pk
     *
     * @param pk the primary key
     * @param con the connection to use
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     * @throws NoRowsException Primary key was not found in database.
     * @throws TooManyRowsException Primary key was not found in database.
     */
    public C retrieveByPK(ObjectKey pk, Connection con)
        throws TorqueException, NoRowsException, TooManyRowsException
    {
        Criteria criteria = buildCriteria(pk);
        List<C> v = doSelect(criteria, con);
        if (v.size() == 0)
        {
            throw new NoRowsException("Failed to select a row.");
        }
        else if (v.size() > 1)
        {
            throw new TooManyRowsException("Failed to select only one row.");
        }
        else
        {
            return (C)v.get(0);
        }
    }


    /**
     * Retrieve a multiple objects by pk
     *
     * @param pks List of primary keys
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<C> retrieveByPKs(Collection<ObjectKey> pks)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(CPeer.DATABASE_NAME);
            List<C> result = retrieveByPKs(pks, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Retrieve multiple objects by pk
     *
     * @param pks List of primary keys
     * @param dbcon the connection to use
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<C> retrieveByPKs(
                Collection<ObjectKey> pks,
                Connection dbcon)
            throws TorqueException
    {
        if (pks == null || pks.size() == 0)
        {
            return new ArrayList<C>();
        }
        Criteria criteria = buildCriteria(pks);
        List<C> result = doSelect(criteria, dbcon);
        return result;
    }


    /**
     * Selects a collection of dbObjectClassName objects pre-filled with their
     * RAb objects.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<C> doSelectJoinRAb(Criteria criteria)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    CPeer.DATABASE_NAME);
            List<C> result
                    = CPeer.doSelectJoinRAb(criteria, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Selects a collection of C objects pre-filled with their
     * RAb objects.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<C> doSelectJoinRAb(
            Criteria criteria, Connection conn)
        throws TorqueException
    {
        setDbName(criteria);

        addSelectColumns(criteria);
        org.apache.torque.test.RAbPeer.addSelectColumns(criteria);

        CompositeMapper compositeMapper = new CompositeMapper();
        compositeMapper.addMapper(
                new CRecordMapper(),
                0);
        compositeMapper.addMapper(
                new RAbRecordMapper(),
                CPeer.numColumns);
        criteria.addJoin(CPeer.A_ID,
            org.apache.torque.test.RAbPeer.A_ID);
        criteria.addJoin(CPeer.B_ID,
            org.apache.torque.test.RAbPeer.B_ID);

        correctBooleans(criteria);

        List<C> result = new ArrayList<C>();
        List<List<Object>> rawResult = doSelect(
                criteria, compositeMapper, conn);
        for (List<Object> rawResultRow : rawResult)
        {
            C obj1 = (C) rawResultRow.get(0);
            RAb obj2 = (RAb) rawResultRow.get(1);

            boolean newObject = true;
            for (int j = 0; j < result.size(); j++)
            {
                C temp_obj1 = result.get(j);
                org.apache.torque.test.RAb temp_obj2 = temp_obj1.getRAb();
                if (temp_obj2.getPrimaryKey().equals(obj2.getPrimaryKey()))
                {
                    newObject = false;
                    temp_obj2.addC(obj1);
                    break;
                }
            }
            if (newObject)
            {
                obj2.initCs();
                obj2.addC(obj1);
            }
            result.add(obj1);
        }
        return result;
    }
    /**
     * Selects a collection of dbObjectClassName objects pre-filled with their
     * A objects.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<C> doSelectJoinA(Criteria criteria)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    CPeer.DATABASE_NAME);
            List<C> result
                    = CPeer.doSelectJoinA(criteria, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Selects a collection of C objects pre-filled with their
     * A objects.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<C> doSelectJoinA(
            Criteria criteria, Connection conn)
        throws TorqueException
    {
        setDbName(criteria);

        addSelectColumns(criteria);
        org.apache.torque.test.APeer.addSelectColumns(criteria);

        CompositeMapper compositeMapper = new CompositeMapper();
        compositeMapper.addMapper(
                new CRecordMapper(),
                0);
        compositeMapper.addMapper(
                new ARecordMapper(),
                CPeer.numColumns);
        criteria.addJoin(CPeer.A_ID,
            org.apache.torque.test.APeer.A_ID);

        correctBooleans(criteria);

        List<C> result = new ArrayList<C>();
        List<List<Object>> rawResult = doSelect(
                criteria, compositeMapper, conn);
        for (List<Object> rawResultRow : rawResult)
        {
            C obj1 = (C) rawResultRow.get(0);
            A obj2 = (A) rawResultRow.get(1);

            boolean newObject = true;
            for (int j = 0; j < result.size(); j++)
            {
                C temp_obj1 = result.get(j);
                org.apache.torque.test.A temp_obj2 = temp_obj1.getA();
                if (temp_obj2.getPrimaryKey().equals(obj2.getPrimaryKey()))
                {
                    newObject = false;
                    temp_obj2.addC(obj1);
                    break;
                }
            }
            if (newObject)
            {
                obj2.initCs();
                obj2.addC(obj1);
            }
            result.add(obj1);
        }
        return result;
    }
    /**
     * Selects a collection of dbObjectClassName objects pre-filled with their
     * B objects.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<C> doSelectJoinB(Criteria criteria)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    CPeer.DATABASE_NAME);
            List<C> result
                    = CPeer.doSelectJoinB(criteria, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Selects a collection of C objects pre-filled with their
     * B objects.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<C> doSelectJoinB(
            Criteria criteria, Connection conn)
        throws TorqueException
    {
        setDbName(criteria);

        addSelectColumns(criteria);
        org.apache.torque.test.BPeer.addSelectColumns(criteria);

        CompositeMapper compositeMapper = new CompositeMapper();
        compositeMapper.addMapper(
                new CRecordMapper(),
                0);
        compositeMapper.addMapper(
                new BRecordMapper(),
                CPeer.numColumns);
        criteria.addJoin(CPeer.B_ID,
            org.apache.torque.test.BPeer.B_ID);

        correctBooleans(criteria);

        List<C> result = new ArrayList<C>();
        List<List<Object>> rawResult = doSelect(
                criteria, compositeMapper, conn);
        for (List<Object> rawResultRow : rawResult)
        {
            C obj1 = (C) rawResultRow.get(0);
            B obj2 = (B) rawResultRow.get(1);

            boolean newObject = true;
            for (int j = 0; j < result.size(); j++)
            {
                C temp_obj1 = result.get(j);
                org.apache.torque.test.B temp_obj2 = temp_obj1.getB();
                if (temp_obj2.getPrimaryKey().equals(obj2.getPrimaryKey()))
                {
                    newObject = false;
                    temp_obj2.addC(obj1);
                    break;
                }
            }
            if (newObject)
            {
                obj2.initCs();
                obj2.addC(obj1);
            }
            result.add(obj1);
        }
        return result;
    }

    /**
     * selects a collection of C objects pre-filled with
     * all related objects.
     *
     * This method is protected by default in order to keep the public
     * api reasonable.  You can provide public methods for those you
     * actually need in CPeerImpl.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    protected List<C> doSelectJoinAllExceptRAb(Criteria criteria)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    CPeer.DATABASE_NAME);
            List<C> result
                    = CPeer.doSelectJoinAllExceptRAb(criteria, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * selects a collection of C objects pre-filled with
     * related objects.
     *
     * This method is protected by default in order to keep the public
     * api reasonable.  You can provide public methods for those you
     * actually need in CPeerImpl.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    protected List<C> doSelectJoinAllExceptRAb(Criteria criteria, Connection conn)
        throws TorqueException
    {
        setDbName(criteria);

        CompositeMapper compositeMapper = new CompositeMapper();
        compositeMapper.addMapper(new CRecordMapper(), 0);
        CPeer.addSelectColumns(criteria);
        int offset = CPeer.numColumns;
 
        org.apache.torque.test.APeer.addSelectColumns(criteria);
        criteria.addJoin(CPeer.A_ID, org.apache.torque.test.APeer.A_ID);
        compositeMapper.addMapper(new ARecordMapper(), offset);
        offset = offset + org.apache.torque.test.APeer.numColumns;
 
        org.apache.torque.test.BPeer.addSelectColumns(criteria);
        criteria.addJoin(CPeer.B_ID, org.apache.torque.test.BPeer.B_ID);
        compositeMapper.addMapper(new BRecordMapper(), offset);
        offset = offset + org.apache.torque.test.BPeer.numColumns;

        correctBooleans(criteria);

        List<C> result = new ArrayList<C>();
        List<List<Object>> rawResult = doSelect(
                criteria, compositeMapper, conn);
        for (List<Object> rawResultRow : rawResult)
        {
            C c = (C) rawResultRow.get(0);

            org.apache.torque.test.A obj1 = (org.apache.torque.test.A) rawResultRow.get(1);
            boolean newObject = true;
            for (int j = 0; j < result.size(); j++)
            {
                org.apache.torque.test.C temp_c = result.get(j);
                org.apache.torque.test.A temp_obj1 = temp_c.getA();
                if (temp_obj1.getPrimaryKey().equals(obj1.getPrimaryKey()))
                {
                    newObject = false;
                    temp_obj1.addC(c);
                    break;
                }
            }
            if (newObject)
            {
                obj1.initCs();
                obj1.addC(c);
            }

            org.apache.torque.test.B obj2 = (org.apache.torque.test.B) rawResultRow.get(2);
            newObject = true;
            for (int j = 0; j < result.size(); j++)
            {
                org.apache.torque.test.C temp_c = result.get(j);
                org.apache.torque.test.B temp_obj2 = temp_c.getB();
                if (temp_obj2.getPrimaryKey().equals(obj2.getPrimaryKey()))
                {
                    newObject = false;
                    temp_obj2.addC(c);
                    break;
                }
            }
            if (newObject)
            {
                obj2.initCs();
                obj2.addC(c);
            }
            result.add(c);
        }
        return result;
    }
    /**
     * selects a collection of C objects pre-filled with
     * all related objects.
     *
     * This method is protected by default in order to keep the public
     * api reasonable.  You can provide public methods for those you
     * actually need in CPeerImpl.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    protected List<C> doSelectJoinAllExceptA(Criteria criteria)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    CPeer.DATABASE_NAME);
            List<C> result
                    = CPeer.doSelectJoinAllExceptA(criteria, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * selects a collection of C objects pre-filled with
     * related objects.
     *
     * This method is protected by default in order to keep the public
     * api reasonable.  You can provide public methods for those you
     * actually need in CPeerImpl.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    protected List<C> doSelectJoinAllExceptA(Criteria criteria, Connection conn)
        throws TorqueException
    {
        setDbName(criteria);

        CompositeMapper compositeMapper = new CompositeMapper();
        compositeMapper.addMapper(new CRecordMapper(), 0);
        CPeer.addSelectColumns(criteria);
        int offset = CPeer.numColumns;
 
        org.apache.torque.test.RAbPeer.addSelectColumns(criteria);
        criteria.addJoin(CPeer.A_ID, org.apache.torque.test.RAbPeer.A_ID);
        criteria.addJoin(CPeer.B_ID, org.apache.torque.test.RAbPeer.B_ID);
        compositeMapper.addMapper(new RAbRecordMapper(), offset);
        offset = offset + org.apache.torque.test.RAbPeer.numColumns;
 
        org.apache.torque.test.BPeer.addSelectColumns(criteria);
        criteria.addJoin(CPeer.B_ID, org.apache.torque.test.BPeer.B_ID);
        compositeMapper.addMapper(new BRecordMapper(), offset);
        offset = offset + org.apache.torque.test.BPeer.numColumns;

        correctBooleans(criteria);

        List<C> result = new ArrayList<C>();
        List<List<Object>> rawResult = doSelect(
                criteria, compositeMapper, conn);
        for (List<Object> rawResultRow : rawResult)
        {
            C c = (C) rawResultRow.get(0);

            org.apache.torque.test.RAb obj1 = (org.apache.torque.test.RAb) rawResultRow.get(1);
            boolean newObject = true;
            for (int j = 0; j < result.size(); j++)
            {
                org.apache.torque.test.C temp_c = result.get(j);
                org.apache.torque.test.RAb temp_obj1 = temp_c.getRAb();
                if (temp_obj1.getPrimaryKey().equals(obj1.getPrimaryKey()))
                {
                    newObject = false;
                    temp_obj1.addC(c);
                    break;
                }
            }
            if (newObject)
            {
                obj1.initCs();
                obj1.addC(c);
            }

            org.apache.torque.test.B obj2 = (org.apache.torque.test.B) rawResultRow.get(2);
            newObject = true;
            for (int j = 0; j < result.size(); j++)
            {
                org.apache.torque.test.C temp_c = result.get(j);
                org.apache.torque.test.B temp_obj2 = temp_c.getB();
                if (temp_obj2.getPrimaryKey().equals(obj2.getPrimaryKey()))
                {
                    newObject = false;
                    temp_obj2.addC(c);
                    break;
                }
            }
            if (newObject)
            {
                obj2.initCs();
                obj2.addC(c);
            }
            result.add(c);
        }
        return result;
    }
    /**
     * selects a collection of C objects pre-filled with
     * all related objects.
     *
     * This method is protected by default in order to keep the public
     * api reasonable.  You can provide public methods for those you
     * actually need in CPeerImpl.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    protected List<C> doSelectJoinAllExceptB(Criteria criteria)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    CPeer.DATABASE_NAME);
            List<C> result
                    = CPeer.doSelectJoinAllExceptB(criteria, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * selects a collection of C objects pre-filled with
     * related objects.
     *
     * This method is protected by default in order to keep the public
     * api reasonable.  You can provide public methods for those you
     * actually need in CPeerImpl.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    protected List<C> doSelectJoinAllExceptB(Criteria criteria, Connection conn)
        throws TorqueException
    {
        setDbName(criteria);

        CompositeMapper compositeMapper = new CompositeMapper();
        compositeMapper.addMapper(new CRecordMapper(), 0);
        CPeer.addSelectColumns(criteria);
        int offset = CPeer.numColumns;
 
        org.apache.torque.test.RAbPeer.addSelectColumns(criteria);
        criteria.addJoin(CPeer.A_ID, org.apache.torque.test.RAbPeer.A_ID);
        criteria.addJoin(CPeer.B_ID, org.apache.torque.test.RAbPeer.B_ID);
        compositeMapper.addMapper(new RAbRecordMapper(), offset);
        offset = offset + org.apache.torque.test.RAbPeer.numColumns;
 
        org.apache.torque.test.APeer.addSelectColumns(criteria);
        criteria.addJoin(CPeer.A_ID, org.apache.torque.test.APeer.A_ID);
        compositeMapper.addMapper(new ARecordMapper(), offset);
        offset = offset + org.apache.torque.test.APeer.numColumns;

        correctBooleans(criteria);

        List<C> result = new ArrayList<C>();
        List<List<Object>> rawResult = doSelect(
                criteria, compositeMapper, conn);
        for (List<Object> rawResultRow : rawResult)
        {
            C c = (C) rawResultRow.get(0);

            org.apache.torque.test.RAb obj1 = (org.apache.torque.test.RAb) rawResultRow.get(1);
            boolean newObject = true;
            for (int j = 0; j < result.size(); j++)
            {
                org.apache.torque.test.C temp_c = result.get(j);
                org.apache.torque.test.RAb temp_obj1 = temp_c.getRAb();
                if (temp_obj1.getPrimaryKey().equals(obj1.getPrimaryKey()))
                {
                    newObject = false;
                    temp_obj1.addC(c);
                    break;
                }
            }
            if (newObject)
            {
                obj1.initCs();
                obj1.addC(c);
            }

            org.apache.torque.test.A obj2 = (org.apache.torque.test.A) rawResultRow.get(2);
            newObject = true;
            for (int j = 0; j < result.size(); j++)
            {
                org.apache.torque.test.C temp_c = result.get(j);
                org.apache.torque.test.A temp_obj2 = temp_c.getA();
                if (temp_obj2.getPrimaryKey().equals(obj2.getPrimaryKey()))
                {
                    newObject = false;
                    temp_obj2.addC(c);
                    break;
                }
            }
            if (newObject)
            {
                obj2.initCs();
                obj2.addC(c);
            }
            result.add(c);
        }
        return result;
    }

    /** 
     * Fills the referenced RAb objects in the aRAb fields
     * of the objects in <code>toFill</code> by reading the database.
     *
     * @param toFill Contains the objects to fill, not null,
     *        may not contain null.
     *
     * @return all read RAb objects.
     *
     * @throws TorqueException if an error querying the database occurs.
     * @throws NullPointerException if toFill is null or contains null elements.
     */
    public List<RAb> fillRAbs(
            Collection<C> toFill) 
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    CPeer.DATABASE_NAME);
            List<RAb> result = fillRAbs(toFill, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /** 
     * Fills the referenced RAb objects in the aRAb fields
     * of the objects in <code>toFill</code> by reading the database.
     *
     * @param toFill Contains the objects to fill, not null,
     *        may not contain null.
     * @param connection the database connection to use, not null.
     *
     * @return all read RAb objects.
     *
     * @throws TorqueException if an error querying the database occurs.
     * @throws NullPointerException if toFill is null or contains null elements.
     */
    public List<RAb> fillRAbs(
            Collection<C> toFill,
            Connection connection) 
        throws TorqueException
    {
        Set<ObjectKey> foreignKeySet = new HashSet<ObjectKey>();
        for (C currentC : toFill)
        {
            ObjectKey foreignKey = currentC.getForeignKeyForRAb();
            if (foreignKey != null)
            {
                foreignKeySet.add(foreignKey);
            }
        }

        int fillerChunkSize = getFillerChunkSize();
        Iterator<ObjectKey> foreignKeyIt = foreignKeySet.iterator();
        List<ObjectKey> foreignKeyListChunk
                = new ArrayList<ObjectKey>(fillerChunkSize);;
        Map<ObjectKey, RAb> referencedKeyToRAbMap 
                = new HashMap<ObjectKey, RAb>();
        while (foreignKeyIt.hasNext())
        {
            ObjectKey currentKey = foreignKeyIt.next();
            foreignKeyListChunk.add(currentKey);
            if (foreignKeyListChunk.size() < fillerChunkSize 
                && foreignKeyIt.hasNext())
            {
                continue;
            }

            List<RAb> referencedRAbList
                 = RAbPeer.retrieveByPKs(
                     foreignKeyListChunk,
                     connection);

            for (RAb referencedObject : referencedRAbList)
            {
                referencedKeyToRAbMap.put(
                        referencedObject.getPrimaryKey(),
                        referencedObject);
            }
            foreignKeyListChunk.clear();
        }

        List<RAb> result = new ArrayList<RAb>();
        for (C objectToFill : toFill)
        {
            ObjectKey foreignKey = objectToFill.getForeignKeyForRAb();
            if (foreignKey == null || foreignKey.getValue() == null)
            {
                continue;
            }
            RAb referenced = referencedKeyToRAbMap.get(
                    foreignKey);
            if (referenced == null)
            {
                throw new NoRowsException("No result found for key " 
                    + foreignKey + " in table r_ab");
            }
            // copy the referenced object for the case that more than one
            // toFill object references the same object; in this case
            // every toFillObject should have its own instance of the referenced
            // object to remain consistent with the other Torque reference
            // treatment
            RAb referencedCopy = referenced.copy(false);
            // copy does not set primary key
            referencedCopy.setPrimaryKey(referenced.getPrimaryKey());
            objectToFill.setRAb(referencedCopy);
            result.add(referencedCopy);
        }
        return result;
    }
    /** 
     * Fills the referenced A objects in the aA fields
     * of the objects in <code>toFill</code> by reading the database.
     *
     * @param toFill Contains the objects to fill, not null,
     *        may not contain null.
     *
     * @return all read A objects.
     *
     * @throws TorqueException if an error querying the database occurs.
     * @throws NullPointerException if toFill is null or contains null elements.
     */
    public List<A> fillAs(
            Collection<C> toFill) 
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    CPeer.DATABASE_NAME);
            List<A> result = fillAs(toFill, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /** 
     * Fills the referenced A objects in the aA fields
     * of the objects in <code>toFill</code> by reading the database.
     *
     * @param toFill Contains the objects to fill, not null,
     *        may not contain null.
     * @param connection the database connection to use, not null.
     *
     * @return all read A objects.
     *
     * @throws TorqueException if an error querying the database occurs.
     * @throws NullPointerException if toFill is null or contains null elements.
     */
    public List<A> fillAs(
            Collection<C> toFill,
            Connection connection) 
        throws TorqueException
    {
        Set<ObjectKey> foreignKeySet = new HashSet<ObjectKey>();
        for (C currentC : toFill)
        {
            ObjectKey foreignKey = currentC.getForeignKeyForA();
            if (foreignKey != null)
            {
                foreignKeySet.add(foreignKey);
            }
        }

        int fillerChunkSize = getFillerChunkSize();
        Iterator<ObjectKey> foreignKeyIt = foreignKeySet.iterator();
        List<ObjectKey> foreignKeyListChunk
                = new ArrayList<ObjectKey>(fillerChunkSize);;
        Map<ObjectKey, A> referencedKeyToAMap 
                = new HashMap<ObjectKey, A>();
        while (foreignKeyIt.hasNext())
        {
            ObjectKey currentKey = foreignKeyIt.next();
            foreignKeyListChunk.add(currentKey);
            if (foreignKeyListChunk.size() < fillerChunkSize 
                && foreignKeyIt.hasNext())
            {
                continue;
            }

            List<A> referencedAList
                 = APeer.retrieveByPKs(
                     foreignKeyListChunk,
                     connection);

            for (A referencedObject : referencedAList)
            {
                referencedKeyToAMap.put(
                        referencedObject.getPrimaryKey(),
                        referencedObject);
            }
            foreignKeyListChunk.clear();
        }

        List<A> result = new ArrayList<A>();
        for (C objectToFill : toFill)
        {
            ObjectKey foreignKey = objectToFill.getForeignKeyForA();
            if (foreignKey == null || foreignKey.getValue() == null)
            {
                continue;
            }
            A referenced = referencedKeyToAMap.get(
                    foreignKey);
            if (referenced == null)
            {
                throw new NoRowsException("No result found for key " 
                    + foreignKey + " in table a");
            }
            // copy the referenced object for the case that more than one
            // toFill object references the same object; in this case
            // every toFillObject should have its own instance of the referenced
            // object to remain consistent with the other Torque reference
            // treatment
            A referencedCopy = referenced.copy(false);
            // copy does not set primary key
            referencedCopy.setPrimaryKey(referenced.getPrimaryKey());
            objectToFill.setA(referencedCopy);
            result.add(referencedCopy);
        }
        return result;
    }
    /** 
     * Fills the referenced B objects in the aB fields
     * of the objects in <code>toFill</code> by reading the database.
     *
     * @param toFill Contains the objects to fill, not null,
     *        may not contain null.
     *
     * @return all read B objects.
     *
     * @throws TorqueException if an error querying the database occurs.
     * @throws NullPointerException if toFill is null or contains null elements.
     */
    public List<B> fillBs(
            Collection<C> toFill) 
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    CPeer.DATABASE_NAME);
            List<B> result = fillBs(toFill, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /** 
     * Fills the referenced B objects in the aB fields
     * of the objects in <code>toFill</code> by reading the database.
     *
     * @param toFill Contains the objects to fill, not null,
     *        may not contain null.
     * @param connection the database connection to use, not null.
     *
     * @return all read B objects.
     *
     * @throws TorqueException if an error querying the database occurs.
     * @throws NullPointerException if toFill is null or contains null elements.
     */
    public List<B> fillBs(
            Collection<C> toFill,
            Connection connection) 
        throws TorqueException
    {
        Set<ObjectKey> foreignKeySet = new HashSet<ObjectKey>();
        for (C currentC : toFill)
        {
            ObjectKey foreignKey = currentC.getForeignKeyForB();
            if (foreignKey != null)
            {
                foreignKeySet.add(foreignKey);
            }
        }

        int fillerChunkSize = getFillerChunkSize();
        Iterator<ObjectKey> foreignKeyIt = foreignKeySet.iterator();
        List<ObjectKey> foreignKeyListChunk
                = new ArrayList<ObjectKey>(fillerChunkSize);;
        Map<ObjectKey, B> referencedKeyToBMap 
                = new HashMap<ObjectKey, B>();
        while (foreignKeyIt.hasNext())
        {
            ObjectKey currentKey = foreignKeyIt.next();
            foreignKeyListChunk.add(currentKey);
            if (foreignKeyListChunk.size() < fillerChunkSize 
                && foreignKeyIt.hasNext())
            {
                continue;
            }

            List<B> referencedBList
                 = BPeer.retrieveByPKs(
                     foreignKeyListChunk,
                     connection);

            for (B referencedObject : referencedBList)
            {
                referencedKeyToBMap.put(
                        referencedObject.getPrimaryKey(),
                        referencedObject);
            }
            foreignKeyListChunk.clear();
        }

        List<B> result = new ArrayList<B>();
        for (C objectToFill : toFill)
        {
            ObjectKey foreignKey = objectToFill.getForeignKeyForB();
            if (foreignKey == null || foreignKey.getValue() == null)
            {
                continue;
            }
            B referenced = referencedKeyToBMap.get(
                    foreignKey);
            if (referenced == null)
            {
                throw new NoRowsException("No result found for key " 
                    + foreignKey + " in table b");
            }
            // copy the referenced object for the case that more than one
            // toFill object references the same object; in this case
            // every toFillObject should have its own instance of the referenced
            // object to remain consistent with the other Torque reference
            // treatment
            B referencedCopy = referenced.copy(false);
            // copy does not set primary key
            referencedCopy.setPrimaryKey(referenced.getPrimaryKey());
            objectToFill.setB(referencedCopy);
            result.add(referencedCopy);
        }
        return result;
    }


    /** 
     * Returns the chunk size for the filler methods.
     *
     * @return the chunk size for the filler methods.
     */
    protected int getFillerChunkSize()
    {
        return 999;
    }


}
