package org.apache.torque.test;

import java.sql.ResultSet;
import java.sql.SQLException;

import org.apache.torque.TorqueException;
import org.apache.torque.om.mapper.RecordMapper;

    
/**
 * Maps ResultSet rows into CurrentDateTable objects.
 *
 * The skeleton for this class was autogenerated by Torque on:
 *
 * [Thu Sep 13 04:35:47 CEST 2012]
 *
 */
public class BaseCurrentDateTableRecordMapper implements RecordMapper<CurrentDateTable>
{
    /**
     * Constructs the object from the current row in the resultSet.
     *
     * @param resultSet the resultSet to operate on, already pointing
     *        to the correct row. Not null.
     * @param a possible offset in the rows to be considered
     *        (if previous rows contain other objects).
     *
     * @return the mapped object, not null.
     *
     * @throws TorqueException when reading fields from the RecordSet fails.
     */
    public CurrentDateTable processRow(ResultSet resultSet, int offset)
            throws TorqueException
    {
        CurrentDateTable currentDateTable = new CurrentDateTable();

        try 
        {
            currentDateTable.setLoading(true);
            currentDateTable.setId(
                    getId(resultSet, offset + 1));
            currentDateTable.setPayload(
                    getPayload(resultSet, offset + 2));
            currentDateTable.setCurrentDateValue(
                    getCurrentDateValue(resultSet, offset + 3));
            currentDateTable.setNew(false);
            currentDateTable.setModified(false);
        }
        finally
        {
            currentDateTable.setLoading(false);
        }
        return currentDateTable;
    }

        /**
         * Reads the value of the <code>columnIndex</code>'th column
         * in the <code>resultSet</code> so that it can be used to set
         * the field id in CurrentDateTable.
         *
         * @param dbObject the database object to fill, not null.
         * @param resultSet the ResultSet to read from, not null.
         * @param columnIndex the index in the resultSet which should be read.
         *
         * @return the content of the column.
         *
         * @throws SQLException if a problem occurs when reading
         *         from the resultSet.
         */
        protected int getId(
                    ResultSet resultSet,
                    int columnIndex)
                throws TorqueException
        {
            try
            {
                return resultSet.getInt(columnIndex);
            }
            catch (SQLException e)
            {
                throw new TorqueException(e);
            }
        }

        /**
         * Reads the value of the <code>columnIndex</code>'th column
         * in the <code>resultSet</code> so that it can be used to set
         * the field payload in CurrentDateTable.
         *
         * @param dbObject the database object to fill, not null.
         * @param resultSet the ResultSet to read from, not null.
         * @param columnIndex the index in the resultSet which should be read.
         *
         * @return the content of the column.
         *
         * @throws SQLException if a problem occurs when reading
         *         from the resultSet.
         */
        protected int getPayload(
                    ResultSet resultSet,
                    int columnIndex)
                throws TorqueException
        {
            try
            {
                return resultSet.getInt(columnIndex);
            }
            catch (SQLException e)
            {
                throw new TorqueException(e);
            }
        }

        /**
         * Reads the value of the <code>columnIndex</code>'th column
         * in the <code>resultSet</code> so that it can be used to set
         * the field currentDateValue in CurrentDateTable.
         *
         * @param dbObject the database object to fill, not null.
         * @param resultSet the ResultSet to read from, not null.
         * @param columnIndex the index in the resultSet which should be read.
         *
         * @return the content of the column.
         *
         * @throws SQLException if a problem occurs when reading
         *         from the resultSet.
         */
        protected java.util.Date getCurrentDateValue(
                    ResultSet resultSet,
                    int columnIndex)
                throws TorqueException
        {
            try
            {
                java.util.Date value = resultSet.getDate(columnIndex);
                if (resultSet.wasNull())
                {
                    value = null;
                }
                return value;
            }
            catch (SQLException e)
            {
                throw new TorqueException(e);
            }
        }


}
