package org.apache.torque.test;

import java.sql.Blob;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.Types;
import java.io.IOException;
import java.io.InputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.Reader;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.torque.NoRowsException;
import org.apache.torque.TooManyRowsException;
import org.apache.torque.Torque;
import org.apache.torque.TorqueException;
import org.apache.torque.TorqueRuntimeException;
import org.apache.torque.criteria.Criteria;
import org.apache.torque.criteria.Criterion;
import org.apache.torque.om.mapper.RecordMapper;
import org.apache.torque.om.mapper.CompositeMapper;
import org.apache.torque.om.DateKey;
import org.apache.torque.om.NumberKey;
import org.apache.torque.om.StringKey;
import org.apache.torque.om.ObjectKey;
import org.apache.torque.om.SimpleKey;
import org.apache.torque.om.ComboKey;
import org.apache.torque.map.TableMap;
import org.apache.torque.util.BasePeer;
import org.apache.torque.util.Transaction;
import org.apache.torque.util.ColumnValues;
import org.apache.torque.util.JdbcTypedValue;



/**
 * The skeleton for this class was autogenerated by Torque on:
 *
 * [Thu Sep 13 04:35:45 CEST 2012]
 *
 * You should not use this class directly.  It should not even be
 * extended; all references should be to DPeer
 */

public abstract class BaseDPeerImpl
    extends org.apache.torque.util.BasePeerImpl<D>
{
    /** The class log. */
    private static Log log = LogFactory.getLog(BaseDPeerImpl.class);

    /** Serial version */
    private static final long serialVersionUID = 1347503745365L;



    /**
     * Constructor.
     * The recordMapper, tableMap and databaseName fields are correctly
     * initialized.
     */
    public BaseDPeerImpl()
    {
        this(new DRecordMapper(),
            DPeer.TABLE,
            DPeer.DATABASE_NAME);
    }

    /**
     * Constructor providing the objects to be injected as parameters.
     *
     * @param recordMapper a record mapper to map JDBC result sets to objects
     * @param tableMap the default table map
     * @param databaseName the name of the database
     */
    public BaseDPeerImpl(
            RecordMapper<D> recordMapper, 
            TableMap tableMap,
            String databaseName)
    {
        super(recordMapper, tableMap, databaseName);
    }


    /**
     * Selects D objects from the database which have
     * the same content as the passed object.
     *
     * @return The list of selected objects, not null.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<D> doSelect(D obj)
            throws TorqueException
    {
        return doSelect(buildSelectCriteria(obj));
    }

    /**
     * Selects at most one D object from the database
     * which has the same content as the passed object.
     *
     * @return the selected Object, or null if no object was selected.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public D doSelectSingleRecord(
                D obj)
            throws TorqueException
    {
        List<D> dList = doSelect(obj);
        D d = null;
        if (dList.size() > 1)
        {
            throw new TooManyRowsException("Object " + obj 
                + " matched more than one record");
        }
        if (!dList.isEmpty())
        {
            d = dList.get(0);
        }
        return d;
    }

    /**
     * Returns a new instance of the Data object class
     */
    public D getDbObjectInstance()
    {
        return new D();
    }


    /**
     * Method to do inserts.
     *
     * @param columnValues the values to insert.
     *
     * @return the primary key of the inserted row.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public ObjectKey doInsert(ColumnValues columnValues) throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    DPeer.DATABASE_NAME);
            ObjectKey result = doInsert(columnValues, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Method to do inserts.  This method is to be used during a transaction,
     * otherwise use the doInsert(Criteria) method.
     *
     * @param columnValues the values to insert.
     * @param con the connection to use, not null.
     *
     * @return the primary key of the inserted row.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public ObjectKey doInsert(ColumnValues columnValues, Connection con)
        throws TorqueException
    {
        correctBooleans(columnValues);
        return super.doInsert(columnValues, con);
    }

    /**
     * Method to do inserts
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public void doInsert(D obj) throws TorqueException
    {
        doInsert(buildColumnValues(obj));
        obj.setNew(false);
        obj.setModified(false);
    }

    /**
     * Method to do inserts.  This method is to be used during a transaction,
     * otherwise use the doInsert(D) method.  It will take
     * care of the connection details internally.
     *
     * @param obj the data object to insert into the database.
     * @param con the connection to use
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public void doInsert(D obj, Connection con)
        throws TorqueException
    {
        doInsert(buildColumnValues(obj), con);
        obj.setNew(false);
        obj.setModified(false);
    }

    /**
     * Method to do updates.
     *
     * @param columnValues the values to update plus the primary key
     *        identifying the row to update.
     *
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(ColumnValues columnValues) throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    DPeer.DATABASE_NAME);
            int result = doUpdate(columnValues, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Method to do updates.  This method is to be used during a transaction,
     * otherwise use the doUpdate(Criteria) method.
     *
     * @param columnValues the values to update plus the primary key
     *        identifying the row to update.
     * @param con the connection to use, not null.
     *
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(ColumnValues columnValues, Connection con)
        throws TorqueException
    {
        Criteria selectCriteria 
                = new Criteria(DPeer.DATABASE_NAME);
        correctBooleans(columnValues);


    
        selectCriteria.where(
                DPeer.D_ID,
                columnValues.remove(DPeer.D_ID).getValue());


    
        selectCriteria.where(
                DPeer.B_ID,
                columnValues.remove(DPeer.B_ID).getValue());

    
        selectCriteria.where(
                DPeer.A_ID,
                columnValues.remove(DPeer.A_ID).getValue());
        return doUpdate(selectCriteria, columnValues, con);
    }

    /**
     * Updates an D in the database.
     * The primary key is used to identify the object to update.
     *
     * @param obj the data object to update in the database.
     *
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(D obj) throws TorqueException
    {
        ColumnValues columnValues = buildColumnValues(obj);
        int result = doUpdate(columnValues);
        obj.setModified(false);
        return result;
    }

    /**
     * Updates a D in the database.
     * The primary key is used to identify the object to update.
     * This method is to be used during a transaction,
     * otherwise the doUpdate(D) method can be used.
     *
     * @param obj the data object to update in the database.
     * @param con the connection to use, not null.
     
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(D obj, Connection con)
        throws TorqueException
    {
        ColumnValues columnValues = buildColumnValues(obj);
        int result = doUpdate(columnValues, con);
        obj.setModified(false);
        return result;
    }

    /**
     * Deletes a data object, i.e. a row in a table, in the database.
     *
     * @param obj the data object to delete in the database, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(D obj) throws TorqueException
    {
        int result = doDelete(buildCriteria(obj.getPrimaryKey()));
        obj.setDeleted(true);
        return result;
    }

    /**
     * Deletes a data object, i.e. a row in a table, in the database.
     * This method is to be used during a transaction, otherwise use the
     * doDelete(D) method.
     *
     * @param obj the data object to delete in the database, not null.
     * @param con the connection to use, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(D obj, Connection con)
        throws TorqueException
    {
        int result = doDelete(buildCriteria(obj.getPrimaryKey()), con);
        obj.setDeleted(true);
        return result;
    }

    /**
     * Deletes data objects, i.e. rows in a table, in the database.
     *
     * @param objects the data object to delete in the database, not null,
     *        may not contain null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(Collection<D> objects)
            throws TorqueException
    {
        int result = doDelete(buildPkCriteria(objects));
        for (D object : objects)
        {
            object.setDeleted(true);
        }
        return result;
    }

    /**
     * Deletes data objects, i.e. rows in a table, in the database.
     * This method uses the passed connection to delete the rows;
     * if a transaction is open in the connection, the deletion happens inside
     * this transaction.
     *
     * @param objects the data objects to delete in the database, not null,
     *        may not contain null.
     * @param con the connection to use for deleting, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(
            Collection<D> objects,
            Connection con)
        throws TorqueException
    {
        int result = doDelete(buildPkCriteria(objects), con);
        for (D object : objects)
        {
            object.setDeleted(true);
        }
        return result;
    }

    /**
     * Deletes a row in the database.
     *
     * @param pk the ObjectKey that identifies the row to delete.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(ObjectKey pk) throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    DPeer.DATABASE_NAME);
            int deletedRows = doDelete(pk, connection);
            Transaction.commit(connection);
            connection = null;
            return deletedRows;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Deletes a row in the database.
     * This method is to be used during a transaction,
     * otherwise use the doDelete(ObjectKey) method.
     *
     * @param pk the ObjectKey that identifies the row to delete.
     * @param con the connection to use for deleting, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(ObjectKey pk, Connection con)
        throws TorqueException
    {
        return doDelete(buildCriteria(pk), con);
    }

    /** 
     * Build a Criteria object which selects all objects which have a given
     * primary key.
     *
     * @param pk the primary key value to build the criteria from, not null.
     */
    public Criteria buildCriteria(ObjectKey pk)
    {
        Criteria criteria = new Criteria();
        SimpleKey[] keys = (SimpleKey[])pk.getValue();
        criteria.and(DPeer.D_ID, keys[0]);
        criteria.and(DPeer.B_ID, keys[1]);
        criteria.and(DPeer.A_ID, keys[2]);
        return criteria;
     }

    /** 
     * Build a Criteria object which selects all objects which primary keys
     * are contained in the passed collection.
     *
     * @param pks the primary key values to build the criteria from, not null,
     *        may not contain null.
     */
    public Criteria buildCriteria(Collection<ObjectKey> pks)
    {
        Criteria criteria = new Criteria();
        for (ObjectKey pk : pks)
        {
            SimpleKey[] keys = (SimpleKey[])pk.getValue();
                Criterion c0 = new Criterion(
                    DPeer.D_ID, keys[0], Criteria.EQUAL);
                Criterion c1 = new Criterion(
                    DPeer.B_ID, keys[1], Criteria.EQUAL);
                    c0.and(c1);
                Criterion c2 = new Criterion(
                    DPeer.A_ID, keys[2], Criteria.EQUAL);
                    c1.and(c2);
            criteria.or(c0);
        }
        return criteria;
     }


    /** 
     * Build a Criteria object which selects all passed objects using their
     * primary key. Objects which do not yet have a primary key are ignored.
     *
     * @param objects the objects to build the criteria from, not null,
     *        may not contain null.
     */
    public Criteria buildPkCriteria(
            Collection<D> objects)
    {
        List<ObjectKey> pks = new ArrayList<ObjectKey>(objects.size());
        for (D object : objects)
        {
            ObjectKey pk = object.getPrimaryKey();
            if (pk != null)
            {
                pks.add(pk);
            }
        }
        return buildCriteria(pks);
    }

    /** 
     * Build a Criteria object from the data object for this peer.
     *
     * @param obj the object to build the criteria from, not null.
     */
    public Criteria buildCriteria(D obj)
    {
        Criteria criteria = new Criteria(DPeer.DATABASE_NAME);
        criteria.and(DPeer.D_ID, obj.getDId());
        criteria.and(DPeer.NAME, obj.getName());
        criteria.and(DPeer.B_ID, obj.getBId());
        criteria.and(DPeer.A_ID, obj.getAId());
        return criteria;
    }

    /** 
     * Build a Criteria object from the data object for this peer,
     * skipping all binary columns.
     *
     * @param obj the object to build the criteria from, not null.
     */
    public Criteria buildSelectCriteria(D obj)
    {
        Criteria criteria = new Criteria(DPeer.DATABASE_NAME);
        criteria.and(DPeer.D_ID, obj.getDId());
        criteria.and(DPeer.NAME, obj.getName());
        criteria.and(DPeer.B_ID, obj.getBId());
        criteria.and(DPeer.A_ID, obj.getAId());
        return criteria;
    }

    /** 
     * Returns the contents of the object as ColumnValues object.
     * Primary key columns which are generated on insertion are not
     * added to the returned object if they still have their initial
     * value. Also, columns which have the useDatabaseDefaultValue
     * flag set to true are also not added to the returned object
     * if they still have their initial value.
     *
     * @throws TorqueException if the table map cannot be retrieved
     *         (should not happen).
     */
    public ColumnValues buildColumnValues(D d)
            throws TorqueException
    {
        ColumnValues columnValues = new ColumnValues();
        columnValues.put(
                DPeer.D_ID,
                new JdbcTypedValue(
                    d.getDId(),
                    4));
        columnValues.put(
                DPeer.NAME,
                new JdbcTypedValue(
                    d.getName(),
                    12));
        columnValues.put(
                DPeer.B_ID,
                new JdbcTypedValue(
                    d.getBId(),
                    4));
        columnValues.put(
                DPeer.A_ID,
                new JdbcTypedValue(
                    d.getAId(),
                    4));
        return columnValues;
    }

    /**
     * retrieve object using using pk values.
     *
     * @param dId int
     * @param bId int
     * @param aId int
     */
    public D retrieveByPK(
            int dId,
            int bId,
            int aId)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(DPeer.DATABASE_NAME);
            D result = retrieveByPK(
               dId,
               bId,
               aId,
               connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * retrieve object using using pk values.
     *
     * @param dId int
     * @param bId int
     * @param aId int
     * @param con Connection
     */
    public D retrieveByPK(
            int dId,
            int bId,
            int aId,
            Connection con) throws TorqueException
    {

        Criteria criteria = new Criteria();
        criteria.and(DPeer.D_ID, dId);
        criteria.and(DPeer.B_ID, bId);
        criteria.and(DPeer.A_ID, aId);
        List<D> v = doSelect(criteria, con);
        if (v.size() == 1)
        {
            return v.get(0);
        }
        else
        {
            throw new TorqueException("Failed to select one and only one row.");
        }
    }

    /**
     * Retrieve a single object by pk
     *
     * @param pk the primary key
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     * @throws NoRowsException Primary key was not found in database.
     * @throws TooManyRowsException Primary key was not found in database.
     */
    public D retrieveByPK(ObjectKey pk)
        throws TorqueException, NoRowsException, TooManyRowsException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(DPeer.DATABASE_NAME);
            D result = retrieveByPK(pk, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Retrieve a single object by pk
     *
     * @param pk the primary key
     * @param con the connection to use
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     * @throws NoRowsException Primary key was not found in database.
     * @throws TooManyRowsException Primary key was not found in database.
     */
    public D retrieveByPK(ObjectKey pk, Connection con)
        throws TorqueException, NoRowsException, TooManyRowsException
    {
        Criteria criteria = buildCriteria(pk);
        List<D> v = doSelect(criteria, con);
        if (v.size() == 0)
        {
            throw new NoRowsException("Failed to select a row.");
        }
        else if (v.size() > 1)
        {
            throw new TooManyRowsException("Failed to select only one row.");
        }
        else
        {
            return (D)v.get(0);
        }
    }


    /**
     * Retrieve a multiple objects by pk
     *
     * @param pks List of primary keys
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<D> retrieveByPKs(Collection<ObjectKey> pks)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(DPeer.DATABASE_NAME);
            List<D> result = retrieveByPKs(pks, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Retrieve multiple objects by pk
     *
     * @param pks List of primary keys
     * @param dbcon the connection to use
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<D> retrieveByPKs(
                Collection<ObjectKey> pks,
                Connection dbcon)
            throws TorqueException
    {
        if (pks == null || pks.size() == 0)
        {
            return new ArrayList<D>();
        }
        Criteria criteria = buildCriteria(pks);
        List<D> result = doSelect(criteria, dbcon);
        return result;
    }


    /**
     * Selects a collection of dbObjectClassName objects pre-filled with their
     * A objects.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<D> doSelectJoinA(Criteria criteria)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    DPeer.DATABASE_NAME);
            List<D> result
                    = DPeer.doSelectJoinA(criteria, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Selects a collection of D objects pre-filled with their
     * A objects.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<D> doSelectJoinA(
            Criteria criteria, Connection conn)
        throws TorqueException
    {
        setDbName(criteria);

        addSelectColumns(criteria);
        org.apache.torque.test.APeer.addSelectColumns(criteria);

        CompositeMapper compositeMapper = new CompositeMapper();
        compositeMapper.addMapper(
                new DRecordMapper(),
                0);
        compositeMapper.addMapper(
                new ARecordMapper(),
                DPeer.numColumns);
        criteria.addJoin(DPeer.A_ID,
            org.apache.torque.test.APeer.A_ID);

        correctBooleans(criteria);

        List<D> result = new ArrayList<D>();
        List<List<Object>> rawResult = doSelect(
                criteria, compositeMapper, conn);
        for (List<Object> rawResultRow : rawResult)
        {
            D obj1 = (D) rawResultRow.get(0);
            A obj2 = (A) rawResultRow.get(1);

            boolean newObject = true;
            for (int j = 0; j < result.size(); j++)
            {
                D temp_obj1 = result.get(j);
                org.apache.torque.test.A temp_obj2 = temp_obj1.getA();
                if (temp_obj2.getPrimaryKey().equals(obj2.getPrimaryKey()))
                {
                    newObject = false;
                    temp_obj2.addD(obj1);
                    break;
                }
            }
            if (newObject)
            {
                obj2.initDs();
                obj2.addD(obj1);
            }
            result.add(obj1);
        }
        return result;
    }
    /**
     * Selects a collection of dbObjectClassName objects pre-filled with their
     * RAb objects.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<D> doSelectJoinRAb(Criteria criteria)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    DPeer.DATABASE_NAME);
            List<D> result
                    = DPeer.doSelectJoinRAb(criteria, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Selects a collection of D objects pre-filled with their
     * RAb objects.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<D> doSelectJoinRAb(
            Criteria criteria, Connection conn)
        throws TorqueException
    {
        setDbName(criteria);

        addSelectColumns(criteria);
        org.apache.torque.test.RAbPeer.addSelectColumns(criteria);

        CompositeMapper compositeMapper = new CompositeMapper();
        compositeMapper.addMapper(
                new DRecordMapper(),
                0);
        compositeMapper.addMapper(
                new RAbRecordMapper(),
                DPeer.numColumns);
        criteria.addJoin(DPeer.A_ID,
            org.apache.torque.test.RAbPeer.A_ID);
        criteria.addJoin(DPeer.B_ID,
            org.apache.torque.test.RAbPeer.B_ID);

        correctBooleans(criteria);

        List<D> result = new ArrayList<D>();
        List<List<Object>> rawResult = doSelect(
                criteria, compositeMapper, conn);
        for (List<Object> rawResultRow : rawResult)
        {
            D obj1 = (D) rawResultRow.get(0);
            RAb obj2 = (RAb) rawResultRow.get(1);

            boolean newObject = true;
            for (int j = 0; j < result.size(); j++)
            {
                D temp_obj1 = result.get(j);
                org.apache.torque.test.RAb temp_obj2 = temp_obj1.getRAb();
                if (temp_obj2.getPrimaryKey().equals(obj2.getPrimaryKey()))
                {
                    newObject = false;
                    temp_obj2.addD(obj1);
                    break;
                }
            }
            if (newObject)
            {
                obj2.initDs();
                obj2.addD(obj1);
            }
            result.add(obj1);
        }
        return result;
    }


    /** 
     * Fills the referenced A objects in the aA fields
     * of the objects in <code>toFill</code> by reading the database.
     *
     * @param toFill Contains the objects to fill, not null,
     *        may not contain null.
     *
     * @return all read A objects.
     *
     * @throws TorqueException if an error querying the database occurs.
     * @throws NullPointerException if toFill is null or contains null elements.
     */
    public List<A> fillAs(
            Collection<D> toFill) 
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    DPeer.DATABASE_NAME);
            List<A> result = fillAs(toFill, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /** 
     * Fills the referenced A objects in the aA fields
     * of the objects in <code>toFill</code> by reading the database.
     *
     * @param toFill Contains the objects to fill, not null,
     *        may not contain null.
     * @param connection the database connection to use, not null.
     *
     * @return all read A objects.
     *
     * @throws TorqueException if an error querying the database occurs.
     * @throws NullPointerException if toFill is null or contains null elements.
     */
    public List<A> fillAs(
            Collection<D> toFill,
            Connection connection) 
        throws TorqueException
    {
        Set<ObjectKey> foreignKeySet = new HashSet<ObjectKey>();
        for (D currentD : toFill)
        {
            ObjectKey foreignKey = currentD.getForeignKeyForA();
            if (foreignKey != null)
            {
                foreignKeySet.add(foreignKey);
            }
        }

        int fillerChunkSize = getFillerChunkSize();
        Iterator<ObjectKey> foreignKeyIt = foreignKeySet.iterator();
        List<ObjectKey> foreignKeyListChunk
                = new ArrayList<ObjectKey>(fillerChunkSize);;
        Map<ObjectKey, A> referencedKeyToAMap 
                = new HashMap<ObjectKey, A>();
        while (foreignKeyIt.hasNext())
        {
            ObjectKey currentKey = foreignKeyIt.next();
            foreignKeyListChunk.add(currentKey);
            if (foreignKeyListChunk.size() < fillerChunkSize 
                && foreignKeyIt.hasNext())
            {
                continue;
            }

            List<A> referencedAList
                 = APeer.retrieveByPKs(
                     foreignKeyListChunk,
                     connection);

            for (A referencedObject : referencedAList)
            {
                referencedKeyToAMap.put(
                        referencedObject.getPrimaryKey(),
                        referencedObject);
            }
            foreignKeyListChunk.clear();
        }

        List<A> result = new ArrayList<A>();
        for (D objectToFill : toFill)
        {
            ObjectKey foreignKey = objectToFill.getForeignKeyForA();
            if (foreignKey == null || foreignKey.getValue() == null)
            {
                continue;
            }
            A referenced = referencedKeyToAMap.get(
                    foreignKey);
            if (referenced == null)
            {
                throw new NoRowsException("No result found for key " 
                    + foreignKey + " in table a");
            }
            // copy the referenced object for the case that more than one
            // toFill object references the same object; in this case
            // every toFillObject should have its own instance of the referenced
            // object to remain consistent with the other Torque reference
            // treatment
            A referencedCopy = referenced.copy(false);
            // copy does not set primary key
            referencedCopy.setPrimaryKey(referenced.getPrimaryKey());
            objectToFill.setA(referencedCopy);
            result.add(referencedCopy);
        }
        return result;
    }
    /** 
     * Fills the referenced RAb objects in the aRAb fields
     * of the objects in <code>toFill</code> by reading the database.
     *
     * @param toFill Contains the objects to fill, not null,
     *        may not contain null.
     *
     * @return all read RAb objects.
     *
     * @throws TorqueException if an error querying the database occurs.
     * @throws NullPointerException if toFill is null or contains null elements.
     */
    public List<RAb> fillRAbs(
            Collection<D> toFill) 
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    DPeer.DATABASE_NAME);
            List<RAb> result = fillRAbs(toFill, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /** 
     * Fills the referenced RAb objects in the aRAb fields
     * of the objects in <code>toFill</code> by reading the database.
     *
     * @param toFill Contains the objects to fill, not null,
     *        may not contain null.
     * @param connection the database connection to use, not null.
     *
     * @return all read RAb objects.
     *
     * @throws TorqueException if an error querying the database occurs.
     * @throws NullPointerException if toFill is null or contains null elements.
     */
    public List<RAb> fillRAbs(
            Collection<D> toFill,
            Connection connection) 
        throws TorqueException
    {
        Set<ObjectKey> foreignKeySet = new HashSet<ObjectKey>();
        for (D currentD : toFill)
        {
            ObjectKey foreignKey = currentD.getForeignKeyForRAb();
            if (foreignKey != null)
            {
                foreignKeySet.add(foreignKey);
            }
        }

        int fillerChunkSize = getFillerChunkSize();
        Iterator<ObjectKey> foreignKeyIt = foreignKeySet.iterator();
        List<ObjectKey> foreignKeyListChunk
                = new ArrayList<ObjectKey>(fillerChunkSize);;
        Map<ObjectKey, RAb> referencedKeyToRAbMap 
                = new HashMap<ObjectKey, RAb>();
        while (foreignKeyIt.hasNext())
        {
            ObjectKey currentKey = foreignKeyIt.next();
            foreignKeyListChunk.add(currentKey);
            if (foreignKeyListChunk.size() < fillerChunkSize 
                && foreignKeyIt.hasNext())
            {
                continue;
            }

            List<RAb> referencedRAbList
                 = RAbPeer.retrieveByPKs(
                     foreignKeyListChunk,
                     connection);

            for (RAb referencedObject : referencedRAbList)
            {
                referencedKeyToRAbMap.put(
                        referencedObject.getPrimaryKey(),
                        referencedObject);
            }
            foreignKeyListChunk.clear();
        }

        List<RAb> result = new ArrayList<RAb>();
        for (D objectToFill : toFill)
        {
            ObjectKey foreignKey = objectToFill.getForeignKeyForRAb();
            if (foreignKey == null || foreignKey.getValue() == null)
            {
                continue;
            }
            RAb referenced = referencedKeyToRAbMap.get(
                    foreignKey);
            if (referenced == null)
            {
                throw new NoRowsException("No result found for key " 
                    + foreignKey + " in table r_ab");
            }
            // copy the referenced object for the case that more than one
            // toFill object references the same object; in this case
            // every toFillObject should have its own instance of the referenced
            // object to remain consistent with the other Torque reference
            // treatment
            RAb referencedCopy = referenced.copy(false);
            // copy does not set primary key
            referencedCopy.setPrimaryKey(referenced.getPrimaryKey());
            objectToFill.setRAb(referencedCopy);
            result.add(referencedCopy);
        }
        return result;
    }


    /** 
     * Returns the chunk size for the filler methods.
     *
     * @return the chunk size for the filler methods.
     */
    protected int getFillerChunkSize()
    {
        return 999;
    }


}
