package org.apache.torque.test;

import java.sql.Blob;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.Types;
import java.io.IOException;
import java.io.InputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.Reader;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.torque.NoRowsException;
import org.apache.torque.TooManyRowsException;
import org.apache.torque.Torque;
import org.apache.torque.TorqueException;
import org.apache.torque.TorqueRuntimeException;
import org.apache.torque.criteria.Criteria;
import org.apache.torque.criteria.Criterion;
import org.apache.torque.om.mapper.RecordMapper;
import org.apache.torque.om.mapper.CompositeMapper;
import org.apache.torque.om.DateKey;
import org.apache.torque.om.NumberKey;
import org.apache.torque.om.StringKey;
import org.apache.torque.om.ObjectKey;
import org.apache.torque.om.SimpleKey;
import org.apache.torque.om.ComboKey;
import org.apache.torque.map.TableMap;
import org.apache.torque.util.BasePeer;
import org.apache.torque.util.Transaction;
import org.apache.torque.util.ColumnValues;
import org.apache.torque.util.JdbcTypedValue;



/**
 * The skeleton for this class was autogenerated by Torque on:
 *
 * [Thu Sep 13 04:35:45 CEST 2012]
 *
 * You should not use this class directly.  It should not even be
 * extended; all references should be to DatabaseDefaultValuesPeer
 */

public abstract class BaseDatabaseDefaultValuesPeerImpl
    extends org.apache.torque.util.BasePeerImpl<DatabaseDefaultValues>
{
    /** The class log. */
    private static Log log = LogFactory.getLog(BaseDatabaseDefaultValuesPeerImpl.class);

    /** Serial version */
    private static final long serialVersionUID = 1347503745503L;



    /**
     * Constructor.
     * The recordMapper, tableMap and databaseName fields are correctly
     * initialized.
     */
    public BaseDatabaseDefaultValuesPeerImpl()
    {
        this(new DatabaseDefaultValuesRecordMapper(),
            DatabaseDefaultValuesPeer.TABLE,
            DatabaseDefaultValuesPeer.DATABASE_NAME);
    }

    /**
     * Constructor providing the objects to be injected as parameters.
     *
     * @param recordMapper a record mapper to map JDBC result sets to objects
     * @param tableMap the default table map
     * @param databaseName the name of the database
     */
    public BaseDatabaseDefaultValuesPeerImpl(
            RecordMapper<DatabaseDefaultValues> recordMapper, 
            TableMap tableMap,
            String databaseName)
    {
        super(recordMapper, tableMap, databaseName);
    }


    /**
     * Selects DatabaseDefaultValues objects from the database which have
     * the same content as the passed object.
     *
     * @return The list of selected objects, not null.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<DatabaseDefaultValues> doSelect(DatabaseDefaultValues obj)
            throws TorqueException
    {
        return doSelect(buildSelectCriteria(obj));
    }

    /**
     * Selects at most one DatabaseDefaultValues object from the database
     * which has the same content as the passed object.
     *
     * @return the selected Object, or null if no object was selected.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public DatabaseDefaultValues doSelectSingleRecord(
                DatabaseDefaultValues obj)
            throws TorqueException
    {
        List<DatabaseDefaultValues> databaseDefaultValuesList = doSelect(obj);
        DatabaseDefaultValues databaseDefaultValues = null;
        if (databaseDefaultValuesList.size() > 1)
        {
            throw new TooManyRowsException("Object " + obj 
                + " matched more than one record");
        }
        if (!databaseDefaultValuesList.isEmpty())
        {
            databaseDefaultValues = databaseDefaultValuesList.get(0);
        }
        return databaseDefaultValues;
    }

    /**
     * Returns a new instance of the Data object class
     */
    public DatabaseDefaultValues getDbObjectInstance()
    {
        return new DatabaseDefaultValues();
    }


    /**
     * Method to do inserts.
     *
     * @param columnValues the values to insert.
     *
     * @return the primary key of the inserted row.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public ObjectKey doInsert(ColumnValues columnValues) throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    DatabaseDefaultValuesPeer.DATABASE_NAME);
            ObjectKey result = doInsert(columnValues, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Method to do inserts.  This method is to be used during a transaction,
     * otherwise use the doInsert(Criteria) method.
     *
     * @param columnValues the values to insert.
     * @param con the connection to use, not null.
     *
     * @return the primary key of the inserted row.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public ObjectKey doInsert(ColumnValues columnValues, Connection con)
        throws TorqueException
    {
        correctBooleans(columnValues);
        return super.doInsert(columnValues, con);
    }

    /**
     * Method to do inserts
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public void doInsert(DatabaseDefaultValues obj) throws TorqueException
    {
        obj.setPrimaryKey(doInsert(buildColumnValues(obj)));
        obj.setNew(false);
        obj.setModified(false);
    }

    /**
     * Method to do inserts.  This method is to be used during a transaction,
     * otherwise use the doInsert(DatabaseDefaultValues) method.  It will take
     * care of the connection details internally.
     *
     * @param obj the data object to insert into the database.
     * @param con the connection to use
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public void doInsert(DatabaseDefaultValues obj, Connection con)
        throws TorqueException
    {
        ObjectKey primaryKey = doInsert(buildColumnValues(obj), con);
        if (primaryKey != null)
        {
            obj.setPrimaryKey(primaryKey);
        }
        obj.setNew(false);
        obj.setModified(false);
    }

    /**
     * Method to do updates.
     *
     * @param columnValues the values to update plus the primary key
     *        identifying the row to update.
     *
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(ColumnValues columnValues) throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    DatabaseDefaultValuesPeer.DATABASE_NAME);
            int result = doUpdate(columnValues, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Method to do updates.  This method is to be used during a transaction,
     * otherwise use the doUpdate(Criteria) method.
     *
     * @param columnValues the values to update plus the primary key
     *        identifying the row to update.
     * @param con the connection to use, not null.
     *
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(ColumnValues columnValues, Connection con)
        throws TorqueException
    {
        Criteria selectCriteria 
                = new Criteria(DatabaseDefaultValuesPeer.DATABASE_NAME);
        correctBooleans(columnValues);


    
        selectCriteria.where(
                DatabaseDefaultValuesPeer.ID,
                columnValues.remove(DatabaseDefaultValuesPeer.ID).getValue());







        return doUpdate(selectCriteria, columnValues, con);
    }

    /**
     * Updates an DatabaseDefaultValues in the database.
     * The primary key is used to identify the object to update.
     *
     * @param obj the data object to update in the database.
     *
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(DatabaseDefaultValues obj) throws TorqueException
    {
        ColumnValues columnValues = buildColumnValues(obj);
        int result = doUpdate(columnValues);
        obj.setModified(false);
        return result;
    }

    /**
     * Updates a DatabaseDefaultValues in the database.
     * The primary key is used to identify the object to update.
     * This method is to be used during a transaction,
     * otherwise the doUpdate(DatabaseDefaultValues) method can be used.
     *
     * @param obj the data object to update in the database.
     * @param con the connection to use, not null.
     
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(DatabaseDefaultValues obj, Connection con)
        throws TorqueException
    {
        ColumnValues columnValues = buildColumnValues(obj);
        int result = doUpdate(columnValues, con);
        obj.setModified(false);
        return result;
    }

    /**
     * Deletes a data object, i.e. a row in a table, in the database.
     *
     * @param obj the data object to delete in the database, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(DatabaseDefaultValues obj) throws TorqueException
    {
        int result = doDelete(buildCriteria(obj.getPrimaryKey()));
        obj.setDeleted(true);
        return result;
    }

    /**
     * Deletes a data object, i.e. a row in a table, in the database.
     * This method is to be used during a transaction, otherwise use the
     * doDelete(DatabaseDefaultValues) method.
     *
     * @param obj the data object to delete in the database, not null.
     * @param con the connection to use, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(DatabaseDefaultValues obj, Connection con)
        throws TorqueException
    {
        int result = doDelete(buildCriteria(obj.getPrimaryKey()), con);
        obj.setDeleted(true);
        return result;
    }

    /**
     * Deletes data objects, i.e. rows in a table, in the database.
     *
     * @param objects the data object to delete in the database, not null,
     *        may not contain null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(Collection<DatabaseDefaultValues> objects)
            throws TorqueException
    {
        int result = doDelete(buildPkCriteria(objects));
        for (DatabaseDefaultValues object : objects)
        {
            object.setDeleted(true);
        }
        return result;
    }

    /**
     * Deletes data objects, i.e. rows in a table, in the database.
     * This method uses the passed connection to delete the rows;
     * if a transaction is open in the connection, the deletion happens inside
     * this transaction.
     *
     * @param objects the data objects to delete in the database, not null,
     *        may not contain null.
     * @param con the connection to use for deleting, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(
            Collection<DatabaseDefaultValues> objects,
            Connection con)
        throws TorqueException
    {
        int result = doDelete(buildPkCriteria(objects), con);
        for (DatabaseDefaultValues object : objects)
        {
            object.setDeleted(true);
        }
        return result;
    }

    /**
     * Deletes a row in the database.
     *
     * @param pk the ObjectKey that identifies the row to delete.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(ObjectKey pk) throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    DatabaseDefaultValuesPeer.DATABASE_NAME);
            int deletedRows = doDelete(pk, connection);
            Transaction.commit(connection);
            connection = null;
            return deletedRows;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Deletes a row in the database.
     * This method is to be used during a transaction,
     * otherwise use the doDelete(ObjectKey) method.
     *
     * @param pk the ObjectKey that identifies the row to delete.
     * @param con the connection to use for deleting, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(ObjectKey pk, Connection con)
        throws TorqueException
    {
        return doDelete(buildCriteria(pk), con);
    }

    /** 
     * Build a Criteria object which selects all objects which have a given
     * primary key.
     *
     * @param pk the primary key value to build the criteria from, not null.
     */
    public Criteria buildCriteria(ObjectKey pk)
    {
        Criteria criteria = new Criteria();
        criteria.and(DatabaseDefaultValuesPeer.ID, pk);
        return criteria;
     }

    /** 
     * Build a Criteria object which selects all objects which primary keys
     * are contained in the passed collection.
     *
     * @param pks the primary key values to build the criteria from, not null,
     *        may not contain null.
     */
    public Criteria buildCriteria(Collection<ObjectKey> pks)
    {
        Criteria criteria = new Criteria();
        criteria.andIn(DatabaseDefaultValuesPeer.ID, pks);
        return criteria;
     }


    /** 
     * Build a Criteria object which selects all passed objects using their
     * primary key. Objects which do not yet have a primary key are ignored.
     *
     * @param objects the objects to build the criteria from, not null,
     *        may not contain null.
     */
    public Criteria buildPkCriteria(
            Collection<DatabaseDefaultValues> objects)
    {
        List<ObjectKey> pks = new ArrayList<ObjectKey>(objects.size());
        for (DatabaseDefaultValues object : objects)
        {
            ObjectKey pk = object.getPrimaryKey();
            if (pk != null)
            {
                pks.add(pk);
            }
        }
        return buildCriteria(pks);
    }

    /** 
     * Build a Criteria object from the data object for this peer.
     * The primary key columns are only added if the object is not new.
     *
     * @param obj the object to build the criteria from, not null.
     */
    public Criteria buildCriteria(DatabaseDefaultValues obj)
    {
        Criteria criteria = new Criteria(DatabaseDefaultValuesPeer.DATABASE_NAME);
        if (!obj.isNew())
        {
            criteria.and(DatabaseDefaultValuesPeer.ID, obj.getId());
        }
        criteria.and(DatabaseDefaultValuesPeer.NORMAL_PAYLOAD, obj.getNormalPayload());
        criteria.and(DatabaseDefaultValuesPeer.O_INTEGER, obj.getOInteger());
        criteria.and(DatabaseDefaultValuesPeer.P_INT, obj.getPInt());
        criteria.and(DatabaseDefaultValuesPeer.VARCHAR_FIELD, obj.getVarcharField());
        criteria.and(DatabaseDefaultValuesPeer.DATE_FIELD, obj.getDateField());
        criteria.and(DatabaseDefaultValuesPeer.TIME_FIELD, obj.getTimeField());
        criteria.and(DatabaseDefaultValuesPeer.TIMESTAMP_FIELD, obj.getTimestampField());
        return criteria;
    }

    /** 
     * Build a Criteria object from the data object for this peer,
     * skipping all binary columns.
     *
     * @param obj the object to build the criteria from, not null.
     */
    public Criteria buildSelectCriteria(DatabaseDefaultValues obj)
    {
        Criteria criteria = new Criteria(DatabaseDefaultValuesPeer.DATABASE_NAME);
        if (!obj.isNew())
        {
            criteria.and(DatabaseDefaultValuesPeer.ID, obj.getId());
        }
        criteria.and(DatabaseDefaultValuesPeer.NORMAL_PAYLOAD, obj.getNormalPayload());
        criteria.and(DatabaseDefaultValuesPeer.O_INTEGER, obj.getOInteger());
        criteria.and(DatabaseDefaultValuesPeer.P_INT, obj.getPInt());
        criteria.and(DatabaseDefaultValuesPeer.VARCHAR_FIELD, obj.getVarcharField());
        criteria.and(DatabaseDefaultValuesPeer.DATE_FIELD, obj.getDateField());
        criteria.and(DatabaseDefaultValuesPeer.TIME_FIELD, obj.getTimeField());
        criteria.and(DatabaseDefaultValuesPeer.TIMESTAMP_FIELD, obj.getTimestampField());
        return criteria;
    }

    /** 
     * Returns the contents of the object as ColumnValues object.
     * Primary key columns which are generated on insertion are not
     * added to the returned object if they still have their initial
     * value. Also, columns which have the useDatabaseDefaultValue
     * flag set to true are also not added to the returned object
     * if they still have their initial value.
     *
     * @throws TorqueException if the table map cannot be retrieved
     *         (should not happen).
     */
    public ColumnValues buildColumnValues(DatabaseDefaultValues databaseDefaultValues)
            throws TorqueException
    {
        ColumnValues columnValues = new ColumnValues();
        if (!databaseDefaultValues.isNew() 
            || databaseDefaultValues.getId() != 0)
        {
            columnValues.put(
                    DatabaseDefaultValuesPeer.ID,
                    new JdbcTypedValue(
                        databaseDefaultValues.getId(),
                        4));
        }
        columnValues.put(
                DatabaseDefaultValuesPeer.NORMAL_PAYLOAD,
                new JdbcTypedValue(
                    databaseDefaultValues.getNormalPayload(),
                    4));
        if (!databaseDefaultValues.isNew() 
            || !Integer.valueOf(2).equals(databaseDefaultValues.getOInteger()))
        {
            columnValues.put(
                    DatabaseDefaultValuesPeer.O_INTEGER,
                    new JdbcTypedValue(
                        databaseDefaultValues.getOInteger(),
                        4));
        }
        if (!databaseDefaultValues.isNew() 
            || databaseDefaultValues.getPInt() != 4)
        {
            columnValues.put(
                    DatabaseDefaultValuesPeer.P_INT,
                    new JdbcTypedValue(
                        databaseDefaultValues.getPInt(),
                        4));
        }
        if (!databaseDefaultValues.isNew() 
            || !"Default!".equals(databaseDefaultValues.getVarcharField()))
        {
            columnValues.put(
                    DatabaseDefaultValuesPeer.VARCHAR_FIELD,
                    new JdbcTypedValue(
                        databaseDefaultValues.getVarcharField(),
                        12));
        }
        if (!databaseDefaultValues.isNew() 
            || databaseDefaultValues.getDateField() != null)
        {
            java.util.Date dateField = databaseDefaultValues.getDateField();
            if (dateField != null)
            {
                dateField = new java.sql.Date(dateField.getTime());
            }
            columnValues.put(
                    DatabaseDefaultValuesPeer.DATE_FIELD,
                    new JdbcTypedValue(dateField, Types.DATE));
        }
        if (!databaseDefaultValues.isNew() 
            || databaseDefaultValues.getTimeField() != null)
        {
            java.util.Date timeField = databaseDefaultValues.getTimeField();
            if (timeField != null)
            {
                timeField = new java.sql.Time(timeField.getTime());
            }
            columnValues.put(
                    DatabaseDefaultValuesPeer.TIME_FIELD,
                    new JdbcTypedValue(timeField, Types.TIME));
        }
        if (!databaseDefaultValues.isNew() 
            || databaseDefaultValues.getTimestampField() != null)
        {
            java.util.Date timestampField = databaseDefaultValues.getTimestampField();
            if (timestampField != null)
            {
                timestampField = new java.sql.Timestamp(timestampField.getTime());
            }
            columnValues.put(
                    DatabaseDefaultValuesPeer.TIMESTAMP_FIELD,
                    new JdbcTypedValue(timestampField, Types.TIMESTAMP));
        }
        return columnValues;
    }

    /**
     * Retrieve a single object by pk
     *
     * @param pk the primary key
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     * @throws NoRowsException Primary key was not found in database.
     * @throws TooManyRowsException Primary key was not found in database.
     */
    public DatabaseDefaultValues retrieveByPK(int pk)
        throws TorqueException, NoRowsException, TooManyRowsException
    {
        return retrieveByPK(SimpleKey.keyFor(pk));
    }

    /**
     * Retrieve a single object by pk
     *
     * @param pk the primary key
     * @param con the connection to use
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     * @throws NoRowsException Primary key was not found in database.
     * @throws TooManyRowsException Primary key was not found in database.
     */
    public DatabaseDefaultValues retrieveByPK(int pk, Connection con)
        throws TorqueException, NoRowsException, TooManyRowsException
    {
        return retrieveByPK(SimpleKey.keyFor(pk), con);
    }
    
    
    

    /**
     * Retrieve a single object by pk
     *
     * @param pk the primary key
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     * @throws NoRowsException Primary key was not found in database.
     * @throws TooManyRowsException Primary key was not found in database.
     */
    public DatabaseDefaultValues retrieveByPK(ObjectKey pk)
        throws TorqueException, NoRowsException, TooManyRowsException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(DatabaseDefaultValuesPeer.DATABASE_NAME);
            DatabaseDefaultValues result = retrieveByPK(pk, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Retrieve a single object by pk
     *
     * @param pk the primary key
     * @param con the connection to use
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     * @throws NoRowsException Primary key was not found in database.
     * @throws TooManyRowsException Primary key was not found in database.
     */
    public DatabaseDefaultValues retrieveByPK(ObjectKey pk, Connection con)
        throws TorqueException, NoRowsException, TooManyRowsException
    {
        Criteria criteria = buildCriteria(pk);
        List<DatabaseDefaultValues> v = doSelect(criteria, con);
        if (v.size() == 0)
        {
            throw new NoRowsException("Failed to select a row.");
        }
        else if (v.size() > 1)
        {
            throw new TooManyRowsException("Failed to select only one row.");
        }
        else
        {
            return (DatabaseDefaultValues)v.get(0);
        }
    }


    /**
     * Retrieve a multiple objects by pk
     *
     * @param pks List of primary keys
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<DatabaseDefaultValues> retrieveByPKs(Collection<ObjectKey> pks)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(DatabaseDefaultValuesPeer.DATABASE_NAME);
            List<DatabaseDefaultValues> result = retrieveByPKs(pks, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Retrieve multiple objects by pk
     *
     * @param pks List of primary keys
     * @param dbcon the connection to use
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<DatabaseDefaultValues> retrieveByPKs(
                Collection<ObjectKey> pks,
                Connection dbcon)
            throws TorqueException
    {
        if (pks == null || pks.size() == 0)
        {
            return new ArrayList<DatabaseDefaultValues>();
        }
        Criteria criteria = buildCriteria(pks);
        List<DatabaseDefaultValues> result = doSelect(criteria, dbcon);
        return result;
    }






    /** 
     * Returns the chunk size for the filler methods.
     *
     * @return the chunk size for the filler methods.
     */
    protected int getFillerChunkSize()
    {
        return 999;
    }


}
