package org.apache.torque.test;

import java.sql.ResultSet;
import java.sql.SQLException;

import org.apache.torque.TorqueException;
import org.apache.torque.om.mapper.RecordMapper;

    
/**
 * Maps ResultSet rows into DatabaseDefaultValues objects.
 *
 * The skeleton for this class was autogenerated by Torque on:
 *
 * [Thu Sep 13 04:35:47 CEST 2012]
 *
 */
public class BaseDatabaseDefaultValuesRecordMapper implements RecordMapper<DatabaseDefaultValues>
{
    /**
     * Constructs the object from the current row in the resultSet.
     *
     * @param resultSet the resultSet to operate on, already pointing
     *        to the correct row. Not null.
     * @param a possible offset in the rows to be considered
     *        (if previous rows contain other objects).
     *
     * @return the mapped object, not null.
     *
     * @throws TorqueException when reading fields from the RecordSet fails.
     */
    public DatabaseDefaultValues processRow(ResultSet resultSet, int offset)
            throws TorqueException
    {
        DatabaseDefaultValues databaseDefaultValues = new DatabaseDefaultValues();

        try 
        {
            databaseDefaultValues.setLoading(true);
            databaseDefaultValues.setId(
                    getId(resultSet, offset + 1));
            databaseDefaultValues.setNormalPayload(
                    getNormalPayload(resultSet, offset + 2));
            databaseDefaultValues.setOInteger(
                    getOInteger(resultSet, offset + 3));
            databaseDefaultValues.setPInt(
                    getPInt(resultSet, offset + 4));
            databaseDefaultValues.setVarcharField(
                    getVarcharField(resultSet, offset + 5));
            databaseDefaultValues.setDateField(
                    getDateField(resultSet, offset + 6));
            databaseDefaultValues.setTimeField(
                    getTimeField(resultSet, offset + 7));
            databaseDefaultValues.setTimestampField(
                    getTimestampField(resultSet, offset + 8));
            databaseDefaultValues.setNew(false);
            databaseDefaultValues.setModified(false);
        }
        finally
        {
            databaseDefaultValues.setLoading(false);
        }
        return databaseDefaultValues;
    }

        /**
         * Reads the value of the <code>columnIndex</code>'th column
         * in the <code>resultSet</code> so that it can be used to set
         * the field id in DatabaseDefaultValues.
         *
         * @param dbObject the database object to fill, not null.
         * @param resultSet the ResultSet to read from, not null.
         * @param columnIndex the index in the resultSet which should be read.
         *
         * @return the content of the column.
         *
         * @throws SQLException if a problem occurs when reading
         *         from the resultSet.
         */
        protected int getId(
                    ResultSet resultSet,
                    int columnIndex)
                throws TorqueException
        {
            try
            {
                return resultSet.getInt(columnIndex);
            }
            catch (SQLException e)
            {
                throw new TorqueException(e);
            }
        }

        /**
         * Reads the value of the <code>columnIndex</code>'th column
         * in the <code>resultSet</code> so that it can be used to set
         * the field normalPayload in DatabaseDefaultValues.
         *
         * @param dbObject the database object to fill, not null.
         * @param resultSet the ResultSet to read from, not null.
         * @param columnIndex the index in the resultSet which should be read.
         *
         * @return the content of the column.
         *
         * @throws SQLException if a problem occurs when reading
         *         from the resultSet.
         */
        protected int getNormalPayload(
                    ResultSet resultSet,
                    int columnIndex)
                throws TorqueException
        {
            try
            {
                return resultSet.getInt(columnIndex);
            }
            catch (SQLException e)
            {
                throw new TorqueException(e);
            }
        }

        /**
         * Reads the value of the <code>columnIndex</code>'th column
         * in the <code>resultSet</code> so that it can be used to set
         * the field oInteger in DatabaseDefaultValues.
         *
         * @param dbObject the database object to fill, not null.
         * @param resultSet the ResultSet to read from, not null.
         * @param columnIndex the index in the resultSet which should be read.
         *
         * @return the content of the column.
         *
         * @throws SQLException if a problem occurs when reading
         *         from the resultSet.
         */
        protected Integer getOInteger(
                    ResultSet resultSet,
                    int columnIndex)
                throws TorqueException
        {
            try
            {
                Integer value = resultSet.getInt(columnIndex);
                if (resultSet.wasNull())
                {
                    value = null;
                }
                return value;
            }
            catch (SQLException e)
            {
                throw new TorqueException(e);
            }
        }

        /**
         * Reads the value of the <code>columnIndex</code>'th column
         * in the <code>resultSet</code> so that it can be used to set
         * the field pInt in DatabaseDefaultValues.
         *
         * @param dbObject the database object to fill, not null.
         * @param resultSet the ResultSet to read from, not null.
         * @param columnIndex the index in the resultSet which should be read.
         *
         * @return the content of the column.
         *
         * @throws SQLException if a problem occurs when reading
         *         from the resultSet.
         */
        protected int getPInt(
                    ResultSet resultSet,
                    int columnIndex)
                throws TorqueException
        {
            try
            {
                return resultSet.getInt(columnIndex);
            }
            catch (SQLException e)
            {
                throw new TorqueException(e);
            }
        }

        /**
         * Reads the value of the <code>columnIndex</code>'th column
         * in the <code>resultSet</code> so that it can be used to set
         * the field varcharField in DatabaseDefaultValues.
         *
         * @param dbObject the database object to fill, not null.
         * @param resultSet the ResultSet to read from, not null.
         * @param columnIndex the index in the resultSet which should be read.
         *
         * @return the content of the column.
         *
         * @throws SQLException if a problem occurs when reading
         *         from the resultSet.
         */
        protected String getVarcharField(
                    ResultSet resultSet,
                    int columnIndex)
                throws TorqueException
        {
            try
            {
                return resultSet.getString(columnIndex);
            }
            catch (SQLException e)
            {
                throw new TorqueException(e);
            }
        }

        /**
         * Reads the value of the <code>columnIndex</code>'th column
         * in the <code>resultSet</code> so that it can be used to set
         * the field dateField in DatabaseDefaultValues.
         *
         * @param dbObject the database object to fill, not null.
         * @param resultSet the ResultSet to read from, not null.
         * @param columnIndex the index in the resultSet which should be read.
         *
         * @return the content of the column.
         *
         * @throws SQLException if a problem occurs when reading
         *         from the resultSet.
         */
        protected java.util.Date getDateField(
                    ResultSet resultSet,
                    int columnIndex)
                throws TorqueException
        {
            try
            {
                java.util.Date value = resultSet.getDate(columnIndex);
                if (resultSet.wasNull())
                {
                    value = null;
                }
                return value;
            }
            catch (SQLException e)
            {
                throw new TorqueException(e);
            }
        }

        /**
         * Reads the value of the <code>columnIndex</code>'th column
         * in the <code>resultSet</code> so that it can be used to set
         * the field timeField in DatabaseDefaultValues.
         *
         * @param dbObject the database object to fill, not null.
         * @param resultSet the ResultSet to read from, not null.
         * @param columnIndex the index in the resultSet which should be read.
         *
         * @return the content of the column.
         *
         * @throws SQLException if a problem occurs when reading
         *         from the resultSet.
         */
        protected java.util.Date getTimeField(
                    ResultSet resultSet,
                    int columnIndex)
                throws TorqueException
        {
            try
            {
                java.util.Date value = resultSet.getTime(columnIndex);
                if (resultSet.wasNull())
                {
                    value = null;
                }
                return value;
            }
            catch (SQLException e)
            {
                throw new TorqueException(e);
            }
        }

        /**
         * Reads the value of the <code>columnIndex</code>'th column
         * in the <code>resultSet</code> so that it can be used to set
         * the field timestampField in DatabaseDefaultValues.
         *
         * @param dbObject the database object to fill, not null.
         * @param resultSet the ResultSet to read from, not null.
         * @param columnIndex the index in the resultSet which should be read.
         *
         * @return the content of the column.
         *
         * @throws SQLException if a problem occurs when reading
         *         from the resultSet.
         */
        protected java.util.Date getTimestampField(
                    ResultSet resultSet,
                    int columnIndex)
                throws TorqueException
        {
            try
            {
                java.util.Date value = resultSet.getTimestamp(columnIndex);
                if (resultSet.wasNull())
                {
                    value = null;
                }
                return value;
            }
            catch (SQLException e)
            {
                throw new TorqueException(e);
            }
        }


}
