package org.apache.torque.test;

import java.sql.Blob;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.Types;
import java.io.IOException;
import java.io.InputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.Reader;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.torque.NoRowsException;
import org.apache.torque.TooManyRowsException;
import org.apache.torque.Torque;
import org.apache.torque.TorqueException;
import org.apache.torque.TorqueRuntimeException;
import org.apache.torque.criteria.Criteria;
import org.apache.torque.criteria.Criterion;
import org.apache.torque.om.mapper.RecordMapper;
import org.apache.torque.om.mapper.CompositeMapper;
import org.apache.torque.om.DateKey;
import org.apache.torque.om.NumberKey;
import org.apache.torque.om.StringKey;
import org.apache.torque.om.ObjectKey;
import org.apache.torque.om.SimpleKey;
import org.apache.torque.om.ComboKey;
import org.apache.torque.map.TableMap;
import org.apache.torque.util.BasePeer;
import org.apache.torque.util.Transaction;
import org.apache.torque.util.ColumnValues;
import org.apache.torque.util.JdbcTypedValue;



/**
 * The skeleton for this class was autogenerated by Torque on:
 *
 * [Thu Sep 13 04:35:46 CEST 2012]
 *
 * You should not use this class directly.  It should not even be
 * extended; all references should be to MultiPkPeer
 */

public abstract class BaseMultiPkPeerImpl
    extends org.apache.torque.util.BasePeerImpl<MultiPk>
{
    /** The class log. */
    private static Log log = LogFactory.getLog(BaseMultiPkPeerImpl.class);

    /** Serial version */
    private static final long serialVersionUID = 1347503746175L;



    /**
     * Constructor.
     * The recordMapper, tableMap and databaseName fields are correctly
     * initialized.
     */
    public BaseMultiPkPeerImpl()
    {
        this(new MultiPkRecordMapper(),
            MultiPkPeer.TABLE,
            MultiPkPeer.DATABASE_NAME);
    }

    /**
     * Constructor providing the objects to be injected as parameters.
     *
     * @param recordMapper a record mapper to map JDBC result sets to objects
     * @param tableMap the default table map
     * @param databaseName the name of the database
     */
    public BaseMultiPkPeerImpl(
            RecordMapper<MultiPk> recordMapper, 
            TableMap tableMap,
            String databaseName)
    {
        super(recordMapper, tableMap, databaseName);
    }


    /**
     * Selects MultiPk objects from the database which have
     * the same content as the passed object.
     *
     * @return The list of selected objects, not null.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<MultiPk> doSelect(MultiPk obj)
            throws TorqueException
    {
        return doSelect(buildSelectCriteria(obj));
    }

    /**
     * Selects at most one MultiPk object from the database
     * which has the same content as the passed object.
     *
     * @return the selected Object, or null if no object was selected.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public MultiPk doSelectSingleRecord(
                MultiPk obj)
            throws TorqueException
    {
        List<MultiPk> multiPkList = doSelect(obj);
        MultiPk multiPk = null;
        if (multiPkList.size() > 1)
        {
            throw new TooManyRowsException("Object " + obj 
                + " matched more than one record");
        }
        if (!multiPkList.isEmpty())
        {
            multiPk = multiPkList.get(0);
        }
        return multiPk;
    }

    /**
     * Returns a new instance of the Data object class
     */
    public MultiPk getDbObjectInstance()
    {
        return new MultiPk();
    }


    /**
     * Method to do inserts.
     *
     * @param columnValues the values to insert.
     *
     * @return the primary key of the inserted row.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public ObjectKey doInsert(ColumnValues columnValues) throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    MultiPkPeer.DATABASE_NAME);
            ObjectKey result = doInsert(columnValues, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Method to do inserts.  This method is to be used during a transaction,
     * otherwise use the doInsert(Criteria) method.
     *
     * @param columnValues the values to insert.
     * @param con the connection to use, not null.
     *
     * @return the primary key of the inserted row.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public ObjectKey doInsert(ColumnValues columnValues, Connection con)
        throws TorqueException
    {
        correctBooleans(columnValues);
        return super.doInsert(columnValues, con);
    }

    /**
     * Method to do inserts
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public void doInsert(MultiPk obj) throws TorqueException
    {
        doInsert(buildColumnValues(obj));
        obj.setNew(false);
        obj.setModified(false);
    }

    /**
     * Method to do inserts.  This method is to be used during a transaction,
     * otherwise use the doInsert(MultiPk) method.  It will take
     * care of the connection details internally.
     *
     * @param obj the data object to insert into the database.
     * @param con the connection to use
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public void doInsert(MultiPk obj, Connection con)
        throws TorqueException
    {
        doInsert(buildColumnValues(obj), con);
        obj.setNew(false);
        obj.setModified(false);
    }

    /**
     * Method to do updates.
     *
     * @param columnValues the values to update plus the primary key
     *        identifying the row to update.
     *
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(ColumnValues columnValues) throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    MultiPkPeer.DATABASE_NAME);
            int result = doUpdate(columnValues, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Method to do updates.  This method is to be used during a transaction,
     * otherwise use the doUpdate(Criteria) method.
     *
     * @param columnValues the values to update plus the primary key
     *        identifying the row to update.
     * @param con the connection to use, not null.
     *
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(ColumnValues columnValues, Connection con)
        throws TorqueException
    {
        Criteria selectCriteria 
                = new Criteria(MultiPkPeer.DATABASE_NAME);
        correctBooleans(columnValues);


    
        selectCriteria.where(
                MultiPkPeer.PK1,
                columnValues.remove(MultiPkPeer.PK1).getValue());

    
        selectCriteria.where(
                MultiPkPeer.PK2,
                columnValues.remove(MultiPkPeer.PK2).getValue());

    
        selectCriteria.where(
                MultiPkPeer.PK3,
                columnValues.remove(MultiPkPeer.PK3).getValue());

    
        selectCriteria.where(
                MultiPkPeer.PK4,
                columnValues.remove(MultiPkPeer.PK4).getValue());

    
        selectCriteria.where(
                MultiPkPeer.PK5,
                columnValues.remove(MultiPkPeer.PK5).getValue());

    
        selectCriteria.where(
                MultiPkPeer.PK6,
                columnValues.remove(MultiPkPeer.PK6).getValue());

    
        selectCriteria.where(
                MultiPkPeer.PK7,
                columnValues.remove(MultiPkPeer.PK7).getValue());

    
        selectCriteria.where(
                MultiPkPeer.PK8,
                columnValues.remove(MultiPkPeer.PK8).getValue());

    
        selectCriteria.where(
                MultiPkPeer.PK9,
                columnValues.remove(MultiPkPeer.PK9).getValue());

    
        selectCriteria.where(
                MultiPkPeer.PK10,
                columnValues.remove(MultiPkPeer.PK10).getValue());


        return doUpdate(selectCriteria, columnValues, con);
    }

    /**
     * Updates an MultiPk in the database.
     * The primary key is used to identify the object to update.
     *
     * @param obj the data object to update in the database.
     *
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(MultiPk obj) throws TorqueException
    {
        ColumnValues columnValues = buildColumnValues(obj);
        int result = doUpdate(columnValues);
        obj.setModified(false);
        return result;
    }

    /**
     * Updates a MultiPk in the database.
     * The primary key is used to identify the object to update.
     * This method is to be used during a transaction,
     * otherwise the doUpdate(MultiPk) method can be used.
     *
     * @param obj the data object to update in the database.
     * @param con the connection to use, not null.
     
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(MultiPk obj, Connection con)
        throws TorqueException
    {
        ColumnValues columnValues = buildColumnValues(obj);
        int result = doUpdate(columnValues, con);
        obj.setModified(false);
        return result;
    }

    /**
     * Deletes a data object, i.e. a row in a table, in the database.
     *
     * @param obj the data object to delete in the database, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(MultiPk obj) throws TorqueException
    {
        int result = doDelete(buildCriteria(obj.getPrimaryKey()));
        obj.setDeleted(true);
        return result;
    }

    /**
     * Deletes a data object, i.e. a row in a table, in the database.
     * This method is to be used during a transaction, otherwise use the
     * doDelete(MultiPk) method.
     *
     * @param obj the data object to delete in the database, not null.
     * @param con the connection to use, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(MultiPk obj, Connection con)
        throws TorqueException
    {
        int result = doDelete(buildCriteria(obj.getPrimaryKey()), con);
        obj.setDeleted(true);
        return result;
    }

    /**
     * Deletes data objects, i.e. rows in a table, in the database.
     *
     * @param objects the data object to delete in the database, not null,
     *        may not contain null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(Collection<MultiPk> objects)
            throws TorqueException
    {
        int result = doDelete(buildPkCriteria(objects));
        for (MultiPk object : objects)
        {
            object.setDeleted(true);
        }
        return result;
    }

    /**
     * Deletes data objects, i.e. rows in a table, in the database.
     * This method uses the passed connection to delete the rows;
     * if a transaction is open in the connection, the deletion happens inside
     * this transaction.
     *
     * @param objects the data objects to delete in the database, not null,
     *        may not contain null.
     * @param con the connection to use for deleting, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(
            Collection<MultiPk> objects,
            Connection con)
        throws TorqueException
    {
        int result = doDelete(buildPkCriteria(objects), con);
        for (MultiPk object : objects)
        {
            object.setDeleted(true);
        }
        return result;
    }

    /**
     * Deletes a row in the database.
     *
     * @param pk the ObjectKey that identifies the row to delete.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(ObjectKey pk) throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    MultiPkPeer.DATABASE_NAME);
            int deletedRows = doDelete(pk, connection);
            Transaction.commit(connection);
            connection = null;
            return deletedRows;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Deletes a row in the database.
     * This method is to be used during a transaction,
     * otherwise use the doDelete(ObjectKey) method.
     *
     * @param pk the ObjectKey that identifies the row to delete.
     * @param con the connection to use for deleting, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(ObjectKey pk, Connection con)
        throws TorqueException
    {
        return doDelete(buildCriteria(pk), con);
    }

    /** 
     * Build a Criteria object which selects all objects which have a given
     * primary key.
     *
     * @param pk the primary key value to build the criteria from, not null.
     */
    public Criteria buildCriteria(ObjectKey pk)
    {
        Criteria criteria = new Criteria();
        SimpleKey[] keys = (SimpleKey[])pk.getValue();
        criteria.and(MultiPkPeer.PK1, keys[0]);
        criteria.and(MultiPkPeer.PK2, keys[1]);
        criteria.and(MultiPkPeer.PK3, keys[2]);
        criteria.and(MultiPkPeer.PK4, keys[3]);
        criteria.and(MultiPkPeer.PK5, keys[4]);
        criteria.and(MultiPkPeer.PK6, keys[5]);
        criteria.and(MultiPkPeer.PK7, keys[6]);
        criteria.and(MultiPkPeer.PK8, keys[7]);
        criteria.and(MultiPkPeer.PK9, keys[8]);
        criteria.and(MultiPkPeer.PK10, keys[9]);
        return criteria;
     }

    /** 
     * Build a Criteria object which selects all objects which primary keys
     * are contained in the passed collection.
     *
     * @param pks the primary key values to build the criteria from, not null,
     *        may not contain null.
     */
    public Criteria buildCriteria(Collection<ObjectKey> pks)
    {
        Criteria criteria = new Criteria();
        for (ObjectKey pk : pks)
        {
            SimpleKey[] keys = (SimpleKey[])pk.getValue();
                Criterion c0 = new Criterion(
                    MultiPkPeer.PK1, keys[0], Criteria.EQUAL);
                Criterion c1 = new Criterion(
                    MultiPkPeer.PK2, keys[1], Criteria.EQUAL);
                    c0.and(c1);
                Criterion c2 = new Criterion(
                    MultiPkPeer.PK3, keys[2], Criteria.EQUAL);
                    c1.and(c2);
                Criterion c3 = new Criterion(
                    MultiPkPeer.PK4, keys[3], Criteria.EQUAL);
                    c2.and(c3);
                Criterion c4 = new Criterion(
                    MultiPkPeer.PK5, keys[4], Criteria.EQUAL);
                    c3.and(c4);
                Criterion c5 = new Criterion(
                    MultiPkPeer.PK6, keys[5], Criteria.EQUAL);
                    c4.and(c5);
                Criterion c6 = new Criterion(
                    MultiPkPeer.PK7, keys[6], Criteria.EQUAL);
                    c5.and(c6);
                Criterion c7 = new Criterion(
                    MultiPkPeer.PK8, keys[7], Criteria.EQUAL);
                    c6.and(c7);
                Criterion c8 = new Criterion(
                    MultiPkPeer.PK9, keys[8], Criteria.EQUAL);
                    c7.and(c8);
                Criterion c9 = new Criterion(
                    MultiPkPeer.PK10, keys[9], Criteria.EQUAL);
                    c8.and(c9);
            criteria.or(c0);
        }
        return criteria;
     }


    /** 
     * Build a Criteria object which selects all passed objects using their
     * primary key. Objects which do not yet have a primary key are ignored.
     *
     * @param objects the objects to build the criteria from, not null,
     *        may not contain null.
     */
    public Criteria buildPkCriteria(
            Collection<MultiPk> objects)
    {
        List<ObjectKey> pks = new ArrayList<ObjectKey>(objects.size());
        for (MultiPk object : objects)
        {
            ObjectKey pk = object.getPrimaryKey();
            if (pk != null)
            {
                pks.add(pk);
            }
        }
        return buildCriteria(pks);
    }

    /** 
     * Build a Criteria object from the data object for this peer.
     *
     * @param obj the object to build the criteria from, not null.
     */
    public Criteria buildCriteria(MultiPk obj)
    {
        Criteria criteria = new Criteria(MultiPkPeer.DATABASE_NAME);
        criteria.and(MultiPkPeer.PK1, obj.getPk1());
        criteria.and(MultiPkPeer.PK2, obj.getPk2());
        criteria.and(MultiPkPeer.PK3, obj.getPk3());
        criteria.and(MultiPkPeer.PK4, obj.getPk4());
        criteria.and(MultiPkPeer.PK5, obj.getPk5());
        criteria.and(MultiPkPeer.PK6, obj.getPk6());
        criteria.and(MultiPkPeer.PK7, obj.getPk7());
        criteria.and(MultiPkPeer.PK8, obj.getPk8());
        criteria.and(MultiPkPeer.PK9, obj.getPk9());
        criteria.and(MultiPkPeer.PK10, obj.getPk10());
        criteria.and(MultiPkPeer.COL_A, obj.getColA());
        criteria.and(MultiPkPeer.COL_B, obj.getColB());
        return criteria;
    }

    /** 
     * Build a Criteria object from the data object for this peer,
     * skipping all binary columns.
     *
     * @param obj the object to build the criteria from, not null.
     */
    public Criteria buildSelectCriteria(MultiPk obj)
    {
        Criteria criteria = new Criteria(MultiPkPeer.DATABASE_NAME);
        criteria.and(MultiPkPeer.PK1, obj.getPk1());
        criteria.and(MultiPkPeer.PK2, obj.getPk2());
        criteria.and(MultiPkPeer.PK3, obj.getPk3());
        criteria.and(MultiPkPeer.PK4, obj.getPk4());
        criteria.and(MultiPkPeer.PK5, obj.getPk5());
        criteria.and(MultiPkPeer.PK6, obj.getPk6());
        criteria.and(MultiPkPeer.PK7, obj.getPk7());
        criteria.and(MultiPkPeer.PK8, obj.getPk8());
        criteria.and(MultiPkPeer.PK9, obj.getPk9());
        criteria.and(MultiPkPeer.PK10, obj.getPk10());
        criteria.and(MultiPkPeer.COL_A, obj.getColA());
        criteria.and(MultiPkPeer.COL_B, obj.getColB());
        return criteria;
    }

    /** 
     * Returns the contents of the object as ColumnValues object.
     * Primary key columns which are generated on insertion are not
     * added to the returned object if they still have their initial
     * value. Also, columns which have the useDatabaseDefaultValue
     * flag set to true are also not added to the returned object
     * if they still have their initial value.
     *
     * @throws TorqueException if the table map cannot be retrieved
     *         (should not happen).
     */
    public ColumnValues buildColumnValues(MultiPk multiPk)
            throws TorqueException
    {
        ColumnValues columnValues = new ColumnValues();
        columnValues.put(
                MultiPkPeer.PK1,
                new JdbcTypedValue(
                    multiPk.getPk1(),
                    12));
        columnValues.put(
                MultiPkPeer.PK2,
                new JdbcTypedValue(
                    multiPk.getPk2(),
                    4));
        columnValues.put(
                MultiPkPeer.PK3,
                new JdbcTypedValue(
                    multiPk.getPk3(),
                    1));
        columnValues.put(
                MultiPkPeer.PK4,
                new JdbcTypedValue(
                    multiPk.getPk4(),
                    4));
        columnValues.put(
                MultiPkPeer.PK5,
                new JdbcTypedValue(
                    multiPk.getPk5(),
                    -6));
        columnValues.put(
                MultiPkPeer.PK6,
                new JdbcTypedValue(
                    multiPk.getPk6(),
                    5));
        columnValues.put(
                MultiPkPeer.PK7,
                new JdbcTypedValue(
                    multiPk.getPk7(),
                    -5));
        columnValues.put(
                MultiPkPeer.PK8,
                new JdbcTypedValue(
                    multiPk.getPk8(),
                    6));
        columnValues.put(
                MultiPkPeer.PK9,
                new JdbcTypedValue(
                    multiPk.getPk9(),
                    8));
        java.util.Date pk10 = multiPk.getPk10();
        if (pk10 != null)
        {
            pk10 = new java.sql.Timestamp(pk10.getTime());
        }
        columnValues.put(
                MultiPkPeer.PK10,
                new JdbcTypedValue(pk10, Types.TIMESTAMP));
        Integer colA = Boolean.TRUE.equals(multiPk.getColA())
                                    ? Integer.valueOf(1)
                                    : Integer.valueOf(0);
        columnValues.put(
                MultiPkPeer.COL_A,
                new JdbcTypedValue(colA, Types.INTEGER));
        Integer colB = null;
        if (Boolean.TRUE.equals(multiPk.getColB()))
        {
            colB =  Integer.valueOf(1);
        }
        else if (Boolean.FALSE.equals(multiPk.getColB()))
        {
            colB =  Integer.valueOf(0);
        }
        columnValues.put(
                MultiPkPeer.COL_B,
                new JdbcTypedValue(colB, Types.INTEGER));
        return columnValues;
    }

    /**
     * retrieve object using using pk values.
     *
     * @param pk1 String
     * @param pk2 int
     * @param pk3 String
     * @param pk4 Integer
     * @param pk5 Byte
     * @param pk6 Short
     * @param pk7 Long
     * @param pk8 Double
     * @param pk9 Double
     * @param pk10 java.util.Date
     */
    public MultiPk retrieveByPK(
            String pk1,
            int pk2,
            String pk3,
            Integer pk4,
            Byte pk5,
            Short pk6,
            Long pk7,
            Double pk8,
            Double pk9,
            java.util.Date pk10)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(MultiPkPeer.DATABASE_NAME);
            MultiPk result = retrieveByPK(
               pk1,
               pk2,
               pk3,
               pk4,
               pk5,
               pk6,
               pk7,
               pk8,
               pk9,
               pk10,
               connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * retrieve object using using pk values.
     *
     * @param pk1 String
     * @param pk2 int
     * @param pk3 String
     * @param pk4 Integer
     * @param pk5 Byte
     * @param pk6 Short
     * @param pk7 Long
     * @param pk8 Double
     * @param pk9 Double
     * @param pk10 java.util.Date
     * @param con Connection
     */
    public MultiPk retrieveByPK(
            String pk1,
            int pk2,
            String pk3,
            Integer pk4,
            Byte pk5,
            Short pk6,
            Long pk7,
            Double pk8,
            Double pk9,
            java.util.Date pk10,
            Connection con) throws TorqueException
    {

        Criteria criteria = new Criteria();
        criteria.and(MultiPkPeer.PK1, pk1);
        criteria.and(MultiPkPeer.PK2, pk2);
        criteria.and(MultiPkPeer.PK3, pk3);
        criteria.and(MultiPkPeer.PK4, pk4);
        criteria.and(MultiPkPeer.PK5, pk5);
        criteria.and(MultiPkPeer.PK6, pk6);
        criteria.and(MultiPkPeer.PK7, pk7);
        criteria.and(MultiPkPeer.PK8, pk8);
        criteria.and(MultiPkPeer.PK9, pk9);
        criteria.and(MultiPkPeer.PK10, pk10);
        List<MultiPk> v = doSelect(criteria, con);
        if (v.size() == 1)
        {
            return v.get(0);
        }
        else
        {
            throw new TorqueException("Failed to select one and only one row.");
        }
    }

    /**
     * Retrieve a single object by pk
     *
     * @param pk the primary key
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     * @throws NoRowsException Primary key was not found in database.
     * @throws TooManyRowsException Primary key was not found in database.
     */
    public MultiPk retrieveByPK(ObjectKey pk)
        throws TorqueException, NoRowsException, TooManyRowsException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(MultiPkPeer.DATABASE_NAME);
            MultiPk result = retrieveByPK(pk, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Retrieve a single object by pk
     *
     * @param pk the primary key
     * @param con the connection to use
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     * @throws NoRowsException Primary key was not found in database.
     * @throws TooManyRowsException Primary key was not found in database.
     */
    public MultiPk retrieveByPK(ObjectKey pk, Connection con)
        throws TorqueException, NoRowsException, TooManyRowsException
    {
        Criteria criteria = buildCriteria(pk);
        List<MultiPk> v = doSelect(criteria, con);
        if (v.size() == 0)
        {
            throw new NoRowsException("Failed to select a row.");
        }
        else if (v.size() > 1)
        {
            throw new TooManyRowsException("Failed to select only one row.");
        }
        else
        {
            return (MultiPk)v.get(0);
        }
    }


    /**
     * Retrieve a multiple objects by pk
     *
     * @param pks List of primary keys
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<MultiPk> retrieveByPKs(Collection<ObjectKey> pks)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(MultiPkPeer.DATABASE_NAME);
            List<MultiPk> result = retrieveByPKs(pks, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Retrieve multiple objects by pk
     *
     * @param pks List of primary keys
     * @param dbcon the connection to use
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<MultiPk> retrieveByPKs(
                Collection<ObjectKey> pks,
                Connection dbcon)
            throws TorqueException
    {
        if (pks == null || pks.size() == 0)
        {
            return new ArrayList<MultiPk>();
        }
        Criteria criteria = buildCriteria(pks);
        List<MultiPk> result = doSelect(criteria, dbcon);
        return result;
    }






    /** 
     * Returns the chunk size for the filler methods.
     *
     * @return the chunk size for the filler methods.
     */
    protected int getFillerChunkSize()
    {
        return 999;
    }


}
