package org.apache.torque.test;

import java.sql.Blob;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.Types;
import java.io.IOException;
import java.io.InputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.Reader;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.torque.NoRowsException;
import org.apache.torque.TooManyRowsException;
import org.apache.torque.Torque;
import org.apache.torque.TorqueException;
import org.apache.torque.TorqueRuntimeException;
import org.apache.torque.criteria.Criteria;
import org.apache.torque.criteria.Criterion;
import org.apache.torque.om.mapper.RecordMapper;
import org.apache.torque.om.mapper.CompositeMapper;
import org.apache.torque.om.DateKey;
import org.apache.torque.om.NumberKey;
import org.apache.torque.om.StringKey;
import org.apache.torque.om.ObjectKey;
import org.apache.torque.om.SimpleKey;
import org.apache.torque.om.ComboKey;
import org.apache.torque.map.TableMap;
import org.apache.torque.util.BasePeer;
import org.apache.torque.util.Transaction;
import org.apache.torque.util.ColumnValues;
import org.apache.torque.util.JdbcTypedValue;



/**
 * Table to test multi references to a same table
 *
 * The skeleton for this class was autogenerated by Torque on:
 *
 * [Thu Sep 13 04:35:46 CEST 2012]
 *
 * You should not use this class directly.  It should not even be
 * extended; all references should be to MultiRefSameTablePeer
 */

public abstract class BaseMultiRefSameTablePeerImpl
    extends org.apache.torque.util.BasePeerImpl<MultiRefSameTable>
{
    /** The class log. */
    private static Log log = LogFactory.getLog(BaseMultiRefSameTablePeerImpl.class);

    /** Serial version */
    private static final long serialVersionUID = 1347503746705L;



    /**
     * Constructor.
     * The recordMapper, tableMap and databaseName fields are correctly
     * initialized.
     */
    public BaseMultiRefSameTablePeerImpl()
    {
        this(new MultiRefSameTableRecordMapper(),
            MultiRefSameTablePeer.TABLE,
            MultiRefSameTablePeer.DATABASE_NAME);
    }

    /**
     * Constructor providing the objects to be injected as parameters.
     *
     * @param recordMapper a record mapper to map JDBC result sets to objects
     * @param tableMap the default table map
     * @param databaseName the name of the database
     */
    public BaseMultiRefSameTablePeerImpl(
            RecordMapper<MultiRefSameTable> recordMapper, 
            TableMap tableMap,
            String databaseName)
    {
        super(recordMapper, tableMap, databaseName);
    }


    /**
     * Selects MultiRefSameTable objects from the database which have
     * the same content as the passed object.
     *
     * @return The list of selected objects, not null.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<MultiRefSameTable> doSelect(MultiRefSameTable obj)
            throws TorqueException
    {
        return doSelect(buildSelectCriteria(obj));
    }

    /**
     * Selects at most one MultiRefSameTable object from the database
     * which has the same content as the passed object.
     *
     * @return the selected Object, or null if no object was selected.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public MultiRefSameTable doSelectSingleRecord(
                MultiRefSameTable obj)
            throws TorqueException
    {
        List<MultiRefSameTable> multiRefSameTableList = doSelect(obj);
        MultiRefSameTable multiRefSameTable = null;
        if (multiRefSameTableList.size() > 1)
        {
            throw new TooManyRowsException("Object " + obj 
                + " matched more than one record");
        }
        if (!multiRefSameTableList.isEmpty())
        {
            multiRefSameTable = multiRefSameTableList.get(0);
        }
        return multiRefSameTable;
    }

    /**
     * Returns a new instance of the Data object class
     */
    public MultiRefSameTable getDbObjectInstance()
    {
        return new MultiRefSameTable();
    }


    /**
     * Method to do inserts.
     *
     * @param columnValues the values to insert.
     *
     * @return the primary key of the inserted row.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public ObjectKey doInsert(ColumnValues columnValues) throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    MultiRefSameTablePeer.DATABASE_NAME);
            ObjectKey result = doInsert(columnValues, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Method to do inserts.  This method is to be used during a transaction,
     * otherwise use the doInsert(Criteria) method.
     *
     * @param columnValues the values to insert.
     * @param con the connection to use, not null.
     *
     * @return the primary key of the inserted row.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public ObjectKey doInsert(ColumnValues columnValues, Connection con)
        throws TorqueException
    {
        correctBooleans(columnValues);
        return super.doInsert(columnValues, con);
    }

    /**
     * Method to do inserts
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public void doInsert(MultiRefSameTable obj) throws TorqueException
    {
        doInsert(buildColumnValues(obj));
        obj.setNew(false);
        obj.setModified(false);
    }

    /**
     * Method to do inserts.  This method is to be used during a transaction,
     * otherwise use the doInsert(MultiRefSameTable) method.  It will take
     * care of the connection details internally.
     *
     * @param obj the data object to insert into the database.
     * @param con the connection to use
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public void doInsert(MultiRefSameTable obj, Connection con)
        throws TorqueException
    {
        doInsert(buildColumnValues(obj), con);
        obj.setNew(false);
        obj.setModified(false);
    }

    /**
     * Method to do updates.
     *
     * @param columnValues the values to update plus the primary key
     *        identifying the row to update.
     *
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(ColumnValues columnValues) throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    MultiRefSameTablePeer.DATABASE_NAME);
            int result = doUpdate(columnValues, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Method to do updates.  This method is to be used during a transaction,
     * otherwise use the doUpdate(Criteria) method.
     *
     * @param columnValues the values to update plus the primary key
     *        identifying the row to update.
     * @param con the connection to use, not null.
     *
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(ColumnValues columnValues, Connection con)
        throws TorqueException
    {
        Criteria selectCriteria 
                = new Criteria(MultiRefSameTablePeer.DATABASE_NAME);
        correctBooleans(columnValues);


    
        selectCriteria.where(
                MultiRefSameTablePeer.MULTI_REF_SAME_TABLE_PK,
                columnValues.remove(MultiRefSameTablePeer.MULTI_REF_SAME_TABLE_PK).getValue());




        return doUpdate(selectCriteria, columnValues, con);
    }

    /**
     * Updates an MultiRefSameTable in the database.
     * The primary key is used to identify the object to update.
     *
     * @param obj the data object to update in the database.
     *
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(MultiRefSameTable obj) throws TorqueException
    {
        ColumnValues columnValues = buildColumnValues(obj);
        int result = doUpdate(columnValues);
        obj.setModified(false);
        return result;
    }

    /**
     * Updates a MultiRefSameTable in the database.
     * The primary key is used to identify the object to update.
     * This method is to be used during a transaction,
     * otherwise the doUpdate(MultiRefSameTable) method can be used.
     *
     * @param obj the data object to update in the database.
     * @param con the connection to use, not null.
     
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(MultiRefSameTable obj, Connection con)
        throws TorqueException
    {
        ColumnValues columnValues = buildColumnValues(obj);
        int result = doUpdate(columnValues, con);
        obj.setModified(false);
        return result;
    }

    /**
     * Deletes a data object, i.e. a row in a table, in the database.
     *
     * @param obj the data object to delete in the database, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(MultiRefSameTable obj) throws TorqueException
    {
        int result = doDelete(buildCriteria(obj.getPrimaryKey()));
        obj.setDeleted(true);
        return result;
    }

    /**
     * Deletes a data object, i.e. a row in a table, in the database.
     * This method is to be used during a transaction, otherwise use the
     * doDelete(MultiRefSameTable) method.
     *
     * @param obj the data object to delete in the database, not null.
     * @param con the connection to use, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(MultiRefSameTable obj, Connection con)
        throws TorqueException
    {
        int result = doDelete(buildCriteria(obj.getPrimaryKey()), con);
        obj.setDeleted(true);
        return result;
    }

    /**
     * Deletes data objects, i.e. rows in a table, in the database.
     *
     * @param objects the data object to delete in the database, not null,
     *        may not contain null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(Collection<MultiRefSameTable> objects)
            throws TorqueException
    {
        int result = doDelete(buildPkCriteria(objects));
        for (MultiRefSameTable object : objects)
        {
            object.setDeleted(true);
        }
        return result;
    }

    /**
     * Deletes data objects, i.e. rows in a table, in the database.
     * This method uses the passed connection to delete the rows;
     * if a transaction is open in the connection, the deletion happens inside
     * this transaction.
     *
     * @param objects the data objects to delete in the database, not null,
     *        may not contain null.
     * @param con the connection to use for deleting, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(
            Collection<MultiRefSameTable> objects,
            Connection con)
        throws TorqueException
    {
        int result = doDelete(buildPkCriteria(objects), con);
        for (MultiRefSameTable object : objects)
        {
            object.setDeleted(true);
        }
        return result;
    }

    /**
     * Deletes a row in the database.
     *
     * @param pk the ObjectKey that identifies the row to delete.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(ObjectKey pk) throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    MultiRefSameTablePeer.DATABASE_NAME);
            int deletedRows = doDelete(pk, connection);
            Transaction.commit(connection);
            connection = null;
            return deletedRows;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Deletes a row in the database.
     * This method is to be used during a transaction,
     * otherwise use the doDelete(ObjectKey) method.
     *
     * @param pk the ObjectKey that identifies the row to delete.
     * @param con the connection to use for deleting, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(ObjectKey pk, Connection con)
        throws TorqueException
    {
        return doDelete(buildCriteria(pk), con);
    }

    /** 
     * Build a Criteria object which selects all objects which have a given
     * primary key.
     *
     * @param pk the primary key value to build the criteria from, not null.
     */
    public Criteria buildCriteria(ObjectKey pk)
    {
        Criteria criteria = new Criteria();
        criteria.and(MultiRefSameTablePeer.MULTI_REF_SAME_TABLE_PK, pk);
        return criteria;
     }

    /** 
     * Build a Criteria object which selects all objects which primary keys
     * are contained in the passed collection.
     *
     * @param pks the primary key values to build the criteria from, not null,
     *        may not contain null.
     */
    public Criteria buildCriteria(Collection<ObjectKey> pks)
    {
        Criteria criteria = new Criteria();
        criteria.andIn(MultiRefSameTablePeer.MULTI_REF_SAME_TABLE_PK, pks);
        return criteria;
     }


    /** 
     * Build a Criteria object which selects all passed objects using their
     * primary key. Objects which do not yet have a primary key are ignored.
     *
     * @param objects the objects to build the criteria from, not null,
     *        may not contain null.
     */
    public Criteria buildPkCriteria(
            Collection<MultiRefSameTable> objects)
    {
        List<ObjectKey> pks = new ArrayList<ObjectKey>(objects.size());
        for (MultiRefSameTable object : objects)
        {
            ObjectKey pk = object.getPrimaryKey();
            if (pk != null)
            {
                pks.add(pk);
            }
        }
        return buildCriteria(pks);
    }

    /** 
     * Build a Criteria object from the data object for this peer.
     *
     * @param obj the object to build the criteria from, not null.
     */
    public Criteria buildCriteria(MultiRefSameTable obj)
    {
        Criteria criteria = new Criteria(MultiRefSameTablePeer.DATABASE_NAME);
        criteria.and(MultiRefSameTablePeer.MULTI_REF_SAME_TABLE_PK, obj.getMultiRefSameTablePk());
        criteria.and(MultiRefSameTablePeer.AN_INFORMATION, obj.getAnInformation());
        criteria.and(MultiRefSameTablePeer.REFERENCE_1, obj.getReference1());
        criteria.and(MultiRefSameTablePeer.REFERENCE_2, obj.getReference2());
        criteria.and(MultiRefSameTablePeer.REFERENCE_3, obj.getReference3());
        return criteria;
    }

    /** 
     * Build a Criteria object from the data object for this peer,
     * skipping all binary columns.
     *
     * @param obj the object to build the criteria from, not null.
     */
    public Criteria buildSelectCriteria(MultiRefSameTable obj)
    {
        Criteria criteria = new Criteria(MultiRefSameTablePeer.DATABASE_NAME);
        criteria.and(MultiRefSameTablePeer.MULTI_REF_SAME_TABLE_PK, obj.getMultiRefSameTablePk());
        criteria.and(MultiRefSameTablePeer.AN_INFORMATION, obj.getAnInformation());
        criteria.and(MultiRefSameTablePeer.REFERENCE_1, obj.getReference1());
        criteria.and(MultiRefSameTablePeer.REFERENCE_2, obj.getReference2());
        criteria.and(MultiRefSameTablePeer.REFERENCE_3, obj.getReference3());
        return criteria;
    }

    /** 
     * Returns the contents of the object as ColumnValues object.
     * Primary key columns which are generated on insertion are not
     * added to the returned object if they still have their initial
     * value. Also, columns which have the useDatabaseDefaultValue
     * flag set to true are also not added to the returned object
     * if they still have their initial value.
     *
     * @throws TorqueException if the table map cannot be retrieved
     *         (should not happen).
     */
    public ColumnValues buildColumnValues(MultiRefSameTable multiRefSameTable)
            throws TorqueException
    {
        ColumnValues columnValues = new ColumnValues();
        columnValues.put(
                MultiRefSameTablePeer.MULTI_REF_SAME_TABLE_PK,
                new JdbcTypedValue(
                    multiRefSameTable.getMultiRefSameTablePk(),
                    12));
        columnValues.put(
                MultiRefSameTablePeer.AN_INFORMATION,
                new JdbcTypedValue(
                    multiRefSameTable.getAnInformation(),
                    12));
        columnValues.put(
                MultiRefSameTablePeer.REFERENCE_1,
                new JdbcTypedValue(
                    multiRefSameTable.getReference1(),
                    12));
        columnValues.put(
                MultiRefSameTablePeer.REFERENCE_2,
                new JdbcTypedValue(
                    multiRefSameTable.getReference2(),
                    12));
        columnValues.put(
                MultiRefSameTablePeer.REFERENCE_3,
                new JdbcTypedValue(
                    multiRefSameTable.getReference3(),
                    12));
        return columnValues;
    }

    /**
     * Retrieve a single object by pk
     *
     * @param pk the primary key
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     * @throws NoRowsException Primary key was not found in database.
     * @throws TooManyRowsException Primary key was not found in database.
     */
    public MultiRefSameTable retrieveByPK(String pk)
        throws TorqueException, NoRowsException, TooManyRowsException
    {
        return retrieveByPK(SimpleKey.keyFor(pk));
    }

    /**
     * Retrieve a single object by pk
     *
     * @param pk the primary key
     * @param con the connection to use
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     * @throws NoRowsException Primary key was not found in database.
     * @throws TooManyRowsException Primary key was not found in database.
     */
    public MultiRefSameTable retrieveByPK(String pk, Connection con)
        throws TorqueException, NoRowsException, TooManyRowsException
    {
        return retrieveByPK(SimpleKey.keyFor(pk), con);
    }
    
    
    

    /**
     * Retrieve a single object by pk
     *
     * @param pk the primary key
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     * @throws NoRowsException Primary key was not found in database.
     * @throws TooManyRowsException Primary key was not found in database.
     */
    public MultiRefSameTable retrieveByPK(ObjectKey pk)
        throws TorqueException, NoRowsException, TooManyRowsException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(MultiRefSameTablePeer.DATABASE_NAME);
            MultiRefSameTable result = retrieveByPK(pk, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Retrieve a single object by pk
     *
     * @param pk the primary key
     * @param con the connection to use
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     * @throws NoRowsException Primary key was not found in database.
     * @throws TooManyRowsException Primary key was not found in database.
     */
    public MultiRefSameTable retrieveByPK(ObjectKey pk, Connection con)
        throws TorqueException, NoRowsException, TooManyRowsException
    {
        Criteria criteria = buildCriteria(pk);
        List<MultiRefSameTable> v = doSelect(criteria, con);
        if (v.size() == 0)
        {
            throw new NoRowsException("Failed to select a row.");
        }
        else if (v.size() > 1)
        {
            throw new TooManyRowsException("Failed to select only one row.");
        }
        else
        {
            return (MultiRefSameTable)v.get(0);
        }
    }


    /**
     * Retrieve a multiple objects by pk
     *
     * @param pks List of primary keys
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<MultiRefSameTable> retrieveByPKs(Collection<ObjectKey> pks)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(MultiRefSameTablePeer.DATABASE_NAME);
            List<MultiRefSameTable> result = retrieveByPKs(pks, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Retrieve multiple objects by pk
     *
     * @param pks List of primary keys
     * @param dbcon the connection to use
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<MultiRefSameTable> retrieveByPKs(
                Collection<ObjectKey> pks,
                Connection dbcon)
            throws TorqueException
    {
        if (pks == null || pks.size() == 0)
        {
            return new ArrayList<MultiRefSameTable>();
        }
        Criteria criteria = buildCriteria(pks);
        List<MultiRefSameTable> result = doSelect(criteria, dbcon);
        return result;
    }


    /**
     * Selects a collection of dbObjectClassName objects pre-filled with their
     * Referenced objects.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<MultiRefSameTable> doSelectJoinReferencedRelatedByReference1(Criteria criteria)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    MultiRefSameTablePeer.DATABASE_NAME);
            List<MultiRefSameTable> result
                    = MultiRefSameTablePeer.doSelectJoinReferencedRelatedByReference1(criteria, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Selects a collection of MultiRefSameTable objects pre-filled with their
     * Referenced objects.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<MultiRefSameTable> doSelectJoinReferencedRelatedByReference1(
            Criteria criteria, Connection conn)
        throws TorqueException
    {
        setDbName(criteria);

        addSelectColumns(criteria);
        org.apache.torque.test.ReferencedPeer.addSelectColumns(criteria);

        CompositeMapper compositeMapper = new CompositeMapper();
        compositeMapper.addMapper(
                new MultiRefSameTableRecordMapper(),
                0);
        compositeMapper.addMapper(
                new ReferencedRecordMapper(),
                MultiRefSameTablePeer.numColumns);
        criteria.addJoin(MultiRefSameTablePeer.REFERENCE_1,
            org.apache.torque.test.ReferencedPeer.REFERENCED_PK);

        correctBooleans(criteria);

        List<MultiRefSameTable> result = new ArrayList<MultiRefSameTable>();
        List<List<Object>> rawResult = doSelect(
                criteria, compositeMapper, conn);
        for (List<Object> rawResultRow : rawResult)
        {
            MultiRefSameTable obj1 = (MultiRefSameTable) rawResultRow.get(0);
            Referenced obj2 = (Referenced) rawResultRow.get(1);

            boolean newObject = true;
            for (int j = 0; j < result.size(); j++)
            {
                MultiRefSameTable temp_obj1 = result.get(j);
                org.apache.torque.test.Referenced temp_obj2 = temp_obj1.getReferencedRelatedByReference1();
                if (temp_obj2.getPrimaryKey().equals(obj2.getPrimaryKey()))
                {
                    newObject = false;
                    temp_obj2.addMultiRefSameTableRelatedByReference1(obj1);
                    break;
                }
            }
            if (newObject)
            {
                obj2.initMultiRefSameTableRelatedByReference1s();
                obj2.addMultiRefSameTableRelatedByReference1(obj1);
            }
            result.add(obj1);
        }
        return result;
    }
    /**
     * Selects a collection of dbObjectClassName objects pre-filled with their
     * Referenced objects.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<MultiRefSameTable> doSelectJoinReferencedRelatedByReference2(Criteria criteria)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    MultiRefSameTablePeer.DATABASE_NAME);
            List<MultiRefSameTable> result
                    = MultiRefSameTablePeer.doSelectJoinReferencedRelatedByReference2(criteria, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Selects a collection of MultiRefSameTable objects pre-filled with their
     * Referenced objects.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<MultiRefSameTable> doSelectJoinReferencedRelatedByReference2(
            Criteria criteria, Connection conn)
        throws TorqueException
    {
        setDbName(criteria);

        addSelectColumns(criteria);
        org.apache.torque.test.ReferencedPeer.addSelectColumns(criteria);

        CompositeMapper compositeMapper = new CompositeMapper();
        compositeMapper.addMapper(
                new MultiRefSameTableRecordMapper(),
                0);
        compositeMapper.addMapper(
                new ReferencedRecordMapper(),
                MultiRefSameTablePeer.numColumns);
        criteria.addJoin(MultiRefSameTablePeer.REFERENCE_2,
            org.apache.torque.test.ReferencedPeer.REFERENCED_PK);

        correctBooleans(criteria);

        List<MultiRefSameTable> result = new ArrayList<MultiRefSameTable>();
        List<List<Object>> rawResult = doSelect(
                criteria, compositeMapper, conn);
        for (List<Object> rawResultRow : rawResult)
        {
            MultiRefSameTable obj1 = (MultiRefSameTable) rawResultRow.get(0);
            Referenced obj2 = (Referenced) rawResultRow.get(1);

            boolean newObject = true;
            for (int j = 0; j < result.size(); j++)
            {
                MultiRefSameTable temp_obj1 = result.get(j);
                org.apache.torque.test.Referenced temp_obj2 = temp_obj1.getReferencedRelatedByReference2();
                if (temp_obj2.getPrimaryKey().equals(obj2.getPrimaryKey()))
                {
                    newObject = false;
                    temp_obj2.addMultiRefSameTableRelatedByReference2(obj1);
                    break;
                }
            }
            if (newObject)
            {
                obj2.initMultiRefSameTableRelatedByReference2s();
                obj2.addMultiRefSameTableRelatedByReference2(obj1);
            }
            result.add(obj1);
        }
        return result;
    }
    /**
     * Selects a collection of dbObjectClassName objects pre-filled with their
     * Referenced objects.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<MultiRefSameTable> doSelectJoinReferencedRelatedByReference3(Criteria criteria)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    MultiRefSameTablePeer.DATABASE_NAME);
            List<MultiRefSameTable> result
                    = MultiRefSameTablePeer.doSelectJoinReferencedRelatedByReference3(criteria, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Selects a collection of MultiRefSameTable objects pre-filled with their
     * Referenced objects.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<MultiRefSameTable> doSelectJoinReferencedRelatedByReference3(
            Criteria criteria, Connection conn)
        throws TorqueException
    {
        setDbName(criteria);

        addSelectColumns(criteria);
        org.apache.torque.test.ReferencedPeer.addSelectColumns(criteria);

        CompositeMapper compositeMapper = new CompositeMapper();
        compositeMapper.addMapper(
                new MultiRefSameTableRecordMapper(),
                0);
        compositeMapper.addMapper(
                new ReferencedRecordMapper(),
                MultiRefSameTablePeer.numColumns);
        criteria.addJoin(MultiRefSameTablePeer.REFERENCE_3,
            org.apache.torque.test.ReferencedPeer.REFERENCED_PK);

        correctBooleans(criteria);

        List<MultiRefSameTable> result = new ArrayList<MultiRefSameTable>();
        List<List<Object>> rawResult = doSelect(
                criteria, compositeMapper, conn);
        for (List<Object> rawResultRow : rawResult)
        {
            MultiRefSameTable obj1 = (MultiRefSameTable) rawResultRow.get(0);
            Referenced obj2 = (Referenced) rawResultRow.get(1);

            boolean newObject = true;
            for (int j = 0; j < result.size(); j++)
            {
                MultiRefSameTable temp_obj1 = result.get(j);
                org.apache.torque.test.Referenced temp_obj2 = temp_obj1.getReferencedRelatedByReference3();
                if (temp_obj2.getPrimaryKey().equals(obj2.getPrimaryKey()))
                {
                    newObject = false;
                    temp_obj2.addMultiRefSameTableRelatedByReference3(obj1);
                    break;
                }
            }
            if (newObject)
            {
                obj2.initMultiRefSameTableRelatedByReference3s();
                obj2.addMultiRefSameTableRelatedByReference3(obj1);
            }
            result.add(obj1);
        }
        return result;
    }

    /**
     * selects a collection of MultiRefSameTable objects pre-filled with
     * all related objects.
     *
     * This method is protected by default in order to keep the public
     * api reasonable.  You can provide public methods for those you
     * actually need in MultiRefSameTablePeerImpl.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    protected List<MultiRefSameTable> doSelectJoinAllExceptReferencedRelatedByReference1(Criteria criteria)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    MultiRefSameTablePeer.DATABASE_NAME);
            List<MultiRefSameTable> result
                    = MultiRefSameTablePeer.doSelectJoinAllExceptReferencedRelatedByReference1(criteria, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * selects a collection of MultiRefSameTable objects pre-filled with
     * related objects.
     *
     * This method is protected by default in order to keep the public
     * api reasonable.  You can provide public methods for those you
     * actually need in MultiRefSameTablePeerImpl.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    protected List<MultiRefSameTable> doSelectJoinAllExceptReferencedRelatedByReference1(Criteria criteria, Connection conn)
        throws TorqueException
    {
        setDbName(criteria);

        CompositeMapper compositeMapper = new CompositeMapper();
        compositeMapper.addMapper(new MultiRefSameTableRecordMapper(), 0);
        MultiRefSameTablePeer.addSelectColumns(criteria);
        int offset = MultiRefSameTablePeer.numColumns;
 
        org.apache.torque.test.ReferencedPeer.addSelectColumns(criteria);
        criteria.addJoin(MultiRefSameTablePeer.REFERENCE_2, org.apache.torque.test.ReferencedPeer.REFERENCED_PK);
        compositeMapper.addMapper(new ReferencedRecordMapper(), offset);
        offset = offset + org.apache.torque.test.ReferencedPeer.numColumns;
 
        org.apache.torque.test.ReferencedPeer.addSelectColumns(criteria);
        criteria.addJoin(MultiRefSameTablePeer.REFERENCE_3, org.apache.torque.test.ReferencedPeer.REFERENCED_PK);
        compositeMapper.addMapper(new ReferencedRecordMapper(), offset);
        offset = offset + org.apache.torque.test.ReferencedPeer.numColumns;

        correctBooleans(criteria);

        List<MultiRefSameTable> result = new ArrayList<MultiRefSameTable>();
        List<List<Object>> rawResult = doSelect(
                criteria, compositeMapper, conn);
        for (List<Object> rawResultRow : rawResult)
        {
            MultiRefSameTable multiRefSameTable = (MultiRefSameTable) rawResultRow.get(0);

            org.apache.torque.test.Referenced obj1 = (org.apache.torque.test.Referenced) rawResultRow.get(1);
            boolean newObject = true;
            for (int j = 0; j < result.size(); j++)
            {
                org.apache.torque.test.MultiRefSameTable temp_multiRefSameTable = result.get(j);
                org.apache.torque.test.Referenced temp_obj1 = temp_multiRefSameTable.getReferencedRelatedByReference2();
                if (temp_obj1.getPrimaryKey().equals(obj1.getPrimaryKey()))
                {
                    newObject = false;
                    temp_obj1.addMultiRefSameTableRelatedByReference2(multiRefSameTable);
                    break;
                }
            }
            if (newObject)
            {
                obj1.initMultiRefSameTableRelatedByReference2s();
                obj1.addMultiRefSameTableRelatedByReference2(multiRefSameTable);
            }

            org.apache.torque.test.Referenced obj2 = (org.apache.torque.test.Referenced) rawResultRow.get(2);
            newObject = true;
            for (int j = 0; j < result.size(); j++)
            {
                org.apache.torque.test.MultiRefSameTable temp_multiRefSameTable = result.get(j);
                org.apache.torque.test.Referenced temp_obj2 = temp_multiRefSameTable.getReferencedRelatedByReference3();
                if (temp_obj2.getPrimaryKey().equals(obj2.getPrimaryKey()))
                {
                    newObject = false;
                    temp_obj2.addMultiRefSameTableRelatedByReference3(multiRefSameTable);
                    break;
                }
            }
            if (newObject)
            {
                obj2.initMultiRefSameTableRelatedByReference3s();
                obj2.addMultiRefSameTableRelatedByReference3(multiRefSameTable);
            }
            result.add(multiRefSameTable);
        }
        return result;
    }
    /**
     * selects a collection of MultiRefSameTable objects pre-filled with
     * all related objects.
     *
     * This method is protected by default in order to keep the public
     * api reasonable.  You can provide public methods for those you
     * actually need in MultiRefSameTablePeerImpl.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    protected List<MultiRefSameTable> doSelectJoinAllExceptReferencedRelatedByReference2(Criteria criteria)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    MultiRefSameTablePeer.DATABASE_NAME);
            List<MultiRefSameTable> result
                    = MultiRefSameTablePeer.doSelectJoinAllExceptReferencedRelatedByReference2(criteria, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * selects a collection of MultiRefSameTable objects pre-filled with
     * related objects.
     *
     * This method is protected by default in order to keep the public
     * api reasonable.  You can provide public methods for those you
     * actually need in MultiRefSameTablePeerImpl.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    protected List<MultiRefSameTable> doSelectJoinAllExceptReferencedRelatedByReference2(Criteria criteria, Connection conn)
        throws TorqueException
    {
        setDbName(criteria);

        CompositeMapper compositeMapper = new CompositeMapper();
        compositeMapper.addMapper(new MultiRefSameTableRecordMapper(), 0);
        MultiRefSameTablePeer.addSelectColumns(criteria);
        int offset = MultiRefSameTablePeer.numColumns;
 
        org.apache.torque.test.ReferencedPeer.addSelectColumns(criteria);
        criteria.addJoin(MultiRefSameTablePeer.REFERENCE_1, org.apache.torque.test.ReferencedPeer.REFERENCED_PK);
        compositeMapper.addMapper(new ReferencedRecordMapper(), offset);
        offset = offset + org.apache.torque.test.ReferencedPeer.numColumns;
 
        org.apache.torque.test.ReferencedPeer.addSelectColumns(criteria);
        criteria.addJoin(MultiRefSameTablePeer.REFERENCE_3, org.apache.torque.test.ReferencedPeer.REFERENCED_PK);
        compositeMapper.addMapper(new ReferencedRecordMapper(), offset);
        offset = offset + org.apache.torque.test.ReferencedPeer.numColumns;

        correctBooleans(criteria);

        List<MultiRefSameTable> result = new ArrayList<MultiRefSameTable>();
        List<List<Object>> rawResult = doSelect(
                criteria, compositeMapper, conn);
        for (List<Object> rawResultRow : rawResult)
        {
            MultiRefSameTable multiRefSameTable = (MultiRefSameTable) rawResultRow.get(0);

            org.apache.torque.test.Referenced obj1 = (org.apache.torque.test.Referenced) rawResultRow.get(1);
            boolean newObject = true;
            for (int j = 0; j < result.size(); j++)
            {
                org.apache.torque.test.MultiRefSameTable temp_multiRefSameTable = result.get(j);
                org.apache.torque.test.Referenced temp_obj1 = temp_multiRefSameTable.getReferencedRelatedByReference1();
                if (temp_obj1.getPrimaryKey().equals(obj1.getPrimaryKey()))
                {
                    newObject = false;
                    temp_obj1.addMultiRefSameTableRelatedByReference1(multiRefSameTable);
                    break;
                }
            }
            if (newObject)
            {
                obj1.initMultiRefSameTableRelatedByReference1s();
                obj1.addMultiRefSameTableRelatedByReference1(multiRefSameTable);
            }

            org.apache.torque.test.Referenced obj2 = (org.apache.torque.test.Referenced) rawResultRow.get(2);
            newObject = true;
            for (int j = 0; j < result.size(); j++)
            {
                org.apache.torque.test.MultiRefSameTable temp_multiRefSameTable = result.get(j);
                org.apache.torque.test.Referenced temp_obj2 = temp_multiRefSameTable.getReferencedRelatedByReference3();
                if (temp_obj2.getPrimaryKey().equals(obj2.getPrimaryKey()))
                {
                    newObject = false;
                    temp_obj2.addMultiRefSameTableRelatedByReference3(multiRefSameTable);
                    break;
                }
            }
            if (newObject)
            {
                obj2.initMultiRefSameTableRelatedByReference3s();
                obj2.addMultiRefSameTableRelatedByReference3(multiRefSameTable);
            }
            result.add(multiRefSameTable);
        }
        return result;
    }
    /**
     * selects a collection of MultiRefSameTable objects pre-filled with
     * all related objects.
     *
     * This method is protected by default in order to keep the public
     * api reasonable.  You can provide public methods for those you
     * actually need in MultiRefSameTablePeerImpl.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    protected List<MultiRefSameTable> doSelectJoinAllExceptReferencedRelatedByReference3(Criteria criteria)
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    MultiRefSameTablePeer.DATABASE_NAME);
            List<MultiRefSameTable> result
                    = MultiRefSameTablePeer.doSelectJoinAllExceptReferencedRelatedByReference3(criteria, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * selects a collection of MultiRefSameTable objects pre-filled with
     * related objects.
     *
     * This method is protected by default in order to keep the public
     * api reasonable.  You can provide public methods for those you
     * actually need in MultiRefSameTablePeerImpl.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    protected List<MultiRefSameTable> doSelectJoinAllExceptReferencedRelatedByReference3(Criteria criteria, Connection conn)
        throws TorqueException
    {
        setDbName(criteria);

        CompositeMapper compositeMapper = new CompositeMapper();
        compositeMapper.addMapper(new MultiRefSameTableRecordMapper(), 0);
        MultiRefSameTablePeer.addSelectColumns(criteria);
        int offset = MultiRefSameTablePeer.numColumns;
 
        org.apache.torque.test.ReferencedPeer.addSelectColumns(criteria);
        criteria.addJoin(MultiRefSameTablePeer.REFERENCE_1, org.apache.torque.test.ReferencedPeer.REFERENCED_PK);
        compositeMapper.addMapper(new ReferencedRecordMapper(), offset);
        offset = offset + org.apache.torque.test.ReferencedPeer.numColumns;
 
        org.apache.torque.test.ReferencedPeer.addSelectColumns(criteria);
        criteria.addJoin(MultiRefSameTablePeer.REFERENCE_2, org.apache.torque.test.ReferencedPeer.REFERENCED_PK);
        compositeMapper.addMapper(new ReferencedRecordMapper(), offset);
        offset = offset + org.apache.torque.test.ReferencedPeer.numColumns;

        correctBooleans(criteria);

        List<MultiRefSameTable> result = new ArrayList<MultiRefSameTable>();
        List<List<Object>> rawResult = doSelect(
                criteria, compositeMapper, conn);
        for (List<Object> rawResultRow : rawResult)
        {
            MultiRefSameTable multiRefSameTable = (MultiRefSameTable) rawResultRow.get(0);

            org.apache.torque.test.Referenced obj1 = (org.apache.torque.test.Referenced) rawResultRow.get(1);
            boolean newObject = true;
            for (int j = 0; j < result.size(); j++)
            {
                org.apache.torque.test.MultiRefSameTable temp_multiRefSameTable = result.get(j);
                org.apache.torque.test.Referenced temp_obj1 = temp_multiRefSameTable.getReferencedRelatedByReference1();
                if (temp_obj1.getPrimaryKey().equals(obj1.getPrimaryKey()))
                {
                    newObject = false;
                    temp_obj1.addMultiRefSameTableRelatedByReference1(multiRefSameTable);
                    break;
                }
            }
            if (newObject)
            {
                obj1.initMultiRefSameTableRelatedByReference1s();
                obj1.addMultiRefSameTableRelatedByReference1(multiRefSameTable);
            }

            org.apache.torque.test.Referenced obj2 = (org.apache.torque.test.Referenced) rawResultRow.get(2);
            newObject = true;
            for (int j = 0; j < result.size(); j++)
            {
                org.apache.torque.test.MultiRefSameTable temp_multiRefSameTable = result.get(j);
                org.apache.torque.test.Referenced temp_obj2 = temp_multiRefSameTable.getReferencedRelatedByReference2();
                if (temp_obj2.getPrimaryKey().equals(obj2.getPrimaryKey()))
                {
                    newObject = false;
                    temp_obj2.addMultiRefSameTableRelatedByReference2(multiRefSameTable);
                    break;
                }
            }
            if (newObject)
            {
                obj2.initMultiRefSameTableRelatedByReference2s();
                obj2.addMultiRefSameTableRelatedByReference2(multiRefSameTable);
            }
            result.add(multiRefSameTable);
        }
        return result;
    }

    /** 
     * Fills the referenced Referenced objects in the aReferencedRelatedByReference1 fields
     * of the objects in <code>toFill</code> by reading the database.
     *
     * @param toFill Contains the objects to fill, not null,
     *        may not contain null.
     *
     * @return all read Referenced objects.
     *
     * @throws TorqueException if an error querying the database occurs.
     * @throws NullPointerException if toFill is null or contains null elements.
     */
    public List<Referenced> fillReferencedRelatedByReference1s(
            Collection<MultiRefSameTable> toFill) 
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    MultiRefSameTablePeer.DATABASE_NAME);
            List<Referenced> result = fillReferencedRelatedByReference1s(toFill, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /** 
     * Fills the referenced Referenced objects in the aReferencedRelatedByReference1 fields
     * of the objects in <code>toFill</code> by reading the database.
     *
     * @param toFill Contains the objects to fill, not null,
     *        may not contain null.
     * @param connection the database connection to use, not null.
     *
     * @return all read Referenced objects.
     *
     * @throws TorqueException if an error querying the database occurs.
     * @throws NullPointerException if toFill is null or contains null elements.
     */
    public List<Referenced> fillReferencedRelatedByReference1s(
            Collection<MultiRefSameTable> toFill,
            Connection connection) 
        throws TorqueException
    {
        Set<ObjectKey> foreignKeySet = new HashSet<ObjectKey>();
        for (MultiRefSameTable currentMultiRefSameTable : toFill)
        {
            ObjectKey foreignKey = currentMultiRefSameTable.getForeignKeyForReferencedRelatedByReference1();
            if (foreignKey != null)
            {
                foreignKeySet.add(foreignKey);
            }
        }

        int fillerChunkSize = getFillerChunkSize();
        Iterator<ObjectKey> foreignKeyIt = foreignKeySet.iterator();
        List<ObjectKey> foreignKeyListChunk
                = new ArrayList<ObjectKey>(fillerChunkSize);;
        Map<ObjectKey, Referenced> referencedKeyToReferencedMap 
                = new HashMap<ObjectKey, Referenced>();
        while (foreignKeyIt.hasNext())
        {
            ObjectKey currentKey = foreignKeyIt.next();
            foreignKeyListChunk.add(currentKey);
            if (foreignKeyListChunk.size() < fillerChunkSize 
                && foreignKeyIt.hasNext())
            {
                continue;
            }

            List<Referenced> referencedReferencedList
                 = ReferencedPeer.retrieveByPKs(
                     foreignKeyListChunk,
                     connection);

            for (Referenced referencedObject : referencedReferencedList)
            {
                referencedKeyToReferencedMap.put(
                        referencedObject.getPrimaryKey(),
                        referencedObject);
            }
            foreignKeyListChunk.clear();
        }

        List<Referenced> result = new ArrayList<Referenced>();
        for (MultiRefSameTable objectToFill : toFill)
        {
            ObjectKey foreignKey = objectToFill.getForeignKeyForReferencedRelatedByReference1();
            if (foreignKey == null || foreignKey.getValue() == null)
            {
                continue;
            }
            Referenced referenced = referencedKeyToReferencedMap.get(
                    foreignKey);
            if (referenced == null)
            {
                throw new NoRowsException("No result found for key " 
                    + foreignKey + " in table referenced");
            }
            // copy the referenced object for the case that more than one
            // toFill object references the same object; in this case
            // every toFillObject should have its own instance of the referenced
            // object to remain consistent with the other Torque reference
            // treatment
            Referenced referencedCopy = referenced.copy(false);
            // copy does not set primary key
            referencedCopy.setPrimaryKey(referenced.getPrimaryKey());
            objectToFill.setReferencedRelatedByReference1(referencedCopy);
            result.add(referencedCopy);
        }
        return result;
    }
    /** 
     * Fills the referenced Referenced objects in the aReferencedRelatedByReference2 fields
     * of the objects in <code>toFill</code> by reading the database.
     *
     * @param toFill Contains the objects to fill, not null,
     *        may not contain null.
     *
     * @return all read Referenced objects.
     *
     * @throws TorqueException if an error querying the database occurs.
     * @throws NullPointerException if toFill is null or contains null elements.
     */
    public List<Referenced> fillReferencedRelatedByReference2s(
            Collection<MultiRefSameTable> toFill) 
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    MultiRefSameTablePeer.DATABASE_NAME);
            List<Referenced> result = fillReferencedRelatedByReference2s(toFill, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /** 
     * Fills the referenced Referenced objects in the aReferencedRelatedByReference2 fields
     * of the objects in <code>toFill</code> by reading the database.
     *
     * @param toFill Contains the objects to fill, not null,
     *        may not contain null.
     * @param connection the database connection to use, not null.
     *
     * @return all read Referenced objects.
     *
     * @throws TorqueException if an error querying the database occurs.
     * @throws NullPointerException if toFill is null or contains null elements.
     */
    public List<Referenced> fillReferencedRelatedByReference2s(
            Collection<MultiRefSameTable> toFill,
            Connection connection) 
        throws TorqueException
    {
        Set<ObjectKey> foreignKeySet = new HashSet<ObjectKey>();
        for (MultiRefSameTable currentMultiRefSameTable : toFill)
        {
            ObjectKey foreignKey = currentMultiRefSameTable.getForeignKeyForReferencedRelatedByReference2();
            if (foreignKey != null)
            {
                foreignKeySet.add(foreignKey);
            }
        }

        int fillerChunkSize = getFillerChunkSize();
        Iterator<ObjectKey> foreignKeyIt = foreignKeySet.iterator();
        List<ObjectKey> foreignKeyListChunk
                = new ArrayList<ObjectKey>(fillerChunkSize);;
        Map<ObjectKey, Referenced> referencedKeyToReferencedMap 
                = new HashMap<ObjectKey, Referenced>();
        while (foreignKeyIt.hasNext())
        {
            ObjectKey currentKey = foreignKeyIt.next();
            foreignKeyListChunk.add(currentKey);
            if (foreignKeyListChunk.size() < fillerChunkSize 
                && foreignKeyIt.hasNext())
            {
                continue;
            }

            List<Referenced> referencedReferencedList
                 = ReferencedPeer.retrieveByPKs(
                     foreignKeyListChunk,
                     connection);

            for (Referenced referencedObject : referencedReferencedList)
            {
                referencedKeyToReferencedMap.put(
                        referencedObject.getPrimaryKey(),
                        referencedObject);
            }
            foreignKeyListChunk.clear();
        }

        List<Referenced> result = new ArrayList<Referenced>();
        for (MultiRefSameTable objectToFill : toFill)
        {
            ObjectKey foreignKey = objectToFill.getForeignKeyForReferencedRelatedByReference2();
            if (foreignKey == null || foreignKey.getValue() == null)
            {
                continue;
            }
            Referenced referenced = referencedKeyToReferencedMap.get(
                    foreignKey);
            if (referenced == null)
            {
                throw new NoRowsException("No result found for key " 
                    + foreignKey + " in table referenced");
            }
            // copy the referenced object for the case that more than one
            // toFill object references the same object; in this case
            // every toFillObject should have its own instance of the referenced
            // object to remain consistent with the other Torque reference
            // treatment
            Referenced referencedCopy = referenced.copy(false);
            // copy does not set primary key
            referencedCopy.setPrimaryKey(referenced.getPrimaryKey());
            objectToFill.setReferencedRelatedByReference2(referencedCopy);
            result.add(referencedCopy);
        }
        return result;
    }
    /** 
     * Fills the referenced Referenced objects in the aReferencedRelatedByReference3 fields
     * of the objects in <code>toFill</code> by reading the database.
     *
     * @param toFill Contains the objects to fill, not null,
     *        may not contain null.
     *
     * @return all read Referenced objects.
     *
     * @throws TorqueException if an error querying the database occurs.
     * @throws NullPointerException if toFill is null or contains null elements.
     */
    public List<Referenced> fillReferencedRelatedByReference3s(
            Collection<MultiRefSameTable> toFill) 
        throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    MultiRefSameTablePeer.DATABASE_NAME);
            List<Referenced> result = fillReferencedRelatedByReference3s(toFill, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /** 
     * Fills the referenced Referenced objects in the aReferencedRelatedByReference3 fields
     * of the objects in <code>toFill</code> by reading the database.
     *
     * @param toFill Contains the objects to fill, not null,
     *        may not contain null.
     * @param connection the database connection to use, not null.
     *
     * @return all read Referenced objects.
     *
     * @throws TorqueException if an error querying the database occurs.
     * @throws NullPointerException if toFill is null or contains null elements.
     */
    public List<Referenced> fillReferencedRelatedByReference3s(
            Collection<MultiRefSameTable> toFill,
            Connection connection) 
        throws TorqueException
    {
        Set<ObjectKey> foreignKeySet = new HashSet<ObjectKey>();
        for (MultiRefSameTable currentMultiRefSameTable : toFill)
        {
            ObjectKey foreignKey = currentMultiRefSameTable.getForeignKeyForReferencedRelatedByReference3();
            if (foreignKey != null)
            {
                foreignKeySet.add(foreignKey);
            }
        }

        int fillerChunkSize = getFillerChunkSize();
        Iterator<ObjectKey> foreignKeyIt = foreignKeySet.iterator();
        List<ObjectKey> foreignKeyListChunk
                = new ArrayList<ObjectKey>(fillerChunkSize);;
        Map<ObjectKey, Referenced> referencedKeyToReferencedMap 
                = new HashMap<ObjectKey, Referenced>();
        while (foreignKeyIt.hasNext())
        {
            ObjectKey currentKey = foreignKeyIt.next();
            foreignKeyListChunk.add(currentKey);
            if (foreignKeyListChunk.size() < fillerChunkSize 
                && foreignKeyIt.hasNext())
            {
                continue;
            }

            List<Referenced> referencedReferencedList
                 = ReferencedPeer.retrieveByPKs(
                     foreignKeyListChunk,
                     connection);

            for (Referenced referencedObject : referencedReferencedList)
            {
                referencedKeyToReferencedMap.put(
                        referencedObject.getPrimaryKey(),
                        referencedObject);
            }
            foreignKeyListChunk.clear();
        }

        List<Referenced> result = new ArrayList<Referenced>();
        for (MultiRefSameTable objectToFill : toFill)
        {
            ObjectKey foreignKey = objectToFill.getForeignKeyForReferencedRelatedByReference3();
            if (foreignKey == null || foreignKey.getValue() == null)
            {
                continue;
            }
            Referenced referenced = referencedKeyToReferencedMap.get(
                    foreignKey);
            if (referenced == null)
            {
                throw new NoRowsException("No result found for key " 
                    + foreignKey + " in table referenced");
            }
            // copy the referenced object for the case that more than one
            // toFill object references the same object; in this case
            // every toFillObject should have its own instance of the referenced
            // object to remain consistent with the other Torque reference
            // treatment
            Referenced referencedCopy = referenced.copy(false);
            // copy does not set primary key
            referencedCopy.setPrimaryKey(referenced.getPrimaryKey());
            objectToFill.setReferencedRelatedByReference3(referencedCopy);
            result.add(referencedCopy);
        }
        return result;
    }


    /** 
     * Returns the chunk size for the filler methods.
     *
     * @return the chunk size for the filler methods.
     */
    protected int getFillerChunkSize()
    {
        return 999;
    }


}
