package org.apache.torque.test;

import java.sql.Blob;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.Types;
import java.io.IOException;
import java.io.InputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.Reader;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.torque.NoRowsException;
import org.apache.torque.TooManyRowsException;
import org.apache.torque.Torque;
import org.apache.torque.TorqueException;
import org.apache.torque.TorqueRuntimeException;
import org.apache.torque.criteria.Criteria;
import org.apache.torque.criteria.Criterion;
import org.apache.torque.om.mapper.RecordMapper;
import org.apache.torque.om.mapper.CompositeMapper;
import org.apache.torque.om.DateKey;
import org.apache.torque.om.NumberKey;
import org.apache.torque.om.StringKey;
import org.apache.torque.om.ObjectKey;
import org.apache.torque.om.SimpleKey;
import org.apache.torque.om.ComboKey;
import org.apache.torque.map.TableMap;
import org.apache.torque.util.BasePeer;
import org.apache.torque.util.Transaction;
import org.apache.torque.util.ColumnValues;
import org.apache.torque.util.JdbcTypedValue;



/**
 * used to test the handling for all supported types
 *
 * The skeleton for this class was autogenerated by Torque on:
 *
 * [Thu Sep 13 04:35:46 CEST 2012]
 *
 * You should not use this class directly.  It should not even be
 * extended; all references should be to TypesObjectPeer
 */

public abstract class BaseTypesObjectPeerImpl
    extends org.apache.torque.util.BasePeerImpl<TypesObject>
{
    /** The class log. */
    private static Log log = LogFactory.getLog(BaseTypesObjectPeerImpl.class);

    /** Serial version */
    private static final long serialVersionUID = 1347503746738L;



    /**
     * Constructor.
     * The recordMapper, tableMap and databaseName fields are correctly
     * initialized.
     */
    public BaseTypesObjectPeerImpl()
    {
        this(new TypesObjectRecordMapper(),
            TypesObjectPeer.TABLE,
            TypesObjectPeer.DATABASE_NAME);
    }

    /**
     * Constructor providing the objects to be injected as parameters.
     *
     * @param recordMapper a record mapper to map JDBC result sets to objects
     * @param tableMap the default table map
     * @param databaseName the name of the database
     */
    public BaseTypesObjectPeerImpl(
            RecordMapper<TypesObject> recordMapper, 
            TableMap tableMap,
            String databaseName)
    {
        super(recordMapper, tableMap, databaseName);
    }


    /**
     * Selects TypesObject objects from the database which have
     * the same content as the passed object.
     *
     * @return The list of selected objects, not null.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<TypesObject> doSelect(TypesObject obj)
            throws TorqueException
    {
        return doSelect(buildSelectCriteria(obj));
    }

    /**
     * Selects at most one TypesObject object from the database
     * which has the same content as the passed object.
     *
     * @return the selected Object, or null if no object was selected.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public TypesObject doSelectSingleRecord(
                TypesObject obj)
            throws TorqueException
    {
        List<TypesObject> typesObjectList = doSelect(obj);
        TypesObject typesObject = null;
        if (typesObjectList.size() > 1)
        {
            throw new TooManyRowsException("Object " + obj 
                + " matched more than one record");
        }
        if (!typesObjectList.isEmpty())
        {
            typesObject = typesObjectList.get(0);
        }
        return typesObject;
    }

    /**
     * Returns a new instance of the Data object class
     */
    public TypesObject getDbObjectInstance()
    {
        return new TypesObject();
    }


    /**
     * Method to do inserts.
     *
     * @param columnValues the values to insert.
     *
     * @return always null (because the table does not have a primary key).
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public ObjectKey doInsert(ColumnValues columnValues) throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    TypesObjectPeer.DATABASE_NAME);
            ObjectKey result = doInsert(columnValues, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Method to do inserts.  This method is to be used during a transaction,
     * otherwise use the doInsert(Criteria) method.
     *
     * @param columnValues the values to insert.
     * @param con the connection to use, not null.
     *
     * @return always null (because the table does not have a primary key).
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public ObjectKey doInsert(ColumnValues columnValues, Connection con)
        throws TorqueException
    {
        correctBooleans(columnValues);
        return super.doInsert(columnValues, con);
    }

    /**
     * Method to do inserts
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public void doInsert(TypesObject obj) throws TorqueException
    {
        doInsert(buildColumnValues(obj));
        obj.setNew(false);
        obj.setModified(false);
    }

    /**
     * Method to do inserts.  This method is to be used during a transaction,
     * otherwise use the doInsert(TypesObject) method.  It will take
     * care of the connection details internally.
     *
     * @param obj the data object to insert into the database.
     * @param con the connection to use
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public void doInsert(TypesObject obj, Connection con)
        throws TorqueException
    {
        doInsert(buildColumnValues(obj), con);
        obj.setNew(false);
        obj.setModified(false);
    }

    /**
     * Method to do updates.
     *
     * @param columnValues the values to update plus the primary key
     *        identifying the row to update.
     *
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(ColumnValues columnValues) throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    TypesObjectPeer.DATABASE_NAME);
            int result = doUpdate(columnValues, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Method to do updates.  This method is to be used during a transaction,
     * otherwise use the doUpdate(Criteria) method.
     *
     * @param columnValues the values to update plus the primary key
     *        identifying the row to update.
     * @param con the connection to use, not null.
     *
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(ColumnValues columnValues, Connection con)
        throws TorqueException
    {
        Criteria selectCriteria 
                = new Criteria(TypesObjectPeer.DATABASE_NAME);
        correctBooleans(columnValues);
























        return doUpdate(selectCriteria, columnValues, con);
    }

    /**
     * Updates an TypesObject in the database.
     * The primary key is used to identify the object to update.
     *
     * @param obj the data object to update in the database.
     *
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(TypesObject obj) throws TorqueException
    {
        throw new TorqueException(
                "doUpdate does not work for objects without primary key");
    }

    /**
     * Updates a TypesObject in the database.
     * The primary key is used to identify the object to update.
     * This method is to be used during a transaction,
     * otherwise the doUpdate(TypesObject) method can be used.
     *
     * @param obj the data object to update in the database.
     * @param con the connection to use, not null.
     
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(TypesObject obj, Connection con)
        throws TorqueException
    {
        throw new TorqueException(
                "doUpdate does not work for objects without primary key");
    }

    /**
     * Deletes a data object, i.e. a row in a table, in the database.
     *
     * @param obj the data object to delete in the database, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(TypesObject obj) throws TorqueException
    {
        int result = doDelete(buildSelectCriteria(obj));
        obj.setDeleted(true);
        return result;
    }

    /**
     * Deletes a data object, i.e. a row in a table, in the database.
     * This method is to be used during a transaction, otherwise use the
     * doDelete(TypesObject) method.
     *
     * @param obj the data object to delete in the database, not null.
     * @param con the connection to use, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(TypesObject obj, Connection con)
        throws TorqueException
    {
        int result = doDelete(buildSelectCriteria(obj), con);
        obj.setDeleted(true);
        return result;
    }


    /** 
     * Build a Criteria object from the data object for this peer.
     *
     * @param obj the object to build the criteria from, not null.
     */
    public Criteria buildCriteria(TypesObject obj)
    {
        Criteria criteria = new Criteria(TypesObjectPeer.DATABASE_NAME);
        criteria.and(TypesObjectPeer.O_BIT, obj.getOBit());
        criteria.and(TypesObjectPeer.O_TINYINT, obj.getOTinyint());
        criteria.and(TypesObjectPeer.O_SMALLINT, obj.getOSmallint());
        criteria.and(TypesObjectPeer.O_BIGINT, obj.getOBigint());
        criteria.and(TypesObjectPeer.O_FLOAT, obj.getOFloat());
        criteria.and(TypesObjectPeer.O_REAL, obj.getOReal());
        criteria.and(TypesObjectPeer.O_NUMERIC, obj.getONumeric());
        criteria.and(TypesObjectPeer.O_DECIMAL, obj.getODecimal());
        criteria.and(TypesObjectPeer.O_CHAR, obj.getOChar());
        criteria.and(TypesObjectPeer.O_VARCHAR, obj.getOVarchar());
        criteria.and(TypesObjectPeer.O_LONGVARCHAR, obj.getOLongvarchar());
        criteria.and(TypesObjectPeer.O_DATE, obj.getODate());
        criteria.and(TypesObjectPeer.O_TIME, obj.getOTime());
        criteria.and(TypesObjectPeer.O_INTEGER, obj.getOInteger());
        criteria.and(TypesObjectPeer.O_TIMESTAMP, obj.getOTimestamp());
        criteria.and(TypesObjectPeer.O_BINARY, obj.getOBinary());
        criteria.and(TypesObjectPeer.O_VARBINARY, obj.getOVarbinary());
        criteria.and(TypesObjectPeer.O_LONGVARBINARY, obj.getOLongvarbinary());
        criteria.and(TypesObjectPeer.O_BLOB, obj.getOBlob());
        criteria.and(TypesObjectPeer.O_CLOB, obj.getOClob());
        criteria.and(TypesObjectPeer.O_BOOLEANINT, obj.getOBooleanint());
        criteria.and(TypesObjectPeer.O_BOOLEANCHAR, obj.getOBooleanchar());
        criteria.and(TypesObjectPeer.O_DOUBLE, obj.getODouble());
        return criteria;
    }

    /** 
     * Build a Criteria object from the data object for this peer,
     * skipping all binary columns.
     *
     * @param obj the object to build the criteria from, not null.
     */
    public Criteria buildSelectCriteria(TypesObject obj)
    {
        Criteria criteria = new Criteria(TypesObjectPeer.DATABASE_NAME);
        criteria.and(TypesObjectPeer.O_BIT, obj.getOBit());
        criteria.and(TypesObjectPeer.O_TINYINT, obj.getOTinyint());
        criteria.and(TypesObjectPeer.O_SMALLINT, obj.getOSmallint());
        criteria.and(TypesObjectPeer.O_BIGINT, obj.getOBigint());
        criteria.and(TypesObjectPeer.O_FLOAT, obj.getOFloat());
        criteria.and(TypesObjectPeer.O_REAL, obj.getOReal());
        criteria.and(TypesObjectPeer.O_NUMERIC, obj.getONumeric());
        criteria.and(TypesObjectPeer.O_DECIMAL, obj.getODecimal());
        criteria.and(TypesObjectPeer.O_CHAR, obj.getOChar());
        criteria.and(TypesObjectPeer.O_VARCHAR, obj.getOVarchar());
        criteria.and(TypesObjectPeer.O_LONGVARCHAR, obj.getOLongvarchar());
        criteria.and(TypesObjectPeer.O_DATE, obj.getODate());
        criteria.and(TypesObjectPeer.O_TIME, obj.getOTime());
        criteria.and(TypesObjectPeer.O_INTEGER, obj.getOInteger());
        criteria.and(TypesObjectPeer.O_TIMESTAMP, obj.getOTimestamp());
        criteria.and(TypesObjectPeer.O_CLOB, obj.getOClob());
        criteria.and(TypesObjectPeer.O_BOOLEANINT, obj.getOBooleanint());
        criteria.and(TypesObjectPeer.O_BOOLEANCHAR, obj.getOBooleanchar());
        criteria.and(TypesObjectPeer.O_DOUBLE, obj.getODouble());
        return criteria;
    }

    /** 
     * Returns the contents of the object as ColumnValues object.
     * Primary key columns which are generated on insertion are not
     * added to the returned object if they still have their initial
     * value. Also, columns which have the useDatabaseDefaultValue
     * flag set to true are also not added to the returned object
     * if they still have their initial value.
     *
     * @throws TorqueException if the table map cannot be retrieved
     *         (should not happen).
     */
    public ColumnValues buildColumnValues(TypesObject typesObject)
            throws TorqueException
    {
        ColumnValues columnValues = new ColumnValues();
        columnValues.put(
                TypesObjectPeer.O_BIT,
                new JdbcTypedValue(
                    typesObject.getOBit(),
                    -7));
        columnValues.put(
                TypesObjectPeer.O_TINYINT,
                new JdbcTypedValue(
                    typesObject.getOTinyint(),
                    -6));
        columnValues.put(
                TypesObjectPeer.O_SMALLINT,
                new JdbcTypedValue(
                    typesObject.getOSmallint(),
                    5));
        columnValues.put(
                TypesObjectPeer.O_BIGINT,
                new JdbcTypedValue(
                    typesObject.getOBigint(),
                    -5));
        columnValues.put(
                TypesObjectPeer.O_FLOAT,
                new JdbcTypedValue(
                    typesObject.getOFloat(),
                    6));
        columnValues.put(
                TypesObjectPeer.O_REAL,
                new JdbcTypedValue(
                    typesObject.getOReal(),
                    7));
        columnValues.put(
                TypesObjectPeer.O_NUMERIC,
                new JdbcTypedValue(
                    typesObject.getONumeric(),
                    2));
        columnValues.put(
                TypesObjectPeer.O_DECIMAL,
                new JdbcTypedValue(
                    typesObject.getODecimal(),
                    3));
        columnValues.put(
                TypesObjectPeer.O_CHAR,
                new JdbcTypedValue(
                    typesObject.getOChar(),
                    1));
        columnValues.put(
                TypesObjectPeer.O_VARCHAR,
                new JdbcTypedValue(
                    typesObject.getOVarchar(),
                    12));
        columnValues.put(
                TypesObjectPeer.O_LONGVARCHAR,
                new JdbcTypedValue(
                    typesObject.getOLongvarchar(),
                    -1));
        java.util.Date oDate = typesObject.getODate();
        if (oDate != null)
        {
            oDate = new java.sql.Date(oDate.getTime());
        }
        columnValues.put(
                TypesObjectPeer.O_DATE,
                new JdbcTypedValue(oDate, Types.DATE));
        java.util.Date oTime = typesObject.getOTime();
        if (oTime != null)
        {
            oTime = new java.sql.Time(oTime.getTime());
        }
        columnValues.put(
                TypesObjectPeer.O_TIME,
                new JdbcTypedValue(oTime, Types.TIME));
        columnValues.put(
                TypesObjectPeer.O_INTEGER,
                new JdbcTypedValue(
                    typesObject.getOInteger(),
                    4));
        java.util.Date oTimestamp = typesObject.getOTimestamp();
        if (oTimestamp != null)
        {
            oTimestamp = new java.sql.Timestamp(oTimestamp.getTime());
        }
        columnValues.put(
                TypesObjectPeer.O_TIMESTAMP,
                new JdbcTypedValue(oTimestamp, Types.TIMESTAMP));
        columnValues.put(
                TypesObjectPeer.O_BINARY,
                new JdbcTypedValue(
                    typesObject.getOBinary(),
                    -2));
        columnValues.put(
                TypesObjectPeer.O_VARBINARY,
                new JdbcTypedValue(
                    typesObject.getOVarbinary(),
                    -3));
        columnValues.put(
                TypesObjectPeer.O_LONGVARBINARY,
                new JdbcTypedValue(
                    typesObject.getOLongvarbinary(),
                    -4));
        columnValues.put(
                TypesObjectPeer.O_BLOB,
                new JdbcTypedValue(
                    typesObject.getOBlob(),
                    2004));
        columnValues.put(
                TypesObjectPeer.O_CLOB,
                new JdbcTypedValue(
                    typesObject.getOClob(),
                    2005));
        Integer oBooleanint = null;
        if (Boolean.TRUE.equals(typesObject.getOBooleanint()))
        {
            oBooleanint =  Integer.valueOf(1);
        }
        else if (Boolean.FALSE.equals(typesObject.getOBooleanint()))
        {
            oBooleanint =  Integer.valueOf(0);
        }
        columnValues.put(
                TypesObjectPeer.O_BOOLEANINT,
                new JdbcTypedValue(oBooleanint, Types.INTEGER));
        String oBooleanchar = null;
        if (Boolean.TRUE.equals(typesObject.getOBooleanchar()))
        {
            oBooleanchar =  "Y";
        }
        else if (Boolean.FALSE.equals(typesObject.getOBooleanchar()))
        {
            oBooleanchar =  "N";
        }
        columnValues.put(
                TypesObjectPeer.O_BOOLEANCHAR,
                new JdbcTypedValue(oBooleanchar, Types.CHAR));
        columnValues.put(
                TypesObjectPeer.O_DOUBLE,
                new JdbcTypedValue(
                    typesObject.getODouble(),
                    8));
        return columnValues;
    }







    /** 
     * Returns the chunk size for the filler methods.
     *
     * @return the chunk size for the filler methods.
     */
    protected int getFillerChunkSize()
    {
        return 999;
    }


}
