package org.apache.torque.test;

import java.sql.Blob;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.Types;
import java.io.IOException;
import java.io.InputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.Reader;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.torque.NoRowsException;
import org.apache.torque.TooManyRowsException;
import org.apache.torque.Torque;
import org.apache.torque.TorqueException;
import org.apache.torque.TorqueRuntimeException;
import org.apache.torque.criteria.Criteria;
import org.apache.torque.criteria.Criterion;
import org.apache.torque.om.mapper.RecordMapper;
import org.apache.torque.om.mapper.CompositeMapper;
import org.apache.torque.om.DateKey;
import org.apache.torque.om.NumberKey;
import org.apache.torque.om.StringKey;
import org.apache.torque.om.ObjectKey;
import org.apache.torque.om.SimpleKey;
import org.apache.torque.om.ComboKey;
import org.apache.torque.map.TableMap;
import org.apache.torque.util.BasePeer;
import org.apache.torque.util.Transaction;
import org.apache.torque.util.ColumnValues;
import org.apache.torque.util.JdbcTypedValue;



/**
 * used to test the handling for all supported types
 *
 * The skeleton for this class was autogenerated by Torque on:
 *
 * [Thu Sep 13 04:35:46 CEST 2012]
 *
 * You should not use this class directly.  It should not even be
 * extended; all references should be to TypesPrimitivePeer
 */

public abstract class BaseTypesPrimitivePeerImpl
    extends org.apache.torque.util.BasePeerImpl<TypesPrimitive>
{
    /** The class log. */
    private static Log log = LogFactory.getLog(BaseTypesPrimitivePeerImpl.class);

    /** Serial version */
    private static final long serialVersionUID = 1347503746753L;



    /**
     * Constructor.
     * The recordMapper, tableMap and databaseName fields are correctly
     * initialized.
     */
    public BaseTypesPrimitivePeerImpl()
    {
        this(new TypesPrimitiveRecordMapper(),
            TypesPrimitivePeer.TABLE,
            TypesPrimitivePeer.DATABASE_NAME);
    }

    /**
     * Constructor providing the objects to be injected as parameters.
     *
     * @param recordMapper a record mapper to map JDBC result sets to objects
     * @param tableMap the default table map
     * @param databaseName the name of the database
     */
    public BaseTypesPrimitivePeerImpl(
            RecordMapper<TypesPrimitive> recordMapper, 
            TableMap tableMap,
            String databaseName)
    {
        super(recordMapper, tableMap, databaseName);
    }


    /**
     * Selects TypesPrimitive objects from the database which have
     * the same content as the passed object.
     *
     * @return The list of selected objects, not null.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public List<TypesPrimitive> doSelect(TypesPrimitive obj)
            throws TorqueException
    {
        return doSelect(buildSelectCriteria(obj));
    }

    /**
     * Selects at most one TypesPrimitive object from the database
     * which has the same content as the passed object.
     *
     * @return the selected Object, or null if no object was selected.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public TypesPrimitive doSelectSingleRecord(
                TypesPrimitive obj)
            throws TorqueException
    {
        List<TypesPrimitive> typesPrimitiveList = doSelect(obj);
        TypesPrimitive typesPrimitive = null;
        if (typesPrimitiveList.size() > 1)
        {
            throw new TooManyRowsException("Object " + obj 
                + " matched more than one record");
        }
        if (!typesPrimitiveList.isEmpty())
        {
            typesPrimitive = typesPrimitiveList.get(0);
        }
        return typesPrimitive;
    }

    /**
     * Returns a new instance of the Data object class
     */
    public TypesPrimitive getDbObjectInstance()
    {
        return new TypesPrimitive();
    }


    /**
     * Method to do inserts.
     *
     * @param columnValues the values to insert.
     *
     * @return always null (because the table does not have a primary key).
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public ObjectKey doInsert(ColumnValues columnValues) throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    TypesPrimitivePeer.DATABASE_NAME);
            ObjectKey result = doInsert(columnValues, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Method to do inserts.  This method is to be used during a transaction,
     * otherwise use the doInsert(Criteria) method.
     *
     * @param columnValues the values to insert.
     * @param con the connection to use, not null.
     *
     * @return always null (because the table does not have a primary key).
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public ObjectKey doInsert(ColumnValues columnValues, Connection con)
        throws TorqueException
    {
        correctBooleans(columnValues);
        return super.doInsert(columnValues, con);
    }

    /**
     * Method to do inserts
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public void doInsert(TypesPrimitive obj) throws TorqueException
    {
        doInsert(buildColumnValues(obj));
        obj.setNew(false);
        obj.setModified(false);
    }

    /**
     * Method to do inserts.  This method is to be used during a transaction,
     * otherwise use the doInsert(TypesPrimitive) method.  It will take
     * care of the connection details internally.
     *
     * @param obj the data object to insert into the database.
     * @param con the connection to use
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public void doInsert(TypesPrimitive obj, Connection con)
        throws TorqueException
    {
        doInsert(buildColumnValues(obj), con);
        obj.setNew(false);
        obj.setModified(false);
    }

    /**
     * Method to do updates.
     *
     * @param columnValues the values to update plus the primary key
     *        identifying the row to update.
     *
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(ColumnValues columnValues) throws TorqueException
    {
        Connection connection = null;
        try
        {
            connection = Transaction.begin(
                    TypesPrimitivePeer.DATABASE_NAME);
            int result = doUpdate(columnValues, connection);
            Transaction.commit(connection);
            connection = null;
            return result;
        }
        finally
        {
            if (connection != null)
            {
                Transaction.safeRollback(connection);
            }
        }
    }

    /**
     * Method to do updates.  This method is to be used during a transaction,
     * otherwise use the doUpdate(Criteria) method.
     *
     * @param columnValues the values to update plus the primary key
     *        identifying the row to update.
     * @param con the connection to use, not null.
     *
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(ColumnValues columnValues, Connection con)
        throws TorqueException
    {
        Criteria selectCriteria 
                = new Criteria(TypesPrimitivePeer.DATABASE_NAME);
        correctBooleans(columnValues);
























        return doUpdate(selectCriteria, columnValues, con);
    }

    /**
     * Updates an TypesPrimitive in the database.
     * The primary key is used to identify the object to update.
     *
     * @param obj the data object to update in the database.
     *
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(TypesPrimitive obj) throws TorqueException
    {
        throw new TorqueException(
                "doUpdate does not work for objects without primary key");
    }

    /**
     * Updates a TypesPrimitive in the database.
     * The primary key is used to identify the object to update.
     * This method is to be used during a transaction,
     * otherwise the doUpdate(TypesPrimitive) method can be used.
     *
     * @param obj the data object to update in the database.
     * @param con the connection to use, not null.
     
     * @return the number of affected rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doUpdate(TypesPrimitive obj, Connection con)
        throws TorqueException
    {
        throw new TorqueException(
                "doUpdate does not work for objects without primary key");
    }

    /**
     * Deletes a data object, i.e. a row in a table, in the database.
     *
     * @param obj the data object to delete in the database, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(TypesPrimitive obj) throws TorqueException
    {
        int result = doDelete(buildSelectCriteria(obj));
        obj.setDeleted(true);
        return result;
    }

    /**
     * Deletes a data object, i.e. a row in a table, in the database.
     * This method is to be used during a transaction, otherwise use the
     * doDelete(TypesPrimitive) method.
     *
     * @param obj the data object to delete in the database, not null.
     * @param con the connection to use, not null.
     *
     * @return the number of deleted rows.
     *
     * @throws TorqueException Any exceptions caught during processing will be
     *         rethrown wrapped into a TorqueException.
     */
    public int doDelete(TypesPrimitive obj, Connection con)
        throws TorqueException
    {
        int result = doDelete(buildSelectCriteria(obj), con);
        obj.setDeleted(true);
        return result;
    }


    /** 
     * Build a Criteria object from the data object for this peer.
     *
     * @param obj the object to build the criteria from, not null.
     */
    public Criteria buildCriteria(TypesPrimitive obj)
    {
        Criteria criteria = new Criteria(TypesPrimitivePeer.DATABASE_NAME);
        criteria.and(TypesPrimitivePeer.P_BIT, obj.getPBit());
        criteria.and(TypesPrimitivePeer.P_TINYINT, obj.getPTinyint());
        criteria.and(TypesPrimitivePeer.P_SMALLINT, obj.getPSmallint());
        criteria.and(TypesPrimitivePeer.P_BIGINT, obj.getPBigint());
        criteria.and(TypesPrimitivePeer.P_FLOAT, obj.getPFloat());
        criteria.and(TypesPrimitivePeer.P_REAL, obj.getPReal());
        criteria.and(TypesPrimitivePeer.P_NUMERIC, obj.getPNumeric());
        criteria.and(TypesPrimitivePeer.P_DECIMAL, obj.getPDecimal());
        criteria.and(TypesPrimitivePeer.P_CHAR, obj.getPChar());
        criteria.and(TypesPrimitivePeer.P_VARCHAR, obj.getPVarchar());
        criteria.and(TypesPrimitivePeer.P_LONGVARCHAR, obj.getPLongvarchar());
        criteria.and(TypesPrimitivePeer.P_DATE, obj.getPDate());
        criteria.and(TypesPrimitivePeer.P_TIME, obj.getPTime());
        criteria.and(TypesPrimitivePeer.P_INTEGER, obj.getPInteger());
        criteria.and(TypesPrimitivePeer.P_TIMESTAMP, obj.getPTimestamp());
        criteria.and(TypesPrimitivePeer.P_BINARY, obj.getPBinary());
        criteria.and(TypesPrimitivePeer.P_VARBINARY, obj.getPVarbinary());
        criteria.and(TypesPrimitivePeer.P_LONGVARBINARY, obj.getPLongvarbinary());
        criteria.and(TypesPrimitivePeer.P_BLOB, obj.getPBlob());
        criteria.and(TypesPrimitivePeer.P_CLOB, obj.getPClob());
        criteria.and(TypesPrimitivePeer.P_BOOLEANINT, obj.getPBooleanint());
        criteria.and(TypesPrimitivePeer.P_BOOLEANCHAR, obj.getPBooleanchar());
        criteria.and(TypesPrimitivePeer.P_DOUBLE, obj.getPDouble());
        return criteria;
    }

    /** 
     * Build a Criteria object from the data object for this peer,
     * skipping all binary columns.
     *
     * @param obj the object to build the criteria from, not null.
     */
    public Criteria buildSelectCriteria(TypesPrimitive obj)
    {
        Criteria criteria = new Criteria(TypesPrimitivePeer.DATABASE_NAME);
        criteria.and(TypesPrimitivePeer.P_BIT, obj.getPBit());
        criteria.and(TypesPrimitivePeer.P_TINYINT, obj.getPTinyint());
        criteria.and(TypesPrimitivePeer.P_SMALLINT, obj.getPSmallint());
        criteria.and(TypesPrimitivePeer.P_BIGINT, obj.getPBigint());
        criteria.and(TypesPrimitivePeer.P_FLOAT, obj.getPFloat());
        criteria.and(TypesPrimitivePeer.P_REAL, obj.getPReal());
        criteria.and(TypesPrimitivePeer.P_NUMERIC, obj.getPNumeric());
        criteria.and(TypesPrimitivePeer.P_DECIMAL, obj.getPDecimal());
        criteria.and(TypesPrimitivePeer.P_CHAR, obj.getPChar());
        criteria.and(TypesPrimitivePeer.P_VARCHAR, obj.getPVarchar());
        criteria.and(TypesPrimitivePeer.P_LONGVARCHAR, obj.getPLongvarchar());
        criteria.and(TypesPrimitivePeer.P_DATE, obj.getPDate());
        criteria.and(TypesPrimitivePeer.P_TIME, obj.getPTime());
        criteria.and(TypesPrimitivePeer.P_INTEGER, obj.getPInteger());
        criteria.and(TypesPrimitivePeer.P_TIMESTAMP, obj.getPTimestamp());
        criteria.and(TypesPrimitivePeer.P_CLOB, obj.getPClob());
        criteria.and(TypesPrimitivePeer.P_BOOLEANINT, obj.getPBooleanint());
        criteria.and(TypesPrimitivePeer.P_BOOLEANCHAR, obj.getPBooleanchar());
        criteria.and(TypesPrimitivePeer.P_DOUBLE, obj.getPDouble());
        return criteria;
    }

    /** 
     * Returns the contents of the object as ColumnValues object.
     * Primary key columns which are generated on insertion are not
     * added to the returned object if they still have their initial
     * value. Also, columns which have the useDatabaseDefaultValue
     * flag set to true are also not added to the returned object
     * if they still have their initial value.
     *
     * @throws TorqueException if the table map cannot be retrieved
     *         (should not happen).
     */
    public ColumnValues buildColumnValues(TypesPrimitive typesPrimitive)
            throws TorqueException
    {
        ColumnValues columnValues = new ColumnValues();
        columnValues.put(
                TypesPrimitivePeer.P_BIT,
                new JdbcTypedValue(
                    typesPrimitive.getPBit(),
                    -7));
        columnValues.put(
                TypesPrimitivePeer.P_TINYINT,
                new JdbcTypedValue(
                    typesPrimitive.getPTinyint(),
                    -6));
        columnValues.put(
                TypesPrimitivePeer.P_SMALLINT,
                new JdbcTypedValue(
                    typesPrimitive.getPSmallint(),
                    5));
        columnValues.put(
                TypesPrimitivePeer.P_BIGINT,
                new JdbcTypedValue(
                    typesPrimitive.getPBigint(),
                    -5));
        columnValues.put(
                TypesPrimitivePeer.P_FLOAT,
                new JdbcTypedValue(
                    typesPrimitive.getPFloat(),
                    6));
        columnValues.put(
                TypesPrimitivePeer.P_REAL,
                new JdbcTypedValue(
                    typesPrimitive.getPReal(),
                    7));
        columnValues.put(
                TypesPrimitivePeer.P_NUMERIC,
                new JdbcTypedValue(
                    typesPrimitive.getPNumeric(),
                    2));
        columnValues.put(
                TypesPrimitivePeer.P_DECIMAL,
                new JdbcTypedValue(
                    typesPrimitive.getPDecimal(),
                    3));
        columnValues.put(
                TypesPrimitivePeer.P_CHAR,
                new JdbcTypedValue(
                    typesPrimitive.getPChar(),
                    1));
        columnValues.put(
                TypesPrimitivePeer.P_VARCHAR,
                new JdbcTypedValue(
                    typesPrimitive.getPVarchar(),
                    12));
        columnValues.put(
                TypesPrimitivePeer.P_LONGVARCHAR,
                new JdbcTypedValue(
                    typesPrimitive.getPLongvarchar(),
                    -1));
        java.util.Date pDate = typesPrimitive.getPDate();
        if (pDate != null)
        {
            pDate = new java.sql.Date(pDate.getTime());
        }
        columnValues.put(
                TypesPrimitivePeer.P_DATE,
                new JdbcTypedValue(pDate, Types.DATE));
        java.util.Date pTime = typesPrimitive.getPTime();
        if (pTime != null)
        {
            pTime = new java.sql.Time(pTime.getTime());
        }
        columnValues.put(
                TypesPrimitivePeer.P_TIME,
                new JdbcTypedValue(pTime, Types.TIME));
        columnValues.put(
                TypesPrimitivePeer.P_INTEGER,
                new JdbcTypedValue(
                    typesPrimitive.getPInteger(),
                    4));
        java.util.Date pTimestamp = typesPrimitive.getPTimestamp();
        if (pTimestamp != null)
        {
            pTimestamp = new java.sql.Timestamp(pTimestamp.getTime());
        }
        columnValues.put(
                TypesPrimitivePeer.P_TIMESTAMP,
                new JdbcTypedValue(pTimestamp, Types.TIMESTAMP));
        columnValues.put(
                TypesPrimitivePeer.P_BINARY,
                new JdbcTypedValue(
                    typesPrimitive.getPBinary(),
                    -2));
        columnValues.put(
                TypesPrimitivePeer.P_VARBINARY,
                new JdbcTypedValue(
                    typesPrimitive.getPVarbinary(),
                    -3));
        columnValues.put(
                TypesPrimitivePeer.P_LONGVARBINARY,
                new JdbcTypedValue(
                    typesPrimitive.getPLongvarbinary(),
                    -4));
        columnValues.put(
                TypesPrimitivePeer.P_BLOB,
                new JdbcTypedValue(
                    typesPrimitive.getPBlob(),
                    2004));
        columnValues.put(
                TypesPrimitivePeer.P_CLOB,
                new JdbcTypedValue(
                    typesPrimitive.getPClob(),
                    2005));
        Integer pBooleanint = Boolean.TRUE.equals(typesPrimitive.getPBooleanint())
                                    ? Integer.valueOf(1)
                                    : Integer.valueOf(0);
        columnValues.put(
                TypesPrimitivePeer.P_BOOLEANINT,
                new JdbcTypedValue(pBooleanint, Types.INTEGER));
        String pBooleanchar = Boolean.TRUE.equals(typesPrimitive.getPBooleanchar())
                                    ? "Y"
                                    : "N";
        columnValues.put(
                TypesPrimitivePeer.P_BOOLEANCHAR,
                new JdbcTypedValue(pBooleanchar, Types.CHAR));
        columnValues.put(
                TypesPrimitivePeer.P_DOUBLE,
                new JdbcTypedValue(
                    typesPrimitive.getPDouble(),
                    8));
        return columnValues;
    }







    /** 
     * Returns the chunk size for the filler methods.
     *
     * @return the chunk size for the filler methods.
     */
    protected int getFillerChunkSize()
    {
        return 999;
    }


}
